
/*------------------------------------------------------------------------------
INCLUDE FILES
------------------------------------------------------------------------------*/

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/slab.h>
#include <linux/jiffies.h>
#include <asm/div64.h>

#include "drxk_driver.h"
#include "drxk.h"
#include "drxk_mc.h"

#include "mt2063.h"

static unsigned int verbose = 0;//1;
module_param(verbose, int, 0644);


//context of BSP_HOST.c	<Henry>	==========================================
//#################################################################
//=================================================================
/**
* \brief  Function to initialise the host bsp module.
* \return DRXStatus_t.
* \retval DRX_STS_OK:    Successful initialization.
* \retval DRX_STS_ERROR: Something went wrong.
*
*/
enum DRXStatus_t DRXBSP_HST_Init( void )
{
   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief  Function to terminate the host bsp module.
* \return DRXStatus_t.
* \retval DRX_STS_OK:    Successful termination.
* \retval DRX_STS_ERROR: Something went wrong.
*
*/
enum DRXStatus_t DRXBSP_HST_Term( void )
{
   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief  Copy n bytes.
* \param  to:   destination address
* \param  from: source address
* \param  n:    size in bytes
*
* Behavior should by like ansi memcpy() function from <string.h> .
*/
void* DRXBSP_HST_Memcpy( void *to, void *from, u32_t n)
{
   return ( memcpy( to, from, (size_t) n) );
}

/*============================================================================*/

/**
* \brief  Compare the first n bytes of byte arrays s1 and s2.
* \param  s1: pointer to first array of bytes
* \param  s2: pointer to second array of bytes
* \param  n:  number of bytes to compare
*
* Behavior should by like ansi memcmp() or strcmp() function from <string.h>.
*/
int DRXBSP_HST_Memcmp( void *s1, void *s2, u32_t n)
{
   return ( memcmp( s1, s2, (size_t) n) );
}

/*============================================================================*/

/**
* \brief  Function returning the value of a millisecond ticker.
* \return Current value of the millisecond ticker.
*
* For cygwin/gcc the type of the tv_sec & tv_usec is time_t.
* For windows this type is long.
* To let this function work both long or time_t must be double sizeof(u16_t).
* This is ok for cygwin/gcc and windows/MSVS C++ 6.0
*
*/

#if 0	//delete by henry
u32_t DRXBSP_HST_Clock( void )
{
	static enum Bool_t firstTimeCalled=TRUE;
	static LARGE_INTEGER tp_start;
	u32_t  ticker = 0;
	LARGE_INTEGER tp_curr;

	KeQuerySystemTime(&tp_curr);
	
	if (firstTimeCalled == TRUE)
	{
		tp_start=tp_curr;
		firstTimeCalled=FALSE;
	}
	if (tp_curr.QuadPart > tp_start.QuadPart)
		ticker = (u32_t) ( ( tp_curr.QuadPart - tp_start.QuadPart) / 10000 );
	else
		ticker = (u32_t) (( -tp_curr.QuadPart + tp_start.QuadPart) / 10000 );		

	return ( ticker ); /* millisec */
}
#endif
	
/*============================================================================*/

/**
* \brief  Function will wait at least n milliseconds before it returns.
* \param  n number of milliseconds to wait before return
* \retval DRX_STS_OK:     Successful sleep.
* \retval DRX_STS_ERROR:  Unsuccessful sleep.
*
* Wait at least n milliseconds before returning.
* This is a provison to allow the host to spend this waiting time on other
* tasks.
*
* NOTE:
* In case an operating system is used it can happen that the sleep function
* provided by the OS can return before n milliseconds have elapsed. This can be
* caused by, for instance, an OS signal. Such events must be handled in the
* implementation this BSP function.
*
*/
enum DRXStatus_t DRXBSP_HST_Sleep( u32_t n )
{
	msleep(n);
	return (DRX_STS_OK);
}
//end of BSP_HOST.C
//=================================================================
//#################################################################
//=================================================================


//context of BSP_I2C.c		<Henry>  ==========================================
//#################################################################
//=================================================================
/* Store errors from your I2C subsystem as-is in this variable */
int DRX_I2C_Error_g;


/******************************
 *
 * DRXStatus_t DRXBSP_I2C_Init (void)
 *
 * Prepare the platform's I2C subsystem for usage.
 * Should be called before any other I2C function is called.
 *
 * Output:
 * - DRX_STS_OK    system is now ready for use
 * - other         something wrong
 *
 ******************************/

enum DRXStatus_t DRXBSP_I2C_Init( void )
{
    /* TODO: initialize your I2C subsystem */
    return DRX_STS_OK;
}


/******************************
 *
 * DRXStatus_t DRXBSP_I2C_Term (void)
 *
 * Shut down the I2C subsystem of the target platform.
 * After this, I2C functions should no longer be called, until the system is
 * initialized again.
 *
 * Output:
 * - DRX_STS_OK     I2C subsystem shut down
 * - other          something wrong
 *
 ******************************/

enum DRXStatus_t DRXBSP_I2C_Term( void )
{
    /* TODO: terminate your I2C subsystem */
    return DRX_STS_OK;
}


//i2c operation
static int drxk3913_writeregs(struct drxk3913_state *state, u8 reg1,
							u8 *data, int len)
{
	int ret=0;
	u8 buf[512];/* = { reg1, data };*/
	
	struct i2c_msg msg = {
					.addr = state->config->demod_address,
					.flags = 0,
					.buf = buf,
					.len = len+1
		};

		msg.buf[0] =  reg1;
		memcpy(msg.buf + 1, data, len);

		ret = i2c_transfer(state->i2c, &msg, 1);

		if (ret < 0)
		{
			return ret;
			//printk("drxk3913_writeregs error ret=%d\n", ret);
		}
		
	return ret;
}

static int drxk3913_read_regs(struct drxk3913_state *state, u8 reg1, u8 *b, u8 len)
{
	int ret;	
	
	struct i2c_msg msg = {
			.addr = state->config->demod_address,
			.flags = I2C_M_RD,
			.buf = b,
			.len = len
	};	

	ret = i2c_transfer(state->i2c, &msg, 1);
	if (ret<0)
		printk("drxk3913_readregs error ret=%d\n", ret);

	return ret;
}




/******************************
 *
 * DRXStatus_t DRXBSP_I2C_WriteRead(
 *             pI2CDeviceAddr_t wDevAddr,     -- Device to write to
 *             u16_t            wCount,       -- nr of bytes to write
 *             pu8_t            wData,        -- the data to write
 *             pI2CDeviceAddr_t rDevAddr,     -- Device to read from
 *             u16_t            rCount,       -- nr of bytes to read
 *             pu8_t            rData     )   -- buffer receiving the data
 *
 * Write data to I2C device, then receive back specified number of bytes.
 * Must also support write-only (when rDevAddr is NULL)
 *               and read-only  (when wDevAddr is NULL).
 *
 * Output:
 * - DRX_STS_OK           data successfully transferred
 *                        in that case: data received, if any, is in rData
 * - DRX_STS_INVALID_ARG  invalid arguments passed to this function
 * - DRX_STS_ERROR        something wrong in the transfer
 *
 ******************************/

enum DRXStatus_t DRXBSP_I2C_WriteRead( pI2CDeviceAddr_t wDevAddr,
                                  u16_t            wCount,
                                  pu8_t            wData,
                                  pI2CDeviceAddr_t rDevAddr,
                                  u16_t            rCount,
                                  pu8_t            rData    )
{
	//drxk_info("ENTER DRXBSP_I2C_WriteRead\n");
	struct dvb_frontend *fe = NULL;		//wDevAddr->userData;
	struct drxk3913_state *state = NULL;	//fe->demodulator_priv;	
	//int j = 0;
	
	if ( !wDevAddr  && !rDevAddr)
	{
		/* Nothing to do! */
		drxk_info("DRX_STS_INVALID_ARG devaddr\n");
		return DRX_STS_INVALID_ARG;
	}
	
	if (wDevAddr)
	{		
		if (!wData && wCount)
		{
		/* Buffer error! */
		drxk_info("DRX_STS_INVALID_ARG wdata wcount\n");
		return DRX_STS_INVALID_ARG;
		}
		/* TODO: prepare for write (e.g. set-up data structures) */
		fe = wDevAddr->userData;
		state =  fe->demodulator_priv;
		//drxk_info("DRXBSP_I2C_WriteRead wDevAddr state=%p\n", state);	
	}

	if (rDevAddr)
	{
		if (!rData && rCount)
		{
			/* Buffer error! */
			drxk_info("DRX_STS_INVALID_ARG rdata rcount\n");
			return DRX_STS_INVALID_ARG;
		}
		/* TODO: prepare for read (e.g. set-up data structures) */
		if (fe == NULL)
		{
			fe = rDevAddr->userData;
			state =  fe->demodulator_priv;
			//drxk_info("DRXBSP_I2C_WriteRead rDevAddr state=%p\n", state);	
		}
	}
	/* TODO: generate I2C-start signal */


	if (wDevAddr)
	{
		/* TODO: write data */
		/* Note: if wCount is zero, only the I2C-address must be written */
		
		if(wCount-1 <= 0)
		{		
			//drxk_info("Write RegAddr=%02x before read", wData[0]);
			if (drxk3913_writeregs(state, wData[0], wData, 0)<0)
			{
				return DRX_STS_ERROR;
			}
		}
		else
		{
			#if 0
			for(j=0;j <wCount; j++)			
				drxk_info("wData[%d]=%02x", j, wData[j]);			
			#endif
			if (drxk3913_writeregs(state, wData[0], &wData[1], wCount-1)<0)
			{
				return DRX_STS_ERROR;
			}
		}
		if (rDevAddr)
		{
			/* TODO: generate I2C-repeated-start signal (or stop + start) */
		}
	}

	if (rDevAddr)
	{
		/* TODO: read data */
		/* Note: if rCount is zero, only the I2C-address must be written,
		no data must be read */		

		if (drxk3913_read_regs(state, 0, rData, rCount)<0)
		{
			return DRX_STS_ERROR;
		}		
#if 0
		for(j = 0;j<rCount;j++)		
			drxk_info(" %x ",rData[j]);
		
		drxk_info("\n");
#endif
	}
	/* TODO: generate I2C-stop signal */

	/* TODO: return OK or ERROR, e.g. depending on DRX_I2C_Error_g */
	if ( DRX_I2C_Error_g )
	{
		drxk_info("DRX_STS_INVALID_ARG 6\n");
		return DRX_STS_ERROR;
	}

	return DRX_STS_OK;
}


/******************************
 *
 * char* DRXBSP_I2C_ErrorText( void )
 *
 * Returns a human readable error.
 * Counter part of numerical DRX_I2C_Error_g.
 *
 * Output:
 * - Pointer to string containing error description.
 *
 ******************************/

char* DRXBSP_I2C_ErrorText( void )
{
   /* TODO: explain your I2C subsystem-specific error code */
   switch ( DRX_I2C_Error_g ) {
      case 0:
         return ("No error");
         break;
      /*
      case SOME_ERROR:
         return ("Some error");
         break;
      */
      default:
         return("Unknown error");
         break;
   } /* switch */

   return("DRXBSP_I2C_ErrorText error") ;
}
//end of BSP_I2C.c
//==================================================================
//#################################################################
//=================================================================

//context of drx_dap_fasi.c <Henry>  ===========================================
//#################################################################
//=================================================================
/* Function prototypes */
static enum  DRXStatus_t DRXDAP_FASI_WriteBlock (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register/memory   */
        u16_t            datasize,      /* size of data                 */
        pu8_t            data,          /* data to send                 */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadBlock (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register/memory   */
        u16_t            datasize,      /* size of data                 */
        pu8_t            data,          /* data to send                 */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_WriteReg8 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        u8_t             data,          /* data to write                */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadReg8 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        pu8_t            data,          /* buffer to receive data       */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg8 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        waddr,         /* address of register          */
        DRXaddr_t        raddr,         /* address to read back from    */
        u8_t            datain,         /* data to send                 */
        pu8_t           dataout);       /* data to receive back         */

static enum  DRXStatus_t DRXDAP_FASI_WriteReg16 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        u16_t            data,          /* data to write                */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadReg16 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        pu16_t           data,          /* buffer to receive data       */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg16 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        waddr,         /* address of register          */
        DRXaddr_t        raddr,         /* address to read back from    */
        u16_t            datain,        /* data to send                 */
        pu16_t           dataout);      /* data to receive back         */

static enum  DRXStatus_t DRXDAP_FASI_WriteReg32 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        u32_t            data,          /* data to write                */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadReg32 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        pu32_t           data,          /* buffer to receive data       */
        DRXflags_t       flags);        /* special device flags         */

static enum  DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg32 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        waddr,         /* address of register          */
        DRXaddr_t        raddr,         /* address to read back from    */
        u32_t            datain,        /* data to send                 */
        pu32_t           dataout);      /* data to receive back         */

/* The version structure of this protocol implementation */
char drxDapFASIModuleName[]  = "FASI Data Access Protocol";
char drxDapFASIVersionText[] = "";

struct DRXVersion_t drxDapFASIVersion = {
  DRX_MODULE_DAP,             /**< type identifier of the module */
  drxDapFASIModuleName,       /**< name or description of module */

  0,                          /**< major version number */
  0,                          /**< minor version number */
  0,                          /**< patch version number */
  drxDapFASIVersionText       /**< version as text string */
};

/* The structure containing the protocol interface */
struct DRXAccessFunc_t drxDapFASIFunct_g = {
   &drxDapFASIVersion,
   DRXDAP_FASI_WriteBlock,               /* Supported */
   DRXDAP_FASI_ReadBlock,                /* Supported */
   DRXDAP_FASI_WriteReg8,                /* Not supported */
   DRXDAP_FASI_ReadReg8,                 /* Not supported */
   DRXDAP_FASI_ReadModifyWriteReg8,      /* Not supported */
   DRXDAP_FASI_WriteReg16,               /* Supported */
   DRXDAP_FASI_ReadReg16,                /* Supported */
   DRXDAP_FASI_ReadModifyWriteReg16,     /* Supported */
   DRXDAP_FASI_WriteReg32,               /* Supported */
   DRXDAP_FASI_ReadReg32,                /* Supported */
   DRXDAP_FASI_ReadModifyWriteReg32      /* Not supported */
};

/*============================================================================*/

/* Functions not supported by protocol*/

static enum DRXStatus_t DRXDAP_FASI_WriteReg8 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        u8_t             data,          /* data to write                */
        DRXflags_t       flags)         /* special device flags         */
{
   return DRX_STS_ERROR;
}

static enum DRXStatus_t DRXDAP_FASI_ReadReg8 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        addr,          /* address of register          */
        pu8_t            data,          /* buffer to receive data       */
        DRXflags_t       flags)         /* special device flags         */
{
   return DRX_STS_ERROR;
}

static enum DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg8 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        waddr,         /* address of register          */
        DRXaddr_t        raddr,         /* address to read back from    */
        u8_t            datain,         /* data to send                 */
        pu8_t           dataout)        /* data to receive back         */
{
   return DRX_STS_ERROR;
}

static enum DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg32 (
        pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
        DRXaddr_t        waddr,         /* address of register          */
        DRXaddr_t        raddr,         /* address to read back from    */
        u32_t            datain,        /* data to send                 */
        pu32_t           dataout)       /* data to receive back         */
{
   return DRX_STS_ERROR;
}

/*============================================================================*/

/******************************
 *
 * DRXStatus_t DRXDAP_FASI_ReadBlock (
 *      pI2CDeviceAddr_t devAddr,      -- address of I2C device
 *      DRXaddr_t        addr,         -- address of chip register/memory
 *      u16_t            datasize,     -- number of bytes to read
 *      pu8_t            data,         -- data to receive
 *      DRXflags_t       flags)        -- special device flags
 *
 * Read block data from chip address. Because the chip is word oriented,
 * the number of bytes to read must be even.
 *
 * Make sure that the buffer to receive the data is large enough.
 *
 * Although this function expects an even number of bytes, it is still byte
 * oriented, and the data read back is NOT translated to the endianness of
 * the target platform.
 *
 * Output:
 * - DRX_STS_OK     if reading was successful
 *                  in that case: data read is in *data.
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_ReadBlock ( pI2CDeviceAddr_t devAddr,
                                           DRXaddr_t        addr,
                                           u16_t            datasize,
                                           pu8_t            data,
                                           DRXflags_t       flags )
{
   u8_t buf[4];
   u16_t bufx;
   enum  DRXStatus_t rc;
   u16_t overheadSize = 0;

   /* Check parameters ********************************************************/
   if ( devAddr == NULL )
   {
      return DRX_STS_INVALID_ARG;
   }

   overheadSize = (IS_I2C_10BIT (devAddr->i2cAddr) ? 2 : 1) +
                   (DRXDAP_FASI_LONG_FORMAT(addr) ? 4 : 2 );

   if ( (DRXDAP_FASI_OFFSET_TOO_LARGE(addr)) ||
        ((!(DRXDAPFASI_LONG_ADDR_ALLOWED))&&DRXDAP_FASI_LONG_FORMAT(addr)) ||
        (overheadSize > (DRXDAP_MAX_WCHUNKSIZE)) ||
        ((datasize!=0) && (data==NULL)) ||
        ((datasize & 1)==1 ) )
   {
      return DRX_STS_INVALID_ARG;
   }

    /* ReadModifyWrite & mode flag bits are not allowed */
    flags &= (~DRXDAP_FASI_RMW & ~DRXDAP_FASI_MODEFLAGS);
#if DRXDAP_SINGLE_MASTER
    flags |= DRXDAP_FASI_SINGLE_MASTER;
#endif

   /* Read block from I2C *****************************************************/
   do {
      u16_t todo = ( datasize <  DRXDAP_MAX_RCHUNKSIZE ? datasize :
                     DRXDAP_MAX_RCHUNKSIZE);

      bufx = 0;

      addr  &= ~DRXDAP_FASI_FLAGS;
      addr |= flags;

#if ( ( DRXDAPFASI_LONG_ADDR_ALLOWED==1 ) && \
      ( DRXDAPFASI_SHORT_ADDR_ALLOWED==1 ) )
      /* short format address preferred but long format otherwise */
      if ( DRXDAP_FASI_LONG_FORMAT(addr) )
      {
#endif
#if ( DRXDAPFASI_LONG_ADDR_ALLOWED==1 )
         buf[bufx++] = (u8_t) (((addr <<  1) & 0xFF)|0x01);
         buf[bufx++] = (u8_t) ((addr >> 16) & 0xFF);
         buf[bufx++] = (u8_t) ((addr >> 24) & 0xFF);
         buf[bufx++] = (u8_t) ((addr >>  7) & 0xFF);
#endif
#if ( ( DRXDAPFASI_LONG_ADDR_ALLOWED==1 ) && \
      ( DRXDAPFASI_SHORT_ADDR_ALLOWED==1 ) )
      } else {
#endif
#if ( DRXDAPFASI_SHORT_ADDR_ALLOWED==1 )
         buf[bufx++] = (u8_t) ((addr <<  1) & 0xFF);
         buf[bufx++] = (u8_t) ( ((addr >> 16) & 0x0F) | ((addr >> 18) & 0xF0) );
#endif
#if ( ( DRXDAPFASI_LONG_ADDR_ALLOWED==1 ) && \
      ( DRXDAPFASI_SHORT_ADDR_ALLOWED==1 ) )
      }
#endif




#if DRXDAP_SINGLE_MASTER
       /*
        * In single master mode, split the read and write actions.
        * No special action is needed for write chunks here.
        */
       rc = DRXBSP_I2C_WriteRead (devAddr, bufx, buf, 0, 0, 0);
       if (rc == DRX_STS_OK)
       {
           rc = DRXBSP_I2C_WriteRead (0, 0, 0, devAddr, todo, data);
       }
#else
       /* In multi master mode, do everything in one RW action */
       rc = DRXBSP_I2C_WriteRead (devAddr, bufx, buf, devAddr, todo, data);
#endif
       data += todo;
       addr += (todo >> 1);
       datasize -= todo;
   } while (datasize && rc == DRX_STS_OK);

   return rc;
}




/******************************
 *
 * DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg16 (
 *      pI2CDeviceAddr_t devAddr,   -- address of I2C device
 *      DRXaddr_t        waddr,     -- address of chip register/memory
 *      DRXaddr_t        raddr,     -- chip address to read back from
 *      u16_t            wdata,     -- data to send
 *      pu16_t           rdata)     -- data to receive back
 *
 * Write 16-bit data, then read back the original contents of that location.
 * Requires long addressing format to be allowed.
 *
 * Before sending data, the data is converted to little endian. The
 * data received back is converted back to the target platform's endianness.
 *
 * WARNING: This function is only guaranteed to work if there is one
 * master on the I2C bus.
 *
 * Output:
 * - DRX_STS_OK     if reading was successful
 *                  in that case: read back data is at *rdata
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_ReadModifyWriteReg16 ( pI2CDeviceAddr_t devAddr,
                                                      DRXaddr_t        waddr,
                                                      DRXaddr_t        raddr,
                                                      u16_t            wdata,
                                                      pu16_t           rdata )
{
    enum  DRXStatus_t rc=DRX_STS_ERROR;

#if ( DRXDAPFASI_LONG_ADDR_ALLOWED==1 )
    if (rdata == NULL)
    {
        return DRX_STS_INVALID_ARG;
    }

    rc = DRXDAP_FASI_WriteReg16 (devAddr, waddr, wdata, DRXDAP_FASI_RMW);
    if (rc == DRX_STS_OK)
    {
        rc = DRXDAP_FASI_ReadReg16 (devAddr, raddr, rdata, 0);
    }
#endif

    return rc;
}




/******************************
 *
 * DRXStatus_t DRXDAP_FASI_ReadReg16 (
 *     pI2CDeviceAddr_t devAddr, -- address of I2C device
 *     DRXaddr_t        addr,    -- address of chip register/memory
 *     pu16_t           data,    -- data to receive
 *     DRXflags_t       flags)   -- special device flags
 *
 * Read one 16-bit register or memory location. The data received back is
 * converted back to the target platform's endianness.
 *
 * Output:
 * - DRX_STS_OK     if reading was successful
 *                  in that case: read data is at *data
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_ReadReg16 ( pI2CDeviceAddr_t devAddr,
                                           DRXaddr_t        addr,
                                           pu16_t           data,
                                           DRXflags_t       flags )
{
    u8_t buf[sizeof (*data)];
    enum  DRXStatus_t rc;

    if (!data)
    {
        return DRX_STS_INVALID_ARG;
    }
    rc = DRXDAP_FASI_ReadBlock (devAddr, addr, sizeof (*data), buf, flags);
    *data = buf[0] + (((u16_t) buf[1]) << 8);
    return rc;
}




/******************************
 *
 * DRXStatus_t DRXDAP_FASI_ReadReg32 (
 *     pI2CDeviceAddr_t devAddr, -- address of I2C device
 *     DRXaddr_t        addr,    -- address of chip register/memory
 *     pu32_t           data,    -- data to receive
 *     DRXflags_t       flags)   -- special device flags
 *
 * Read one 32-bit register or memory location. The data received back is
 * converted back to the target platform's endianness.
 *
 * Output:
 * - DRX_STS_OK     if reading was successful
 *                  in that case: read data is at *data
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_ReadReg32 ( pI2CDeviceAddr_t devAddr,
                                           DRXaddr_t        addr,
                                           pu32_t           data,
                                           DRXflags_t       flags )
{
    u8_t buf[sizeof (*data)];
    enum  DRXStatus_t rc;

    if (!data)
    {
        return DRX_STS_INVALID_ARG;
    }
    rc = DRXDAP_FASI_ReadBlock (devAddr, addr, sizeof (*data), buf, flags);
    *data = (((u32_t) buf[0]) <<  0) +
            (((u32_t) buf[1]) <<  8) +
            (((u32_t) buf[2]) << 16) +
            (((u32_t) buf[3]) << 24);
    return rc;
}




/******************************
 *
 * DRXStatus_t DRXDAP_FASI_WriteBlock (
 *      pI2CDeviceAddr_t devAddr,    -- address of I2C device
 *      DRXaddr_t        addr,       -- address of chip register/memory
 *      u16_t            datasize,   -- number of bytes to read
 *      pu8_t            data,       -- data to receive
 *      DRXflags_t       flags)      -- special device flags
 *
 * Write block data to chip address. Because the chip is word oriented,
 * the number of bytes to write must be even.
 *
 * Although this function expects an even number of bytes, it is still byte
 * oriented, and the data being written is NOT translated from the endianness of
 * the target platform.
 *
 * Output:
 * - DRX_STS_OK     if writing was successful
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_WriteBlock ( pI2CDeviceAddr_t devAddr,
                                            DRXaddr_t        addr,
                                            u16_t            datasize,
                                            pu8_t            data,
                                            DRXflags_t       flags )
{
   u8_t        buf[ DRXDAP_MAX_WCHUNKSIZE ];
   enum  DRXStatus_t st           = DRX_STS_ERROR;
   u16_t       overheadSize = 0;
   u16_t       blockSize    = 0;

   /* Check parameters ********************************************************/
   if ( devAddr == NULL )
   {
      return DRX_STS_INVALID_ARG;
   }

   overheadSize = (IS_I2C_10BIT (devAddr->i2cAddr) ? 2 : 1) +
                  (DRXDAP_FASI_LONG_FORMAT(addr) ? 4 : 2 );

   if ( ( DRXDAP_FASI_OFFSET_TOO_LARGE(addr) ) ||
        ( ( !(DRXDAPFASI_LONG_ADDR_ALLOWED) ) &&
            DRXDAP_FASI_LONG_FORMAT( addr ) ) ||
        (overheadSize > (DRXDAP_MAX_WCHUNKSIZE)) ||
        ((datasize!=0) && (data==NULL)) ||
        ((datasize & 1)==1 ) )
   {
      return DRX_STS_INVALID_ARG;
   }

    flags &= DRXDAP_FASI_FLAGS;
    flags &= ~DRXDAP_FASI_MODEFLAGS;
#if DRXDAP_SINGLE_MASTER
    flags |= DRXDAP_FASI_SINGLE_MASTER;
#endif

   /* Write block to I2C ******************************************************/
   blockSize = ( (DRXDAP_MAX_WCHUNKSIZE) - overheadSize) & ~1;
   do
   {
      u16_t todo = 0;
      u16_t bufx = 0;

      /* Buffer device address */
      addr  &= ~DRXDAP_FASI_FLAGS;
      addr |= flags;
#if ( ( (DRXDAPFASI_LONG_ADDR_ALLOWED)==1 ) && \
      ( (DRXDAPFASI_SHORT_ADDR_ALLOWED)==1 ) )
      /* short format address preferred but long format otherwise */
      if ( DRXDAP_FASI_LONG_FORMAT(addr) )
      {
#endif
#if ( (DRXDAPFASI_LONG_ADDR_ALLOWED)==1 )
         buf[bufx++] = (u8_t) (((addr <<  1) & 0xFF)|0x01);
         buf[bufx++] = (u8_t) ((addr >> 16) & 0xFF);
         buf[bufx++] = (u8_t) ((addr >> 24) & 0xFF);
         buf[bufx++] = (u8_t) ((addr >>  7) & 0xFF);
#endif
#if ( ( (DRXDAPFASI_LONG_ADDR_ALLOWED)==1 ) && \
      ( (DRXDAPFASI_SHORT_ADDR_ALLOWED)==1 ) )
      } else {
#endif
#if ( (DRXDAPFASI_SHORT_ADDR_ALLOWED)==1 )
         buf[bufx++] = (u8_t) ((addr <<  1) & 0xFF);
         buf[bufx++] = (u8_t) ( ((addr >> 16) & 0x0F) | ((addr >> 18) & 0xF0) );
#endif
#if ( ( (DRXDAPFASI_LONG_ADDR_ALLOWED)==1 ) && \
      ( (DRXDAPFASI_SHORT_ADDR_ALLOWED)==1 ) )
      }
#endif

      /*
      In single master mode blockSize can be 0. In such a case this I2C
      sequense will be visible: (1) write address {i2c addr,
      4 bytes chip address} (2) write data {i2c addr, 4 bytes data }
      (3) write address (4) write data etc...
      Addres must be rewriten because HI is reset after data transport and
      expects an address.
      */
      todo = (blockSize < datasize ? blockSize : datasize);
      if (todo==0)
      {
         u16_t overheadSizeI2cAddr = 0;
         u16_t dataBlockSize       = 0;

         overheadSizeI2cAddr = (IS_I2C_10BIT (devAddr->i2cAddr) ? 2 : 1);
         dataBlockSize = ( DRXDAP_MAX_WCHUNKSIZE - overheadSizeI2cAddr) & ~1;

         /* write device address */
       //printk("%s %d write Block len=%d\n", __func__, __LINE__, bufx);
         st = DRXBSP_I2C_WriteRead ( devAddr,
                                     (u16_t) (bufx),
                                     buf,
                                     (pI2CDeviceAddr_t)(NULL),
                                     0,
                                     (pu8_t)(NULL) );
         if(st != DRX_STS_OK)
         {
		 drxk_info("DRXDAP_FASI_WriteBlock::DRXBSP_I2C_WriteRead fail!\n ");
		 return st;
         }
         bufx = 0;
         todo = (dataBlockSize < datasize ? dataBlockSize : datasize);
      }
      DRXBSP_HST_Memcpy (&buf[bufx], data, todo);
      /* write (address if can do and) data */

      //printk("%s %d write Block len=%d\n",  __func__, __LINE__, bufx+todo);
      st = DRXBSP_I2C_WriteRead ( devAddr,
                                  (u16_t)(bufx + todo),
                                  buf,
                                  (pI2CDeviceAddr_t)(NULL),
                                  0,
                                  (pu8_t)(NULL) );
         if(st != DRX_STS_OK)
         {
		 drxk_info("DRXDAP_FASI_WriteBlock::DRXBSP_I2C_WriteRead fail!\n ");
		 return st;
         }
	  
      datasize -= todo;
      data += todo;
      addr += (todo >> 1);
   } while (datasize);

   return st;
}




/******************************
 *
 * DRXStatus_t DRXDAP_FASI_WriteReg16 (
 *     pI2CDeviceAddr_t devAddr, -- address of I2C device
 *     DRXaddr_t        addr,    -- address of chip register/memory
 *     u16_t            data,    -- data to send
 *     DRXflags_t       flags)   -- special device flags
 *
 * Write one 16-bit register or memory location. The data being written is
 * converted from the target platform's endianness to little endian.
 *
 * Output:
 * - DRX_STS_OK     if writing was successful
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_WriteReg16 ( pI2CDeviceAddr_t devAddr,
                                            DRXaddr_t        addr,
                                            u16_t            data,
                                            DRXflags_t       flags )
{
    u8_t buf[sizeof (data)];

    buf[0] = (u8_t) ( (data >> 0 ) & 0xFF );
    buf[1] = (u8_t) ( (data >> 8 ) & 0xFF );

    return DRXDAP_FASI_WriteBlock (devAddr, addr, sizeof (data), buf, flags);
}




/******************************
 *
 * DRXStatus_t DRXDAP_FASI_WriteReg32 (
 *     pI2CDeviceAddr_t devAddr, -- address of I2C device
 *     DRXaddr_t        addr,    -- address of chip register/memory
 *     u32_t            data,    -- data to send
 *     DRXflags_t       flags)   -- special device flags
 *
 * Write one 32-bit register or memory location. The data being written is
 * converted from the target platform's endianness to little endian.
 *
 * Output:
 * - DRX_STS_OK     if writing was successful
 * - DRX_STS_ERROR  if anything went wrong
 *
 ******************************/

static enum DRXStatus_t DRXDAP_FASI_WriteReg32 ( pI2CDeviceAddr_t devAddr,
                                            DRXaddr_t        addr,
                                            u32_t            data,
                                            DRXflags_t       flags )
{
    u8_t buf[sizeof (data)];

    buf[0] = (u8_t) ( (data >> 0 ) & 0xFF );
    buf[1] = (u8_t) ( (data >> 8 ) & 0xFF );
    buf[2] = (u8_t) ( (data >> 16) & 0xFF );
    buf[3] = (u8_t) ( (data >> 24) & 0xFF );

    return DRXDAP_FASI_WriteBlock (devAddr, addr, sizeof (data), buf, flags);
}
//end of drx_dsp_fasi.c
//=================================================================
//#################################################################
//=================================================================


//context of BSP_tuner.c	<Henry>==========================================
//#################################################################
//=================================================================
/**
* \fn DRXStatus_t DRXBSP_TUNER_Open( pTUNERInstance_t tuner )
* \brief Open a tuner.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Opened tuner with success.
* \retval DRX_STS_ERROR Something went wrong.
* \retval DRX_STS_INVALID_ARG Invalid tuner instance.
*/
enum DRXStatus_t
DRXBSP_TUNER_Open( pTUNERInstance_t tuner )
{
   enum DRXStatus_t err;

   /*
      do restricted sanity checks of the tuner instance data structures,
      so other functions can rely on the parameters in the structures
    */


   if ( ( tuner == NULL )             ||
        ( tuner->myCommonAttr == NULL ) ||
        ( tuner->myFunct == NULL ) )

   {
      /* incomplete or no tuner structure */
      return DRX_STS_INVALID_ARG;
   }

   if ( tuner->myCommonAttr->selfCheck == &(tuner->myCommonAttr->selfCheck) )
   {
   	drxk_info("/* tuner already opened */");
      /* tuner already opened */
      //return DRX_STS_ERROR;//dennis comment this.
   }

   err = (*(tuner->myFunct->openFunc))( tuner );

   if ( err == DRX_STS_OK )
   {
      tuner->myCommonAttr->programmed = FALSE;
      tuner->myCommonAttr->selfCheck  = &tuner->myCommonAttr->selfCheck;
   } else {
      tuner->myCommonAttr->selfCheck  = NULL;
   }

   return ( err );
}

/*============================================================================*/

/**
* \fn DRXStatus_t DRXBSP_TUNER_Close( pTUNERInstance_t tuner )
* \brief Close a tuner.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Closed tuner with success.
* \retval DRX_STS_ERROR Something went wrong.
* \retval DRX_STS_INVALID_ARG Invalid tuner instance.
*/
enum DRXStatus_t
DRXBSP_TUNER_Close( pTUNERInstance_t tuner )
{
   enum DRXStatus_t err = DRX_STS_OK;;

   if ( ( tuner == NULL )             ||
        ( tuner->myCommonAttr == NULL ) ||
        ( tuner->myFunct == NULL ) )

   {
      /* incomplete or no tuner structure */
      return DRX_STS_INVALID_ARG;
   }
   
   
   if ( tuner->myCommonAttr->selfCheck == &(tuner->myCommonAttr->selfCheck) )
   {
      /* selfcheck OK, close the tuner */
      err = (*(tuner->myFunct->closeFunc))( tuner );
   }
   
   if ( err == DRX_STS_OK )
   {
      tuner->myCommonAttr->selfCheck = NULL;
   }

   return( err );
}

/*============================================================================*/

/**
* \fn DRXStatus_t DRXBSP_TUNER_SetFrequency( pTUNERInstance_t tuner,
                                             TUNERMode_t mode,
                                             DRXFrequency_t centerFrequency )
* \brief Program tuner at given center frequency for given mode.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Programmed tuner successfully.
* \retval DRX_STS_ERROR Something went wrong.
* \retval DRX_STS_INVALID_ARG Invalid tuner instance.
*/
enum DRXStatus_t
DRXBSP_TUNER_SetFrequency( pTUNERInstance_t tuner,
                           TUNERMode_t      mode,
                           DRXFrequency_t   centerFrequency )
{
   enum DRXStatus_t err;

   if ( ( tuner == NULL )             ||
        ( tuner->myCommonAttr == NULL ) ||
        ( tuner->myFunct == NULL ) )

   {
      /* incomplete or no tuner structure */
      return DRX_STS_INVALID_ARG;
   }

   if ( tuner->myCommonAttr->selfCheck != &(tuner->myCommonAttr->selfCheck) )
   {
      /* tuner not opened */
      return DRX_STS_ERROR;
   }

   err = (*(tuner->myFunct->setFrequencyFunc))( tuner, mode, centerFrequency );

   if ( err == DRX_STS_OK )
   {
      tuner->myCommonAttr->programmed = TRUE;
   } else {
      tuner->myCommonAttr->programmed = FALSE;
   }

   return (err);
}

/*============================================================================*/

/**
* \fn DRXStatus_t DRXBSP_TUNER_GetFrequency( pTUNERInstance_t tuner,
                                             pDRXFrequency_t  RFfrequency,
                                             pDRXFrequency_t  IFfrequency )
* \brief Get tuned center frequency.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Returned frequency successfully.
* \retval DRX_STS_ERROR Something went wrong.
* \retval DRX_STS_INVALID_ARG Invalid tuner instance.
*/
enum DRXStatus_t
DRXBSP_TUNER_GetFrequency( pTUNERInstance_t tuner,
                           pDRXFrequency_t  RFfrequency,
                           pDRXFrequency_t  IFfrequency )
{
   if ( ( tuner == NULL )             ||
        ( tuner->myCommonAttr == NULL ) ||
        ( tuner->myFunct == NULL ) )

   {
      /* incomplete or no tuner structure */
      return DRX_STS_INVALID_ARG;
   }

   if ( ( tuner->myCommonAttr->selfCheck !=
          &(tuner->myCommonAttr->selfCheck) ) ||
         !tuner->myCommonAttr->programmed )
   {
      /* tuner not opened, or no frequency programmed yet */
      return DRX_STS_ERROR;
   }

   return( (*(tuner->myFunct->getFrequencyFunc))( tuner,
                                                  RFfrequency,
                                                  IFfrequency) );
}

/*============================================================================*/

/**
* \fn DRXStatus_t DRXBSP_TUNER_LockStatus( pTUNERInstance_t tuner,
      pTUNERLockStatus_t lockStat )
* \brief Get tuner locking status.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Acquired locking status successfully.
* \retval DRX_STS_ERROR Something went wrong.
* \retval DRX_STS_INVALID_ARG Invalid tuner instance.
*/
enum DRXStatus_t
DRXBSP_TUNER_LockStatus( pTUNERInstance_t    tuner,
                         pTUNERLockStatus_t  lockStat )
{
   if ( ( tuner == NULL )             ||
        ( tuner->myCommonAttr == NULL ) ||
        ( tuner->myFunct == NULL ) )

   {
      /* incomplete or no tuner structure */
      return DRX_STS_INVALID_ARG;
   }

   if ( tuner->myCommonAttr->selfCheck != &(tuner->myCommonAttr->selfCheck) )
   {
      /* tuner not opened */
      return DRX_STS_ERROR;
   }

   return ( (*(tuner->myFunct->lockStatusFunc))( tuner, lockStat ) );
}

/*============================================================================*/

/**
* \fn DRXStatus_t DRXBSP_TUNER_DefaultI2CWriteRead(
               pTUNERInstance_t tuner,
               pI2CDeviceAddr_t wDevAddr,
               u16_t            wCount,
               pu8_t            wData,
               pI2CDeviceAddr_t rDevAddr,
               u16_t            rCount,
               pu8_t            rData);
* \brief Standard I2C read write function.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Acquired locking status successfully.
* \retval DRX_STS_ERROR Something went wrong.
* \retval DRX_STS_INVALID_ARG Invalid tuner instance.
*
* Default implementation of I2CWriteRead function for tuners, via bsp_i2c.
*
*/
enum DRXStatus_t
DRXBSP_TUNER_DefaultI2CWriteRead( pTUNERInstance_t   tuner,
                                  pI2CDeviceAddr_t wDevAddr,
                                  u16_t            wCount,
                                  pu8_t            wData,
                                  pI2CDeviceAddr_t rDevAddr,
                                  u16_t            rCount,
                                  pu8_t            rData)
{
   /* Default to BSP_I2C routine */
   return ( DRXBSP_I2C_WriteRead( wDevAddr, wCount, wData,
                                  rDevAddr, rCount, rData) );
}

//end of BSP_tuner.c
//=================================================================
//#################################################################
//=================================================================

/*------------------------------------------------------------------------------
DEFINES
------------------------------------------------------------------------------*/

/*============================================================================*/
/*=== MICROCODE RELATED DEFINES ==============================================*/
/*============================================================================*/

/** \brief Magic word for checking correct Endianess of microcode data. */
#ifndef DRX_UCODE_MAGIC_WORD
#define DRX_UCODE_MAGIC_WORD         ((((u16_t)'H')<<8)+((u16_t)'L'))
#endif

/** \brief CRC flag in ucode header, flags field. */
#ifndef DRX_UCODE_CRC_FLAG
#define DRX_UCODE_CRC_FLAG           (0x0001)
#endif

/** \brief Compression flag in ucode header, flags field. */
#ifndef DRX_UCODE_COMPRESSION_FLAG
#define DRX_UCODE_COMPRESSION_FLAG   (0x0002)
#endif

/** \brief Maximum size of buffer used to verify the microcode.
      Must be an even number. */
#ifndef DRX_UCODE_MAX_BUF_SIZE
#define DRX_UCODE_MAX_BUF_SIZE       (DRXDAP_MAX_RCHUNKSIZE)
#endif
#if DRX_UCODE_MAX_BUF_SIZE & 1
#error DRX_UCODE_MAX_BUF_SIZE must be an even number
#endif

/*============================================================================*/
/*=== CHANNEL SCAN RELATED DEFINES ===========================================*/
/*============================================================================*/

/**
* \brief Maximum progress indication.
*
* Progress indication will run from 0 upto DRX_SCAN_MAX_PROGRESS during scan.
*
*/
#ifndef DRX_SCAN_MAX_PROGRESS
#define DRX_SCAN_MAX_PROGRESS 1000
#endif

/** \brief Distance in kHz between two centre frequencies in 8Mhz grid */
#ifndef DRX_SCAN_8MHZ_STEP
#define DRX_SCAN_8MHZ_STEP 8000
#endif

/** \brief Distance in kHz between two centre frequencies in 7Mhz grid */
#ifndef DRX_SCAN_7MHZ_STEP
#define DRX_SCAN_7MHZ_STEP 7000
#endif

/** \brief Distance in kHz between two centre frequencies in 6Mhz grid */
#ifndef DRX_SCAN_6MHZ_STEP
#define DRX_SCAN_6MHZ_STEP 6000
#endif

/*============================================================================*/
/*=== MACROS =================================================================*/
/*============================================================================*/

#define DRX_ISPOWERDOWNMODE( mode ) (  ( mode == DRX_POWER_MODE_9  ) || \
                                       ( mode == DRX_POWER_MODE_10 ) || \
                                       ( mode == DRX_POWER_MODE_11 ) || \
                                       ( mode == DRX_POWER_MODE_12 ) || \
                                       ( mode == DRX_POWER_MODE_13 ) || \
                                       ( mode == DRX_POWER_MODE_14 ) || \
                                       ( mode == DRX_POWER_MODE_15 ) || \
                                       ( mode == DRX_POWER_MODE_16 ) || \
                                       ( mode == DRX_POWER_DOWN    ) )

/*------------------------------------------------------------------------------
GLOBAL VARIABLES
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
STRUCTURES
------------------------------------------------------------------------------*/
/** \brief  Structure of the microcode block headers */
struct DRXUCodeBlockHdr_t{
   u32_t addr;    /**< Destination address of the data in this block */
   u16_t size;    /**< Size of the block data following this header counted in
                        16 bits words */
   u16_t flags;   /**< Flags for this data block:
                        - bit[0]= CRC on/off
                        - bit[1]= compression on/off
                        - bit[15..2]=reserved */
   u16_t CRC;     /**< CRC value of the data block, only valid if CRC flag is
                        set. */
};
typedef struct DRXUCodeBlockHdr_t* pDRXUCodeBlockHdr_t;
/*------------------------------------------------------------------------------
FUNCTIONS
------------------------------------------------------------------------------*/

/*============================================================================*/
/*============================================================================*/
/*== Channel Scan Functions ==================================================*/
/*============================================================================*/
/*============================================================================*/

#ifndef DRX_EXCLUDE_SCAN

/* Prototype of default scanning function */
static enum DRXStatus_t
DefaultScanFunction( pDRXDemodInstance_t demod,
                     enum DRXScanCommand_t    scanCommand,
                     pDRXChannel_t       scanChannel,
                     enum Bool_t             *getNextChannel  );

/**
* \brief Get pointer to scanning function.
* \param demod:    Pointer to demodulator instance.
* \return DRXScanFunc_t.
*/
static DRXScanFunc_t
GetScanFunction( pDRXDemodInstance_t demod )
{
   pDRXCommonAttr_t commonAttr = (pDRXCommonAttr_t)(NULL);
   DRXScanFunc_t    scanFunc   = (DRXScanFunc_t)(NULL);

   /* get scan function from common attributes */
   commonAttr  = (pDRXCommonAttr_t)demod -> myCommonAttr;
   scanFunc    = commonAttr -> scanFunction;

   if ( scanFunc != NULL )
   {
      /* return device-specific scan function if it's not NULL */
      return scanFunc;
   }
   /* otherwise return default scan function in core driver */
   return &DefaultScanFunction;
}

/*============================================================================*/

/**
* \brief Wait for lock while scanning.
* \param demod:    Pointer to demodulator instance.
* \param lockStat: Pointer to bool indicating if end result is lock or not.
* \return DRXStatus_t.
* \retval DRX_STS_OK:    Success
* \retval DRX_STS_ERROR: I2C failure or bsp function failure.
*
* Wait until timeout, desired lock or NEVER_LOCK.
* Assume:
* - lock function returns : at least DRX_NOT_LOCKED and a lock state
*   higher then DRX_NOT_LOCKED.
* - BSP has a clock function to retrieve a millisecond ticker value.
* - BSP has a sleep function to enable sleep of n millisecond.
*
* In case DRX_NEVER_LOCK is returned the poll-wait will be aborted.
*
*/
static enum DRXStatus_t
ScanWaitForLock(  pDRXDemodInstance_t demod,
                  enum Bool_t             *isLocked )
{
   enum Bool_t           doneWaiting = FALSE;
   enum DRXLockStatus_t  lockState = DRX_NOT_LOCKED;
   enum DRXLockStatus_t  desiredLockState = DRX_NOT_LOCKED;
   u32_t            timeoutValue = 0;
   u32_t            time = 0;


   *isLocked = FALSE;
   timeoutValue = (u32_t) demod->myCommonAttr->scanDemodLockTimeout;
   desiredLockState = demod->myCommonAttr->scanDesiredLock;

   time = 0;

   /* Start polling loop, checking for lock & timeout */
   while ( doneWaiting == FALSE ) {

      if ( DRX_Ctrl( demod, DRX_CTRL_LOCK_STATUS, &lockState ) != DRX_STS_OK )
      {
         return (DRX_STS_ERROR);
      }
      msleep(10);
	time++;
      
      if ( lockState >= desiredLockState )
      {
               *isLocked = TRUE;
               doneWaiting = TRUE;
      } /* if ( lockState >= desiredLockState ) .. */
      else if ( lockState == DRX_NEVER_LOCK )
      {
               doneWaiting = TRUE;
      } /* if ( lockState == DRX_NEVER_LOCK ) .. */
      else if ( (time*10) > timeoutValue )
      {
         /* lockState==DRX_NOT_LOCKED  and timeout */
         doneWaiting = TRUE;
      }
      else
      {
         if ( DRXBSP_HST_Sleep( 10 ) != DRX_STS_OK )
         {
            return (DRX_STS_ERROR);
         }
      }/* if ( timerValue > timeoutValue ) .. */

   } /* while */

   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief Determine next frequency to scan.
* \param demod: Pointer to demodulator instance.
* \return DRXStatus_t.
* \retval DRX_STS_OK:          Succes.
* \retval DRX_STS_INVALID_ARG: Invalid frequency plan.
*
* Helper function for CtrlScanNext() function.
* Compute next frequency & index in frequency plan.
* Check if scan is ready.
*
*/
static enum DRXStatus_t
ScanPrepareNextScan ( pDRXDemodInstance_t  demod )
{
   pDRXCommonAttr_t     commonAttr=(pDRXCommonAttr_t)(NULL);
   u16_t                tableIndex = 0;
   u16_t                frequencyPlanSize = 0;
   pDRXFrequencyPlan_t  frequencyPlan = (pDRXFrequencyPlan_t)(NULL);
   DRXFrequency_t       nextFrequency = 0;
   DRXFrequency_t       frequencyStep = 0;
   DRXFrequency_t       tunerMinFrequency = 0;
   DRXFrequency_t       tunerMaxFrequency = 0;

   commonAttr        = (pDRXCommonAttr_t)demod->myCommonAttr;
   tableIndex        = commonAttr->scanFreqPlanIndex;
   frequencyPlan     = commonAttr->scanParam->frequencyPlan;
   nextFrequency     = commonAttr->scanNextFrequency;
   tunerMinFrequency = commonAttr->tunerMinFreqRF;
   tunerMaxFrequency = commonAttr->tunerMaxFreqRF;

   do {
      /* Determine frequency step of channel to scan */
      switch ( frequencyPlan[tableIndex].bandwidth ) {
         case DRX_BANDWIDTH_8MHZ:
            frequencyStep = DRX_SCAN_8MHZ_STEP;
            break;
         case DRX_BANDWIDTH_7MHZ:
            frequencyStep = DRX_SCAN_7MHZ_STEP;
            break;
         case DRX_BANDWIDTH_6MHZ:
            frequencyStep = DRX_SCAN_6MHZ_STEP;
            break;
         default:
            /* Frequency plan content is invalid */
            return (DRX_STS_INVALID_ARG);
            break;
      };

      /* Search next frequency to scan */
      nextFrequency += frequencyStep;
      if ( nextFrequency > frequencyPlan[tableIndex].last )
      {
         /* reached end of this band */
         tableIndex++;
         frequencyPlanSize = commonAttr->scanParam->frequencyPlanSize;
         if ( tableIndex >= frequencyPlanSize )
         {
            /* reached end of frequency plan */
            commonAttr->scanReady = TRUE;
         } else {
            nextFrequency = frequencyPlan[tableIndex].first;
         }
      }
      if ( nextFrequency > (tunerMaxFrequency) )
      {
         /* reached end of tuner range */
         commonAttr->scanReady = TRUE;
      }
   } while( ( nextFrequency < tunerMinFrequency ) &&
            ( commonAttr->scanReady == FALSE ) );

   /* Store new values */
   commonAttr->scanFreqPlanIndex = tableIndex;
   commonAttr->scanNextFrequency = nextFrequency;

   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief Default DTV scanning function.
*
* \param demod:          Pointer to demodulator instance.
* \param scanCommand:    Scanning command: INIT, NEXT or STOP.
* \param scanChannel:    Channel to check: frequency and bandwidth, others AUTO
* \param getNextChannel: Return TRUE if next frequency is desired at next call
*
* \return DRXStatus_t.
* \retval DRX_STS_OK:      Channel found, DRX_CTRL_GET_CHANNEL can be used
*                             to retrieve channel parameters.
* \retval DRX_STS_BUSY:    Channel not found (yet).
* \retval DRX_STS_ERROR:   Something went wrong.
*
* scanChannel and getNextChannel will be NULL for INIT and STOP.
*/
static enum DRXStatus_t
DefaultScanFunction( pDRXDemodInstance_t demod,
                     enum DRXScanCommand_t    scanCommand,
                     pDRXChannel_t       scanChannel,
                     enum Bool_t             *getNextChannel  )
{
   enum DRXStatus_t status   = DRX_STS_ERROR;
   enum Bool_t      isLocked = FALSE;

   if ( scanCommand != DRX_SCAN_COMMAND_NEXT )
   {
      /* just return OK if not doing "scan next" */
      return DRX_STS_OK;
   }

   *getNextChannel = FALSE;

   status = DRX_Ctrl( demod, DRX_CTRL_SET_CHANNEL, scanChannel );
   if ( status != DRX_STS_OK )
   {
      return (status);
   }

   status = ScanWaitForLock( demod, &isLocked );
   if ( status != DRX_STS_OK )
   {
      return (status);
   }

   /* done with this channel, move to next one */
   *getNextChannel = TRUE;

   if ( isLocked == FALSE )
   {
      /* no channel found */
      return (DRX_STS_BUSY);
   }
   /* channel found */
   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief Initialize for channel scan.
* \param demod:     Pointer to demodulator instance.
* \param scanParam: Pointer to scan parameters.
* \return DRXStatus_t.
* \retval DRX_STS_OK:          Initialized for scan.
* \retval DRX_STS_ERROR:       No overlap between frequency plan and tuner
*                              range.
* \retval DRX_STS_INVALID_ARG: Wrong parameters.
*
* This function should be called before starting a complete channel scan.
* It will prepare everything for a complete channel scan.
* After calling this function the DRX_CTRL_SCAN_NEXT control function can be
* used to perform the actual scanning. Scanning will start at the first
* center frequency of the frequency plan that is within the tuner range.
*
*/
static enum DRXStatus_t
CtrlScanInit(  pDRXDemodInstance_t   demod,
               pDRXScanParam_t       scanParam )
{
   enum DRXStatus_t status = DRX_STS_ERROR;
   pDRXCommonAttr_t commonAttr=(pDRXCommonAttr_t)(NULL);
   DRXFrequency_t maxTunerFreq = 0;
   DRXFrequency_t minTunerFreq = 0;
   u16_t nrChannelsInPlan = 0;
   u16_t i = 0;

   commonAttr = (pDRXCommonAttr_t)demod -> myCommonAttr;
   commonAttr->scanActive = TRUE;

   /* invalidate a previous SCAN_INIT */
   commonAttr->scanParam = (pDRXScanParam_t)(NULL);
   commonAttr->scanNextFrequency = 0;

   /* Check parameters */
   if (  (( demod->myTuner == NULL )&&( scanParam->numTries !=1)) ||
         ( scanParam == NULL) ||
         ( scanParam->numTries == 0) ||
         ( scanParam->frequencyPlan == NULL) ||
         ( scanParam->frequencyPlanSize == 0 )
      )
   {
      commonAttr->scanActive = FALSE;
      return (DRX_STS_INVALID_ARG);
   } /* if */

   /* Check frequency plan contents */
   maxTunerFreq = commonAttr->tunerMaxFreqRF;
   minTunerFreq = commonAttr->tunerMinFreqRF;
   for( i = 0; i < (scanParam->frequencyPlanSize); i++ )
   {
      DRXFrequency_t width = 0;
      DRXFrequency_t step = 0;
      DRXFrequency_t firstFreq = scanParam->frequencyPlan[i].first;
      DRXFrequency_t lastFreq  = scanParam->frequencyPlan[i].last;
      DRXFrequency_t minFreq = 0;
      DRXFrequency_t maxFreq = 0;

      if ( firstFreq > lastFreq )
      {
            /* First centre frequency is higher then last centre frequency */
            commonAttr->scanActive = FALSE;
            return (DRX_STS_INVALID_ARG);
      } /* if */

      width = lastFreq - firstFreq;
      switch ( scanParam->frequencyPlan[i].bandwidth ) {
         case DRX_BANDWIDTH_6MHZ:
            step = DRX_SCAN_6MHZ_STEP;
            break;
         case DRX_BANDWIDTH_7MHZ:
            step = DRX_SCAN_7MHZ_STEP;
            break;
         case DRX_BANDWIDTH_8MHZ:
            step = DRX_SCAN_8MHZ_STEP;
            break;
         default:
            /* Invalid bandwidth parameter in frequency plan */
            commonAttr->scanActive = FALSE;
            return (DRX_STS_INVALID_ARG);
            break;
      }; /* switch */

      if ( (width % step) != 0 )
      {
            /* Difference between last and first centre frequency is not
               an integer number of bandwidths */
            commonAttr->scanActive = FALSE;
            return (DRX_STS_INVALID_ARG);
      } /* if */

      /* Check if frequency plan entry intersects with tuner range */
      if ( lastFreq >= minTunerFreq )
      {
         if ( firstFreq <= maxTunerFreq )
         {
            if (  firstFreq >= minTunerFreq )
            {
               minFreq = firstFreq;
            } else {
               DRXFrequency_t n=0;

               n=( minTunerFreq - firstFreq )/step;
               if ( (( minTunerFreq - firstFreq )%step) != 0 )
               {
                  n++;
               }
               minFreq = firstFreq + n*step;
            }
            if ( lastFreq <= maxTunerFreq )
            {
               maxFreq = lastFreq;
            } else {
               DRXFrequency_t n=0;

               n=( lastFreq - maxTunerFreq )/step;
               if ( (( lastFreq - maxTunerFreq )%step) !=0 )
               {
                  n++;
               }
               maxFreq = lastFreq - n*step;
            }
         }
      }

      /* Keep track of total number of channels within tuner range
         in this frequency plan. */
      if ( (minFreq!=0) && (maxFreq!=0) )
      {
         nrChannelsInPlan += (u16_t)(( (maxFreq-minFreq) / step ) +1 );

         /* Determine first frequency (within tuner range) to scan */
         if ( commonAttr->scanNextFrequency == 0 )
         {
            commonAttr->scanNextFrequency = minFreq;
            commonAttr->scanFreqPlanIndex = i;
         }
      }

   }/* for ( ... ) */

   if ( nrChannelsInPlan == 0 )
   {
      /* Tuner range and frequency plan ranges do not overlap */
      commonAttr->scanActive = FALSE;
      return (DRX_STS_ERROR);
   }

   /* Store parameters */
   commonAttr->scanReady = FALSE;
   commonAttr->scanMaxChannels = nrChannelsInPlan;
   commonAttr->scanChannelsScanned = 0;
   commonAttr->scanParam = scanParam; /* SCAN_NEXT is now allowed */

   /* Now call default or device-specific scanning init function */
   status = (*(GetScanFunction( demod )))
                  ( demod, DRX_SCAN_COMMAND_INIT, NULL, NULL );

   /* All done */
   commonAttr->scanActive = FALSE;
   return (status);
}

/*============================================================================*/

/**
* \brief Stop scanning.
* \param demod:         Pointer to demodulator instance.
* \return DRXStatus_t.
* \retval DRX_STS_OK:          Scan stopped.
* \retval DRX_STS_ERROR:       Something went wrong.
* \retval DRX_STS_INVALID_ARG: Wrong parameters.
*/
static enum DRXStatus_t
CtrlScanStop( pDRXDemodInstance_t  demod  )
{
   enum DRXStatus_t status = DRX_STS_ERROR;
   pDRXCommonAttr_t commonAttr = (pDRXCommonAttr_t)(NULL);
   commonAttr = (pDRXCommonAttr_t)demod -> myCommonAttr;

   commonAttr->scanActive = TRUE;

   if (  ( commonAttr->scanParam == NULL ) ||
         ( commonAttr->scanMaxChannels == 0 )
      )
   {
      /* Scan was not running, just return OK */
      commonAttr->scanActive = FALSE;
      return (DRX_STS_OK);
   }

   /* Call default or device-specific scanning stop function */
   status = (*(GetScanFunction( demod )))
                  ( demod, DRX_SCAN_COMMAND_STOP, NULL, NULL );

   /* All done, invalidate scan-init */
   commonAttr->scanParam         = NULL;
   commonAttr->scanMaxChannels   = 0;
   commonAttr->scanActive        = FALSE;

   return (status);
}

/*============================================================================*/

/**
* \brief Scan for next channel.
* \param demod:         Pointer to demodulator instance.
* \param scanProgress:  Pointer to scan progress.
* \return DRXStatus_t.
* \retval DRX_STS_OK:          Channel found, DRX_CTRL_GET_CHANNEL can be used
*                              to retrieve channel parameters.
* \retval DRX_STS_BUSY:        Tried part of the channels, as specified in
*                              numTries field of scan parameters. At least one
*                              more call to DRX_CTRL_SCAN_NEXT is needed to
*                              complete scanning.
* \retval DRX_STS_READY:       Reached end of scan range.
* \retval DRX_STS_ERROR:       Something went wrong.
* \retval DRX_STS_INVALID_ARG: Wrong parameters. The scanProgress may be NULL.
*
* Progress indication will run from 0 upto DRX_SCAN_MAX_PROGRESS during scan.
*
*/
static enum DRXStatus_t
CtrlScanNext(  pDRXDemodInstance_t  demod,
               pu16_t               scanProgress )
{
   pDRXCommonAttr_t  commonAttr  = (pDRXCommonAttr_t)(NULL);
   enum Bool_t           *scanReady   = (pBool_t)(NULL);
   u16_t             maxProgress = DRX_SCAN_MAX_PROGRESS;
   u32_t             numTries    = 0;
   u32_t             i           = 0;

   commonAttr = (pDRXCommonAttr_t)demod -> myCommonAttr;
   commonAttr->scanActive = TRUE;
   if (  ( commonAttr->scanParam == NULL ) ||
         ( commonAttr->scanMaxChannels == 0 )
      )
   {
      /* CtrlScanInit() was not called succesfully before CtrlScanNext() */
      commonAttr->scanActive = FALSE;
      return (DRX_STS_ERROR);
   }

   /* Check scan parameters */
   if ( scanProgress == NULL )
   {
      commonAttr->scanActive = FALSE;
      return (DRX_STS_INVALID_ARG);
   }
   *scanProgress = (u16_t)(((commonAttr->scanChannelsScanned)*
                              ((u32_t)(maxProgress)))/
                                 (commonAttr->scanMaxChannels));
   /* Scan */
   numTries = commonAttr->scanParam->numTries;
   scanReady = &(commonAttr->scanReady);

   for ( i = 0;
         ( i < numTries ) && ( (*scanReady) == FALSE );
         i++ )
   {
      enum DRXStatus_t          status      = DRX_STS_ERROR;
      pDRXFrequencyPlan_t  freqPlan    = (pDRXFrequencyPlan_t)(NULL);
      struct DRXChannel_t         scanChannel = { 0 };
      enum Bool_t               nextChannel = FALSE;

      freqPlan =
         &(commonAttr->scanParam->frequencyPlan[commonAttr->scanFreqPlanIndex]);

      /* Channel to scan */
      scanChannel.frequency      = commonAttr->scanNextFrequency;
      scanChannel.bandwidth      = freqPlan->bandwidth;
      scanChannel.mirror         = DRX_MIRROR_AUTO;
      scanChannel.constellation  = DRX_CONSTELLATION_AUTO;
      scanChannel.hierarchy      = DRX_HIERARCHY_AUTO;
      scanChannel.priority       = DRX_PRIORITY_HIGH;
      scanChannel.coderate       = DRX_CODERATE_AUTO;
      scanChannel.guard          = DRX_GUARD_AUTO;
      scanChannel.fftmode        = DRX_FFTMODE_AUTO;
      scanChannel.classification = DRX_CLASSIFICATION_AUTO;
      scanChannel.symbolrate     = 0;
      scanChannel.interleavemode = DRX_INTERLEAVEMODE_AUTO;
      scanChannel.ldpc           = DRX_LDPC_AUTO;
      scanChannel.interleaver    = DRX_INTERLEAVER_AUTO;
      scanChannel.carrier        = DRX_CARRIER_AUTO;
      scanChannel.framemode      = DRX_FRAMEMODE_AUTO;

      /* Call default or device-specific scanning function */
      status = (*(GetScanFunction( demod )))
                  ( demod, DRX_SCAN_COMMAND_NEXT, &scanChannel, &nextChannel );

      /* Proceed to next channel if requested */
      if ( nextChannel == TRUE )
      {
         enum DRXStatus_t nextStatus;

         /* Keep track of progress */
         (commonAttr->scanChannelsScanned) ++;
         *scanProgress = (u16_t)(((commonAttr->scanChannelsScanned)*
                              ((u32_t)(maxProgress)))/
                              (commonAttr->scanMaxChannels));

         nextStatus = ScanPrepareNextScan( demod );
         if ( nextStatus != DRX_STS_OK )
         {
            commonAttr->scanActive = FALSE;
            return (nextStatus);
         }
      }
      if ( status != DRX_STS_BUSY )
      {
         /* channel found or error */
         commonAttr->scanActive = FALSE;
         return status;
      }
   } /* for ( i = 0; i < numTries && !scanReady ; i++ ) */

   if ( (*scanReady) == TRUE )
   {
      /* End of scan reached: call stop-scan, ignore any error */
      CtrlScanStop( demod );
      commonAttr->scanActive = FALSE;
      return (DRX_STS_READY);
   }

   commonAttr->scanActive = FALSE;
   return (DRX_STS_BUSY);
}

#endif /* #ifndef DRX_EXCLUDE_SCAN */

/*============================================================================*/
/*============================================================================*/
/*===Microcode related functions==============================================*/
/*============================================================================*/
/*============================================================================*/

/**
* \brief Read a 16 bits word, expects big endian data.
* \param addr: Pointer to memory from which to read the 16 bits word.
* \return u16_t The data read.
*
* This function takes care of the possible difference in endianness between the
* host and the data contained in the microcode image file.
*
*/
static u16_t
UCodeRead16( pu8_t addr)
{
   /* Works fo any host processor */

   u16_t word=0;

   word = ((u16_t)addr[0]);
   word <<= 8;
   word |=((u16_t)addr[1]);

   return ( word );
}

/*============================================================================*/

/**
* \brief Read a 32 bits word, expects big endian data.
* \param addr: Pointer to memory from which to read the 32 bits word.
* \return u32_t The data read.
*
* This function takes care of the possible difference in endianness between the
* host and the data contained in the microcode image file.
*
*/
static u32_t
UCodeRead32( pu8_t addr)
{
   /* Works fo any host processor */

   u32_t word=0;

   word = ((u16_t)addr[0]);
   word <<= 8;
   word |= ((u16_t)addr[1]);
   word <<= 8;
   word |= ((u16_t)addr[2]);
   word <<= 8;
   word |= ((u16_t)addr[3]);

   return ( word );
}

/*============================================================================*/

/**
* \brief Compute CRC of block of microcode data.
* \param blockData: Pointer to microcode data.
* \param nrWords:   Size of microcode block (number of 16 bits words).
* \return u16_t The computed CRC residu.
*/
static u16_t
UCodeComputeCRC (pu8_t blockData, u16_t nrWords)
{
   u16_t i = 0;
   u16_t j = 0;
   u32_t CRCWord=0;
   u32_t carry=0;

   while (i < nrWords)
   {
      CRCWord |= (u32_t) UCodeRead16(blockData);
      for (j = 0; j < 16; j++)
      {
         CRCWord <<= 1;
         if (carry != 0)
         {
            CRCWord ^= 0x80050000UL;
         }
         carry = CRCWord & 0x80000000UL;
      }
      i++;
      blockData += (sizeof(u16_t));
   }
   return ((u16_t) (CRCWord >> 16));
}

/*============================================================================*/

/**
* \brief Handle microcode upload or verify.
* \param devAddr: Address of device.
* \param mcInfo:  Pointer to information about microcode data.
* \param action:  Either UCODE_UPLOAD or UCODE_VERIFY
* \return DRXStatus_t.
* \retval DRX_STS_OK:
*                    - In case of UCODE_UPLOAD: code is successfully uploaded.
*                    - In case of UCODE_VERIFY: image on device is equal to
*                      image provided to this control function.
* \retval DRX_STS_ERROR:
*                    - In case of UCODE_UPLOAD: I2C error.
*                    - In case of UCODE_VERIFY: I2C error or image on device
*                      is not equal to image provided to this control function.
* \retval DRX_STS_INVALID_ARG:
*                    - Invalid arguments.
*                    - Provided image is corrupt
*/
static enum DRXStatus_t
CtrlUCode(  pDRXDemodInstance_t demod,
            pDRXUCodeInfo_t  mcInfo,
            enum DRXUCodeAction_t action)
{
   u16_t  i = 0;
   u16_t  mcNrOfBlks = 0;
   u16_t  mcMagicWord = 0;
   pu8_t  mcData = (pu8_t)(NULL);
   pI2CDeviceAddr_t devAddr = (pI2CDeviceAddr_t)(NULL);

   devAddr = demod -> myI2CDevAddr;

   /* Check arguments */
   if (  ( mcInfo == NULL ) ||
         ( mcInfo->mcData == NULL ) )
   {
      return DRX_STS_INVALID_ARG;
   }

   mcData = mcInfo->mcData;

   /* Check data */
   mcMagicWord = UCodeRead16( mcData );
   mcData += sizeof( u16_t );
   mcNrOfBlks = UCodeRead16( mcData );
   mcData += sizeof( u16_t );

   if (  ( mcMagicWord != DRX_UCODE_MAGIC_WORD ) ||
         ( mcNrOfBlks == 0 ) )
   {
      /* wrong endianess or wrong data ? */
      return DRX_STS_INVALID_ARG;
   }

   /* Process microcode blocks */
   for( i = 0 ; i<mcNrOfBlks ; i++ )
   {
      struct DRXUCodeBlockHdr_t blockHdr;
      u16_t mcBlockNrBytes = 0;

      /* Process block header */
      blockHdr.addr = UCodeRead32( mcData );
      mcData += sizeof(u32_t);
      blockHdr.size = UCodeRead16( mcData );
      mcData += sizeof(u16_t);
      blockHdr.flags = UCodeRead16( mcData );
      mcData += sizeof(u16_t);
      blockHdr.CRC = UCodeRead16( mcData );
      mcData += sizeof(u16_t);

      /* Check block header on:
         - data larger then 64Kb
         - if CRC enabled check CRC
      */
      if (  ( blockHdr.size > 0x7FFF ) ||
            ( (( blockHdr.flags & DRX_UCODE_CRC_FLAG ) != 0) &&
               ( blockHdr.CRC != UCodeComputeCRC( mcData, blockHdr.size)) )
         )
      {
         /* Wrong data ! */
         return DRX_STS_INVALID_ARG;
      }

      mcBlockNrBytes = blockHdr.size * ((u16_t)sizeof(u16_t));

      if (blockHdr.size != 0 )
      {
         /* Perform the desired action */
         switch ( action ) {
            /*================================================================*/
            case UCODE_UPLOAD :
               {
                  /* Upload microcode */
                  if ( demod->myAccessFunct->writeBlockFunc(
                                    devAddr,
                                    (DRXaddr_t) blockHdr.addr,
                                    mcBlockNrBytes,
                                    mcData,
                                    0x0000) != DRX_STS_OK)
                  {
                     return (DRX_STS_ERROR);
                  } /* if */
               };
               break;

            /*================================================================*/
            case UCODE_VERIFY :
               {
                  int         result = 0;
                  u8_t        mcDataBuffer[DRX_UCODE_MAX_BUF_SIZE];
                  u32_t       bytesToCompare=0;
                  u32_t       bytesLeftToCompare=0;
                  DRXaddr_t   currAddr = (DRXaddr_t)0;
                  pu8_t       currPtr =NULL;

                  bytesLeftToCompare = mcBlockNrBytes;
                  currAddr           = blockHdr.addr;
                  currPtr            = mcData;

                  while( bytesLeftToCompare != 0 )
                  {
                     if (bytesLeftToCompare > ((u32_t)DRX_UCODE_MAX_BUF_SIZE) )
                     {
                        bytesToCompare = ((u32_t)DRX_UCODE_MAX_BUF_SIZE);
                     } else {
                        bytesToCompare = bytesLeftToCompare;
                     }
                     if ( demod->myAccessFunct->readBlockFunc(
                                    devAddr,
                                    currAddr,
                                    (u16_t)bytesToCompare,
                                    (pu8_t)mcDataBuffer,
                                    0x0000) != DRX_STS_OK)
                     {
                        return (DRX_STS_ERROR);
                     }

                     result = DRXBSP_HST_Memcmp(currPtr,
                                                mcDataBuffer,
                                                bytesToCompare);

                     if ( result != 0 )
                     {
                        return (DRX_STS_ERROR);
                     };

                     currAddr           += ((DRXaddr_t)(bytesToCompare/2));
                     currPtr            = &(currPtr[bytesToCompare]);
                     bytesLeftToCompare -= ((u32_t)bytesToCompare);
                  } /* while( bytesToCompare > DRX_UCODE_MAX_BUF_SIZE ) */
               };
               break;

            /*================================================================*/
            default:
               return DRX_STS_INVALID_ARG;
               break;

         } /* switch ( action ) */
      } /* if (blockHdr.size != 0 ) */

      /* Next block */
      mcData += mcBlockNrBytes;

   } /* for( i = 0 ; i<mcNrOfBlks ; i++ ) */

   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief Build list of version information.
* \param demod: A pointer to a demodulator instance.
* \param versionList: Pointer to linked list of versions.
* \return DRXStatus_t.
* \retval DRX_STS_OK:          Version information stored in versionList
* \retval DRX_STS_INVALID_ARG: Invalid arguments.
*/
static enum DRXStatus_t
CtrlVersion(   pDRXDemodInstance_t demod,
               pDRXVersionList_t   *versionList )
{
   static char drxDriverCoreModuleName[]  = "Core driver";
   static char drxDriverCoreVersionText[] =
         DRX_VERSIONSTRING( VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH );

   static struct DRXVersion_t drxDriverCoreVersion;
   static struct DRXVersionList_t drxDriverCoreVersionList;

   pDRXVersionList_t demodVersionList = (pDRXVersionList_t)(NULL);
   enum DRXStatus_t returnStatus = DRX_STS_ERROR;

   /* Check arguments */
   if ( versionList == NULL )
   {
      return ( DRX_STS_INVALID_ARG );
   }

   /* Get version info list from demod */
   returnStatus = (*(demod->myDemodFunct->ctrlFunc))(
                           demod,
                           DRX_CTRL_VERSION,
                           (void *) &demodVersionList );

   /* Always fill in the information of the driver SW . */
   drxDriverCoreVersion.moduleType  = DRX_MODULE_DRIVERCORE;
   drxDriverCoreVersion.moduleName  = drxDriverCoreModuleName;
   drxDriverCoreVersion.vMajor      = VERSION_MAJOR;
   drxDriverCoreVersion.vMinor      = VERSION_MINOR;
   drxDriverCoreVersion.vPatch      = VERSION_PATCH;
   drxDriverCoreVersion.vString     = drxDriverCoreVersionText;

   drxDriverCoreVersionList.version = &drxDriverCoreVersion;
   drxDriverCoreVersionList.next    = (pDRXVersionList_t)(NULL);

   if (( returnStatus == DRX_STS_OK ) && ( demodVersionList != NULL ))
   {
      /* Append versioninfo from driver to versioninfo from demod  */
      /* Return version info in "bottom-up" order. This way, multiple
         devices can be handled without using malloc. */
      pDRXVersionList_t currentListElement = demodVersionList;
      while ( currentListElement->next != NULL )
      {
         currentListElement = currentListElement->next;
      }
      currentListElement->next = &drxDriverCoreVersionList;

      *versionList = demodVersionList;
   }
   else
   {
      /* Just return versioninfo from driver */
      *versionList = &drxDriverCoreVersionList;
   }

   return (DRX_STS_OK);
}

/*============================================================================*/
/*============================================================================*/
/*== Exported functions ======================================================*/
/*============================================================================*/
/*============================================================================*/



/**
* \brief This function is obsolete.
* \param demods: Don't care, parameter is ignored.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK: Initialization completed.
*
* This function is obsolete, prototype available for backward compatability.
*
*/

enum DRXStatus_t
DRX_Init(  pDRXDemodInstance_t demods[]  )
{
   return DRX_STS_OK;
}

/*============================================================================*/

/**
* \brief This function is obsolete.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK: Terminated driver successful.
*
* This function is obsolete, prototype available for backward compatability.
*
*/

enum DRXStatus_t
DRX_Term( struct dvb_frontend *fe )
{
   return (DRX_STS_OK);
}

/*============================================================================*/

/**
* \brief Open a demodulator instance.
* \param demod: A pointer to a demodulator instance.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK:          Opened demod instance with succes.
* \retval DRX_STS_ERROR:       Driver not initialized or unable to initialize
*                              demod.
* \retval DRX_STS_INVALID_ARG: Demod instance has invalid content.
*
*/

enum DRXStatus_t
DRX_Open(pDRXDemodInstance_t demod)
{
   enum DRXStatus_t status = DRX_STS_OK;

   if (  ( demod == NULL )               ||
         ( demod->myDemodFunct == NULL ) ||
         ( demod->myCommonAttr == NULL ) ||
         ( demod->myExtAttr == NULL )    ||
         ( demod->myI2CDevAddr == NULL ) ||
         ( demod->myCommonAttr->isOpened == TRUE ))
   {
      //return (DRX_STS_INVALID_ARG);
   }

   status = (*(demod->myDemodFunct->openFunc))( demod );

   if ( status == DRX_STS_OK )
   {
      //printk("%s %d isOpened ok\n", __func__, __LINE__);	
      demod->myCommonAttr->isOpened = TRUE;
   }

   return ( status );
}

/*============================================================================*/

/**
* \brief Close device.
* \param demod: A pointer to a demodulator instance.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK:          Closed demod instance with succes.
* \retval DRX_STS_ERROR:       Driver not initialized or error during close
*                              demod.
* \retval DRX_STS_INVALID_ARG: Demod instance has invalid content.
*
* Free resources occupied by device instance.
* Put device into sleep mode.
*/

enum DRXStatus_t
DRX_Close(pDRXDemodInstance_t demod)
{
   enum DRXStatus_t status = DRX_STS_OK;

   if (  ( demod == NULL )               ||
         ( demod->myDemodFunct == NULL ) ||
         ( demod->myCommonAttr == NULL ) ||
         ( demod->myExtAttr == NULL )    ||
         ( demod->myI2CDevAddr == NULL ) ||
         ( demod->myCommonAttr->isOpened == FALSE ))
   {
      //printk("%s %d DRXK_Close return DRX_STS_INVALID_ARG\n", __func__, __LINE__);
      return (DRX_STS_INVALID_ARG);
   }

   status = (*(demod->myDemodFunct->closeFunc))( demod );

   demod->myCommonAttr->isOpened = FALSE;
   return ( status );
}

/*============================================================================*/

/**
* \brief Control the device.
* \param demod:    A pointer to a demodulator instance.
* \param ctrl:     Reference to desired control function.
* \param ctrlData: Pointer to data structure for control function.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK:                 Control function completed successful.
* \retval DRX_STS_ERROR:              Driver not initialized or error during
*                                     control demod.
* \retval DRX_STS_INVALID_ARG:        Demod instance or ctrlData has invalid
*                                     content.
* \retval DRX_STS_FUNC_NOT_AVAILABLE: Specified control function is not
*                                     available.
*
* Data needed or returned by the control function is stored in ctrlData.
*
*/

enum DRXStatus_t
DRX_Ctrl(pDRXDemodInstance_t demod, enum DRXCtrlIndex_t ctrl, void *ctrlData)
{
   enum DRXStatus_t status = DRX_STS_ERROR;

   if (  ( demod               == NULL ) ||
         ( demod->myDemodFunct == NULL ) ||
         ( demod->myCommonAttr == NULL ) ||
         ( demod->myExtAttr    == NULL ) ||
         ( demod->myI2CDevAddr == NULL )
      )
   {
      printk("%s %d DRX_STS_INVALID_ARG demod=%d myDemodFunct=%d myCommonAttr=%d myExAttr=%d myI2CDevAddr=%d \n",
	  	__func__, __LINE__, demod==NULL?0:1, demod->myDemodFunct == NULL?0:1, demod->myCommonAttr == NULL?0:1, 
	  	demod->myExtAttr    == NULL?0:1, demod->myI2CDevAddr==NULL?0:1);
      return (DRX_STS_INVALID_ARG);
   }

   if ( ( ( demod->myCommonAttr->isOpened == FALSE ) &&
            ( ctrl != DRX_CTRL_PROBE_DEVICE ) )
      )
   {
   	//printk("%s %d DRX_STS_INVALID_ARG\n", __func__, __LINE__);
      return (DRX_STS_INVALID_ARG);
   }

   if ( ( DRX_ISPOWERDOWNMODE( demod->myCommonAttr->currentPowerMode ) &&
            ( ctrl != DRX_CTRL_POWER_MODE )   &&
            ( ctrl != DRX_CTRL_PROBE_DEVICE ) &&
            ( ctrl != DRX_CTRL_NOP )
         )
      )
   {
   	//printk("%s %d DRX_STS_FUNC_NOT_AVAILABLE  \n", __func__, __LINE__);
      return (DRX_STS_FUNC_NOT_AVAILABLE);
   }

   /* Fixed control functions */
   switch ( ctrl ) {
      /*======================================================================*/
      case DRX_CTRL_NOP:
         /* No operation */
	
         return (DRX_STS_OK);
         break;

      /*======================================================================*/
      case DRX_CTRL_VERSION:
         {		
            return CtrlVersion( demod, (pDRXVersionList_t *) ctrlData );
         }
         break;

      /*======================================================================*/
      default :
         /* Do nothing */
         break;
   }

   /* Virtual functions */
   /* First try calling function from derived class */

   status = (*(demod->myDemodFunct->ctrlFunc))( demod, ctrl, ctrlData );
   if ( status == DRX_STS_FUNC_NOT_AVAILABLE )
   {
      /* Now try calling a the base class function */
  
      switch ( ctrl ) {
         /*===================================================================*/
         case DRX_CTRL_LOAD_UCODE:
            {
               return CtrlUCode (   demod,
                                    (pDRXUCodeInfo_t) ctrlData,
                                    UCODE_UPLOAD);
            }
            break;

         /*===================================================================*/
         case DRX_CTRL_VERIFY_UCODE:
            {
               return CtrlUCode (   demod,
                                    (pDRXUCodeInfo_t) ctrlData,
                                    UCODE_VERIFY);
            }
            break;

#ifndef DRX_EXCLUDE_SCAN
         /*===================================================================*/
         case DRX_CTRL_SCAN_INIT:
            {
               return CtrlScanInit( demod, (pDRXScanParam_t) ctrlData );
            }
            break;

         /*===================================================================*/
         case DRX_CTRL_SCAN_NEXT:
            {
               return CtrlScanNext( demod, (pu16_t) ctrlData );
            }
            break;

         /*===================================================================*/
         case DRX_CTRL_SCAN_STOP:
            {
               return CtrlScanStop( demod );
            }
            break;

         /*===================================================================*/
#endif /* #ifndef DRX_EXCLUDE_SCAN */

         default :
            return (DRX_STS_FUNC_NOT_AVAILABLE);
      }
   } else {      
      return (status);
   } /* if ( status == DRX_STS_FUNC_NOT_AVAILABLE ) */

   return (DRX_STS_OK);
}


/*============================================================================*/
enum DRXStatus_t MicronasSetTS(struct dvb_frontend *fe)
{
	enum DRXStatus_t status = DRX_STS_OK;
	struct drxk3913_state *state = fe->demodulator_priv;
	//struct drxk3913_config *config = state->config;

 /* If required, change MPEG output configuration */
	struct DRXCfgMPEGOutput_t  outputConfig;
	struct DRXCfg_t            config = { DRX_CFG_MPEG_OUTPUT, NULL };

	/* re-configure MPEG output */
	config.cfgData                = &outputConfig;
	outputConfig.enableMPEGOutput = TRUE;
	outputConfig.insertRSByte     = FALSE;
	outputConfig.enableParallel   = TRUE;
	outputConfig.invertDATA       = FALSE;
	outputConfig.invertERR        = FALSE;
	outputConfig.invertSTR        = FALSE;
	outputConfig.invertVAL        = FALSE;
	outputConfig.invertCLK        = FALSE;
	outputConfig.staticCLK        = FALSE;
	outputConfig.bitrate          = 32000000UL;

	if ( DRX_Ctrl( &state->demod, DRX_CTRL_SET_CFG, &config ) != DRX_STS_OK )
	{
		drxk_info( "failure: error re-configuring MPEG output\n" );
		return DRX_STS_ERROR;
	}
	drxk_info( "success: re-configured MPEG output\n" );
	
	return status;
}

enum DRXStatus_t MicronasSetUIO(pDRXDemodInstance_t demod)
   /* if required, use the user I/O pin (UIO) for output/input */
   /*    SAW_SW >>> DRX_UIO1                                   */
   /*    GPIO2  >>> DRX_UIO2                                   */
   /*    GPIO1  >>> DRX_UIO3                                   */
   /* This example code uses DRX_UIO1 (SAW_SW)                 */
{
	enum DRXStatus_t status = DRX_STS_OK;

	struct DRXUIOCfg_t  uioCfg  = { DRX_UIO1, DRX_UIO_MODE_READWRITE };
	struct DRXUIOData_t uioData = { DRX_UIO1, FALSE };

	/* Set UIO pin to read/write mode */
	if( DRX_Ctrl( demod, DRX_CTRL_UIO_CFG, &uioCfg ) != DRX_STS_OK )
	{
		drxk_info( "failure: error configuring UIO1 pin\n" );
		return DRX_STS_ERROR;
	}
	drxk_info( "success: configured UIO1 pin\n" );

	/* Set UIO pin to desired level (FALSE = LOW, TRUE = HIGH) */
	uioData.value = TRUE;
	if( DRX_Ctrl( demod, DRX_CTRL_UIO_WRITE, &uioData ) != DRX_STS_OK )
	{
		drxk_info( "failure: error setting UIO1 output value\n" );
		return DRX_STS_ERROR;
	}
	drxk_info( "success: UIO1 pin set to %d\n", uioData.value );

	/* Get value at the UIO pin (FALSE = LOW, TRUE = HIGH) */
	if( DRX_Ctrl( demod, DRX_CTRL_UIO_READ, &uioData ) != DRX_STS_OK )
	{
		drxk_info( "failure: error retrieving UIO1 input value\n" );
		return DRX_STS_ERROR;
	}
	drxk_info( "success: value at UIO1 pin is %d\n", uioData.value );
	return status;
}

enum DRXStatus_t MicronasSetAGC(pDRXDemodInstance_t demod)
 /* If required, change the RF-AGC configuration.
     set to auto control mode */
{
	enum DRXStatus_t status = DRX_STS_OK;
	
#if 0
	DRXKCfgAgc_t ifAgcCfg;
	/* IF AGC CONFIG */
	DRXCfg_t ifconfig = { (DRXCfgType_t)DRXK_CFG_AGC_IF, NULL };
	ifconfig.cfgData = &ifAgcCfg;
	ifAgcCfg.standard = DRX_STANDARD_DVBT;

	DRX_Ctrl( &pdx->m_demod, DRX_CTRL_GET_CFG, &ifconfig );

	ifAgcCfg.ctrlMode = DRXK_AGC_CTRL_OFF;
	DRX_Ctrl( &pdx->m_demod, DRX_CTRL_SET_CFG, &ifconfig );
	
	
	ifAgcCfg.ctrlMode = DRXK_AGC_CTRL_AUTO;
	//ifAgcCfg.outputLevel = 0x2600;//user fixed
	ifAgcCfg.cutOffCurrent = 0;
	ifAgcCfg.minOutputLevel = 0x800;
	ifAgcCfg.maxOutputLevel = 0xffff;
	ifAgcCfg.speed = 3;
	ifAgcCfg.top = 0x3000;
	DRX_Ctrl( &pdx->m_demod, DRX_CTRL_SET_CFG, &ifconfig );
#endif

	struct DRXKCfgAgc_t rfAgcCfg;
	/* IF AGC CONFIG */
	struct DRXCfg_t rfconfig = { (enum DRXCfgType_t)DRXK_CFG_AGC_RF, NULL };
	rfconfig.cfgData = &rfAgcCfg;
	rfAgcCfg.standard = DRX_STANDARD_DVBT;

	DRX_Ctrl( demod, DRX_CTRL_GET_CFG, &rfconfig );

	rfAgcCfg.ctrlMode = DRXK_AGC_CTRL_OFF;
	DRX_Ctrl( demod, DRX_CTRL_SET_CFG, &rfconfig );
	
	
	rfAgcCfg.ctrlMode = DRXK_AGC_CTRL_AUTO;
	//rfAgcCfg.outputLevel = 0x2000;//user fixed
	rfAgcCfg.cutOffCurrent = 4000;
	//rfAgcCfg.minOutputLevel = 0x800;
	//rfAgcCfg.maxOutputLevel = 0xffff;
	rfAgcCfg.speed = 3;
	rfAgcCfg.top = 9500;
	
	DRX_Ctrl( demod, DRX_CTRL_SET_CFG, &rfconfig );	
	#if 0
	if ( DRX_Ctrl( &pdx->m_demod, DRX_CTRL_SET_CFG, &ifconfig ) != DRX_STS_OK )
	{
		drxk_info( "failure: error re-configuring IF-AGC for DVB\n" );
		return DRX_STS_ERROR;
	}
	drxk_info( "success: re-configured IF-AGC for DVB\n" );
	#endif

	return status;	
}
bool MicronasCheckLock(pDRXDemodInstance_t demod, enum Bool_t bDelay)
{

	enum DRXLockStatus_t lockStatus;
	
	int cnt = 0;
	while(cnt++ < 75)//50
	{
		if ( DRX_Ctrl( demod, DRX_CTRL_LOCK_STATUS, &lockStatus ) != DRX_STS_OK )
		{			
			return FALSE;
		}

		/* check for breakouts */
		if ( lockStatus == DRX_LOCKED )
		{
			printk("^_^ ^_^ ^_^ ^_^  ^_^ drxk39xx has locked!!!\n");
			break; /* system locked */
		}
		if ( lockStatus == DRX_NEVER_LOCK )
		{
			printk( " @_@ @_@ @_@ @_@ @_@cannot lock to signal [%s]\n",
					DRX_STR_LOCKSTATUS( lockStatus ) );
			break; 
		}

		if(bDelay)
			msleep(20);		
		
	}

	if ( lockStatus != DRX_LOCKED )
	{
		printk( "cannot lock to signal [%s]\n",
					DRX_STR_LOCKSTATUS( lockStatus ) );
		return FALSE;
	}
	return TRUE;
}

enum DRXStatus_t MicronasSetTune(struct dvb_frontend *fe,
								enum DRXStandard_t     standard,
								struct DRXChannel_t      channel)
{
	enum DRXStatus_t status = DRX_STS_OK;
	struct drxk3913_state *state = fe->demodulator_priv;

	drxk_info("%s %d fe=%p\n", __func__, __LINE__, fe);
	/*-------------------------------------------------------------------------*/
	/*  Program channel and wait for lock                                 */
	/*-------------------------------------------------------------------------*/
	//DRXStandard_t curStandard = DRX_STANDARD_UNKNOWN;
	//status = DRX_Ctrl( &pdx->m_demod, DRX_CTRL_GET_STANDARD, &curStandard );
	//if(curStandard != standard)//dennis 2008-09-12, this may cause S3/S4 issue?
	{
		status = DRX_Ctrl( &state->demod, DRX_CTRL_SET_STANDARD, &standard );
		if(status != DRX_STS_OK )
		{
			drxk_info( "failure: error selecting standard %s\n",
			DRX_STR_STANDARD( standard ) );
			return DRX_STS_ERROR;
		}
	}


	drxk_info("set_tuner %s frequency=%d  bandwidth=%d\n", 
		(state->config->standard==9)?"DVBC":"DVBT", (int)channel.frequency, (int)channel.bandwidth);
	state->config->set_tuner(fe, channel.frequency*1000, (8-channel.bandwidth)*1000000, state->config->standard);

	
	/* program channel */
	if ( DRX_Ctrl( &state->demod, DRX_CTRL_SET_CHANNEL, &channel ) != DRX_STS_OK )
	{
		drxk_info( "failure: error programming desired channel\n" );
		return DRX_STS_ERROR;

	}
	drxk_info( "channel programmed... waiting for lock\n" );

	return status;
}								

enum DRXStatus_t MicronasInit(struct dvb_frontend *fe)
{
	enum DRXStatus_t status = DRX_STS_OK;
	struct drxk3913_state *state = fe->demodulator_priv;
	//struct drxk3913_config *config = state->config;
		
	state->demod               = DRXKDefaultDemod_g;
	memcpy(state->demod.myI2CDevAddr, &DRXKDefaultAddr_g, sizeof(struct I2CDeviceAddr_t));
	memcpy(state->demod.myCommonAttr, &DRXKDefaultCommAttr_g, sizeof(struct DRXCommonAttr_t));
	memcpy(state->demod.myExtAttr, &DRXKData_g, sizeof(struct DRXKData_t));

	state->demod.myTuner  = NULL;
	state->demod.myCommonAttr->intermediateFreq = 36125;
	((pDRXKData_t)state->demod.myExtAttr)->presetName = "Microtune MT2063";


	state->demod.myI2CDevAddr->i2cAddr   = 0x52;            /* i2c address */
	state->demod.myI2CDevAddr->i2cDevId  = 1;               /* device ID   */
	state->demod.myI2CDevAddr->userData = (void *)fe; /*attached device obj*/
				
	if(TRUE)
	{
		state->demod.myCommonAttr->microcode       = DRXK_MC_MAIN;
		state->demod.myCommonAttr->verifyMicrocode = FALSE;
	}
	//set antenna switch power
	//SetAntennaSwitchPower(device, ANTENNA_SWITCH_POWER_OFF);//set dvbt/c switch power off, power on only when dvbt channels 

	
	//pDRXVersionList_t versionList = NULL;
	
	if ( DRX_Ctrl( &state->demod, DRX_CTRL_PROBE_DEVICE, NULL ) != DRX_STS_OK )
	{
		drxk_info("!!!failure: no DRX-K detected!!!\r\n");
		return DRX_STS_ERROR;
	}
	
	//state->demod.myCommonAttr->isOpened = FALSE;//dennis add for issue s3/s4 wake up micronas reset error 
	if ( DRX_Open( &state->demod) != DRX_STS_OK )
	{
		drxk_info( "failure: error opening demodulator\r\n" );
		return DRX_STS_ERROR;
	}
#if 0 /* show version information */
	if ( DRX_Ctrl( device, &pdx->m_demod, DRX_CTRL_VERSION, &versionList ) != DRX_STS_OK )
	{
		drxk_info( "failure: error retrieving version information\n" );
		return DRX_STS_ERROR;
	}
	drxk_info( "\nVersion Info:\n" );
	while ( versionList != NULL )
	{
		pDRXVersion_t version = versionList->version;
		if (version)
		{
			drxk_info( "%s :  %s\n", version->moduleName, version->vString );
		}
		versionList = versionList->next;
	}
#endif
	//MicronasSetUIO(device);
	
	if(MicronasSetTS(fe) != DRX_STS_OK )
	{
		drxk_info( "failure: error Setting TS mode!\r\n" );
		return DRX_STS_ERROR;
	}
	
	//MicronasSetAGC(device);		
	return status;
}

enum DRXStatus_t MicronasClose(struct dvb_frontend *fe)
{
	//drxk_info("MicronasClose");
	struct drxk3913_state *state = fe->demodulator_priv;
	enum DRXStatus_t status = DRX_STS_OK;
	
	if ( DRX_Close( &state->demod) != DRX_STS_OK )
	{
		
		if(state->demod.myTuner != NULL)//Dennis add for S3/S4 issue
		{
			state->demod.myTuner->myCommonAttr->selfCheck = NULL;
		}
		drxk_info( "failure: error closing demodulator instance\n" );
	}
	//drxk_info("DRXK CLOSE END");
	/* terminate host bsp module */
	if ( DRXBSP_HST_Term() != DRX_STS_OK )
	{
		drxk_info( "failure: error terminating host bsp\n" );
	}
	/* disable I2C communication */
	if ( DRXBSP_I2C_Term() != DRX_STS_OK )
	{
		drxk_info( "failure: error terminating i2c\n" );
	}	
	state->demod.myCommonAttr->isOpened = FALSE;	
	//drxk_info("MicronasClose end");
	
	return status;
}
/*----------  micronas functions   ----------*/


static void drxk3913_release(struct dvb_frontend *fe)
{
	struct drxk3913_state *state = fe->demodulator_priv;

	//call MicronasClose
	//MicronasClose(fe);
	/* post process event */
	//drxk3913_postproc(state, STB0899_POSTPROC_GPIO_POWER, 0);
	if (state!=NULL)
	{
		kfree(state);
		state = NULL;
	}
}

static int drxk3913_init(struct dvb_frontend *fe)
{
	//struct drxk3913_state *state = fe->demodulator_priv;
	//struct drxk3913_config *config = state->config;
	//call MicronasInit
	if (MicronasInit(fe) != DRX_STS_OK)
		return -1;

	return 0;
}

int drxk3913_i2c_gate_ctrl(struct dvb_frontend *fe, int enable)
{	
	struct drxk3913_state *state = fe->demodulator_priv;
	return DRX_Ctrl(&state->demod,DRX_CTRL_I2C_BRIDGE,&enable);	
}

static enum dvbfe_algo drxk3913_frontend_algo(struct dvb_frontend *fe)
{
	return DVBFE_ALGO_CUSTOM;
}


static enum dvbfe_search drxk3913_search(struct dvb_frontend *fe, struct dvb_frontend_parameters *p)
{
	struct drxk3913_state *state		= fe->demodulator_priv;
	struct DRXChannel_t channel;

	//channel.frequency = 690000;
	channel.frequency = p->frequency/1000;
	
	if (state->config->standard == MTTUNEA_DVBT)
	{
		//case DRX_STANDARD_DVBT:
		//DVB_T
		
		/* set dvbt channel parameters */		
		channel.bandwidth = 0;//p->u.ofdm.bandwidth;
		
		channel.mirror         = DRX_MIRROR_AUTO;
		channel.constellation  = DRX_CONSTELLATION_AUTO;
		channel.hierarchy      = DRX_HIERARCHY_AUTO;
		channel.priority       = DRX_PRIORITY_HIGH;
		channel.coderate       = DRX_CODERATE_AUTO;
		channel.guard          = DRX_GUARD_AUTO;
		channel.fftmode        = DRX_FFTMODE_AUTO;
		channel.classification = DRX_CLASSIFICATION_AUTO;

		printk("########### DVB-T MicronasSetTune channel.frequency = %d,   channel.bandwidth = %d \n", 
				(int)channel.frequency, channel.bandwidth);
		//call Micronas_SetTune
		if (MicronasSetTune(fe, DRX_STANDARD_DVBT,channel) != DRX_STS_OK)
			return -1;
	}	
	else
	{
		//case DRX_STANDARD_ITU_A:
		//DVB_C	
		channel.symbolrate    = p->u.qam.symbol_rate; /* Symb/s */
		channel.bandwidth   = DRX_BANDWIDTH_8MHZ;
		switch(p->u.qam.modulation)
		{
			case QAM_16:
				channel.constellation  = DRX_CONSTELLATION_QAM16;
				break;
			case QAM_32:
				channel.constellation  = DRX_CONSTELLATION_QAM32;
				break;
			case QAM_64:
				channel.constellation  = DRX_CONSTELLATION_QAM64;
				break;
			case QAM_128:
				channel.constellation  = DRX_CONSTELLATION_QAM128;
				break;
			case QAM_256:
				channel.constellation  = DRX_CONSTELLATION_QAM256;
				break;
			default:
				channel.constellation  = DRX_CONSTELLATION_AUTO;
				break;
		}
		
		channel.mirror         = DRX_MIRROR_AUTO;
		channel.hierarchy      = DRX_HIERARCHY_AUTO;
		channel.priority       = DRX_PRIORITY_HIGH;
		channel.coderate       = DRX_CODERATE_AUTO;
		channel.guard          = DRX_GUARD_AUTO;
		channel.fftmode        = DRX_FFTMODE_AUTO;
		channel.classification = DRX_CLASSIFICATION_AUTO;

		printk("########### DVBC  MicronasSetTune channel.frequency = %d,   channel.bandwidth = %d \n", 
				(int)channel.frequency, channel.bandwidth);
		//call Micronas_SetTune
		if (MicronasSetTune(fe, DRX_STANDARD_ITU_A,channel) != DRX_STS_OK)
			return -1;
	}

	return 0;
}

static int drxk3913_track(struct dvb_frontend *fe, struct dvb_frontend_parameters *p)
{
	return 0;
}

static int drxk3913_get_frontend(struct dvb_frontend *fe, struct dvb_frontend_parameters *p)
{
	//struct drxk3913_state *state		= fe->demodulator_priv;

	//p->u.qpsk.symbol_rate = internal->srate;

	return 0;
}

static int drxk3913_read_status(struct dvb_frontend *fe, enum fe_status *status)
{
	struct drxk3913_state *state		= fe->demodulator_priv;

	if (MicronasCheckLock(&state->demod, TRUE))
	{
		*status = FE_HAS_SIGNAL | FE_HAS_CARRIER | FE_HAS_VITERBI | FE_HAS_SYNC | FE_HAS_LOCK;
	}
	//printk("============== leave drxk3913_read_status=%02x\n", *status);
	return 0;
}

static int drxk3913_read_snr(struct dvb_frontend *fe, u16 *snr)
{
	struct drxk3913_state *state		= fe->demodulator_priv;
	struct DRXSigQuality_t sigQuality;

	if( DRX_STS_OK == DRX_Ctrl( &state->demod, DRX_CTRL_SIG_QUALITY, &sigQuality))
	{
		*snr = sigQuality.indicator;		
	}
	//printk("============== leave drxk3913_read_snr = %d\n", *snr);
	return 0;
}

static int drxk3913_read_signal_strength(struct dvb_frontend *fe, u16 *strength)
{
	struct drxk3913_state *state		= fe->demodulator_priv;
	
	DRX_Ctrl( &state->demod, DRX_CTRL_SIG_STRENGTH, strength);	
	//printk("============== leave drxk3913_read_signal_strength=%d\n", *strength);
	return 0;
}

static int drxk3913_read_ber(struct dvb_frontend *fe, u32 *ber)
{
	struct drxk3913_state *state		= fe->demodulator_priv;
	struct DRXSigQuality_t sigQuality;
	
	if( DRX_STS_OK == DRX_Ctrl( &state->demod, DRX_CTRL_SIG_QUALITY, &sigQuality))
	{
		*ber = sigQuality.scaleFactorBER;
	}
	//printk("============== leave drxk3913_read_ber=%d\n", *ber);
	return 0;
}

static int drxk3913_wakeup(struct dvb_frontend *fe)
{
	//struct drxk3913_state *state = fe->demodulator_priv;

	return 0;
}

static int drxk3913_sleep(struct dvb_frontend *fe)
{
	//struct drxk3913_state *state = fe->demodulator_priv;

	return 0;
}

static struct dvb_frontend_ops drxk3913_ops_dvbt = {

	.info = {
		.name 			= "DRXK3913 Multistandard DVB_T",
		.type 			= FE_OFDM,
		.frequency_min		= 174000000,
		.frequency_max		= 862000000,
		.frequency_stepsize	= 166667,
		.frequency_tolerance	= 0,
		.caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 |
			FE_CAN_FEC_3_4 | FE_CAN_FEC_5_6 | FE_CAN_FEC_7_8 |
			FE_CAN_FEC_AUTO |
			FE_CAN_QPSK | FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_QAM_AUTO |
			FE_CAN_TRANSMISSION_MODE_AUTO | FE_CAN_GUARD_INTERVAL_AUTO |
			FE_CAN_HIERARCHY_AUTO | FE_CAN_RECOVER |
			FE_CAN_MUTE_TS
	},

	.release			= drxk3913_release,
	.init				= drxk3913_init,
	.sleep				= drxk3913_sleep,
//	.wakeup				= stb0899_wakeup,

	.i2c_gate_ctrl			= drxk3913_i2c_gate_ctrl,
	.search				= drxk3913_search,
	.get_frontend_algo		= drxk3913_frontend_algo,
	.search				= drxk3913_search,
	.track				= drxk3913_track,
	.get_frontend			= drxk3913_get_frontend,


	.read_status			= drxk3913_read_status,
	.read_snr			= drxk3913_read_snr,
	.read_signal_strength		= drxk3913_read_signal_strength,
	.read_ber			= drxk3913_read_ber,
};

static struct dvb_frontend_ops drxk3913_ops_dvbc = {

	.info = {
		.name 			= "DRXK3913 Multistandard DVB_C",
		.type = FE_QAM,
		.frequency_stepsize = 62500,
		.frequency_min = 47000000,
		.frequency_max = 862000000,
		.caps = FE_CAN_QAM_16 |
			FE_CAN_QAM_32 |
			FE_CAN_QAM_64 |
			FE_CAN_QAM_128 |
			FE_CAN_QAM_256 |
			FE_CAN_FEC_AUTO
	},

	.release			= drxk3913_release,
	.init				= drxk3913_init,
	.sleep				= drxk3913_sleep,
//	.wakeup				= stb0899_wakeup,

	.i2c_gate_ctrl			= drxk3913_i2c_gate_ctrl,
	.search				= drxk3913_search,
	.get_frontend_algo		= drxk3913_frontend_algo,
	.search				= drxk3913_search,
	.track				= drxk3913_track,
	.get_frontend			= drxk3913_get_frontend,


	.read_status			= drxk3913_read_status,
	.read_snr			= drxk3913_read_snr,
	.read_signal_strength		= drxk3913_read_signal_strength,
	.read_ber			= drxk3913_read_ber,
};

struct dvb_frontend *drxk3913_attach(struct drxk3913_config *config, struct i2c_adapter *i2c)
{
	struct drxk3913_state *state = NULL;

	state = kzalloc(sizeof (struct drxk3913_state), GFP_KERNEL);
	if (state == NULL)
		goto error;

	//inversion				= config->inversion;
	state->verbose				= &verbose;
	state->config				= config;
	state->i2c				= i2c;
	if (config->standard==MTTUNEA_DVBT)
	{
		printk("state->frontend.ops = drxk3913_ops_dvbt\n");
		state->frontend.ops = drxk3913_ops_dvbt;
	}
	else
	{
		printk("state->frontend.ops = drxk3913_ops_dvbc\n");
		state->frontend.ops = drxk3913_ops_dvbc;
	}
	state->frontend.demodulator_priv	= state;
	//state->internal.inversion		= inversion;

	drxk3913_wakeup(&state->frontend);
		
	return &state->frontend;

error:	
	return NULL;
}
EXPORT_SYMBOL(drxk3913_attach);
MODULE_PARM_DESC(verbose, "Set Verbosity level");
MODULE_AUTHOR("Henry Wang");
MODULE_DESCRIPTION("DRXK3913 Multi-Std frontend");
MODULE_LICENSE("GPL");



/* END OF FILE */
