#ifndef __DRXDRIVER_H__
#define __DRXDRIVER_H__
/*-------------------------------------------------------------------------
INCLUDES
-------------------------------------------------------------------------*/
#include <linux/dvb/frontend.h>
#include "drxk_type.h"
#include "dvb_frontend.h"
#include "mt2063.h"

//context of BSP_HOST.h		<Henry>		======================================
//#################################################################
//=================================================================
/*-------------------------------------------------------------------------
Exported FUNCTIONS
-------------------------------------------------------------------------*/
enum  DRXStatus_t DRXBSP_HST_Init( void );

enum  DRXStatus_t DRXBSP_HST_Term( void );

void* DRXBSP_HST_Memcpy( void *to, void *from, u32_t n);

int DRXBSP_HST_Memcmp( void *s1, void *s2, u32_t n);

//u32_t DRXBSP_HST_Clock( void );

enum  DRXStatus_t DRXBSP_HST_Sleep( u32_t n );

//end of BSP_HOST.h
//=================================================================
//#################################################################
//=================================================================

//context of BSP_I2C.h		<Henry>		======================================
//#################################################################
//=================================================================
/**
* \typedef I2Caddr_t
* \brief I2C device address (7-bit or 10-bit)
*/
typedef u16_t I2Caddr_t;

/**
* \typedef I2CdevId_t
* \brief Device identifier.
*
* The device ID can be useful if several devices share an I2C address,
* or if multiple I2C busses are used.
* It can be used to control a "switch" selecting the correct device and/or
* I2C bus.
*
*/
typedef u16_t I2CdevId_t;

/**
* \struct _I2CDeviceAddr_t
* \brief I2C device parameters.
*
* This structure contains the I2C address, the device ID and a userData pointer.
* The userData pointer can be used for application specific purposes.
*
*/
struct I2CDeviceAddr_t {
   I2Caddr_t  i2cAddr;        /**< The I2C address of the device. */
   I2CdevId_t i2cDevId;       /**< The device identifier. */
   void       *userData;      /**< User data pointer */
};

/**
* \typedef I2CDeviceAddr_t
* \brief I2C device parameters.
*
* This structure contains the I2C address and the device ID.
*
*/

/**
* \typedef pI2CDeviceAddr_t
* \brief Pointer to I2C device parameters.
*/
typedef struct I2CDeviceAddr_t *pI2CDeviceAddr_t;

/*-----------------------------------------------------------------------------
DEFINES
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
MACROS
------------------------------------------------------------------------------*/

/**
* \def IS_I2C_10BIT( addr )
* \brief Determine if I2C address 'addr' is a 10 bits address or not.
* \param addr The I2C address.
* \return int.
* \retval 0 if address is not a 10 bits I2C address.
* \retval 1 if address is a 10 bits I2C address.
*/
#define IS_I2C_10BIT(addr) \
        (((addr) & 0xF8) == 0xF0)

/*------------------------------------------------------------------------------
ENUM
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
STRUCTS
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
Exported FUNCTIONS
------------------------------------------------------------------------------*/


/**
* \fn DRXBSP_I2C_Init()
* \brief Initialize I2C communication module.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Initialization successful.
* \retval DRX_STS_ERROR Initialization failed.
*/
enum  DRXStatus_t DRXBSP_I2C_Init( void );


/**
* \fn DRXBSP_I2C_Term()
* \brief Terminate I2C communication module.
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Termination successful.
* \retval DRX_STS_ERROR Termination failed.
*/
enum  DRXStatus_t DRXBSP_I2C_Term( void );

/**
* \fn DRXStatus_t DRXBSP_I2C_WriteRead( pI2CDeviceAddr_t wDevAddr,
                                        u16_t wCount,
                                        pu8_t wData,
                                        pI2CDeviceAddr_t rDevAddr,
                                        u16_t rCount,
                                        pu8_t rData)
* \brief Read and/or write count bytes from I2C bus, store them in data[].
* \param wDevAddr The device i2c address and the device ID to write to
* \param wCount   The number of bytes to write
* \param wData    The array to write the data to
* \param rDevAddr The device i2c address and the device ID to read from
* \param rCount   The number of bytes to read
* \param rData    The array to read the data from
* \return DRXStatus_t Return status.
* \retval DRX_STS_OK Succes.
* \retval DRX_STS_ERROR Failure.
* \retval DRX_STS_INVALID_ARG Parameter 'wcount' is not zero but parameter
*                                       'wdata' contains NULL.
*                                       Idem for 'rcount' and 'rdata'.
*                                       Both wDevAddr and rDevAddr are NULL.
*
* This function must implement an atomic write and/or read action on the I2C bus
* No other process may use the I2C bus when this function is executing.
* The critical section of this function runs from and including the I2C
* write, up to and including the I2C read action.
*
* The device ID can be useful if several devices share an I2C address.
* It can be used to control a "switch" on the I2C bus to the correct device.
*/
enum  DRXStatus_t DRXBSP_I2C_WriteRead( pI2CDeviceAddr_t wDevAddr,
                                  u16_t            wCount,
                                  pu8_t            wData,
                                  pI2CDeviceAddr_t rDevAddr,
                                  u16_t            rCount,
                                  pu8_t            rData);


/**
* \fn DRXBSP_I2C_ErrorText()
* \brief Returns a human readable error.
* Counter part of numerical DRX_I2C_Error_g.
*
* \return char* Pointer to human readable error text.
*/
char* DRXBSP_I2C_ErrorText( void );

/**
* \var DRX_I2C_Error_g;
* \brief I2C specific error codes, platform dependent.
*/
extern int DRX_I2C_Error_g;
//end of BSP_I2C.h
//================================================================
//#################################################################
//=================================================================


//context of drx_dap_fasi.h		<Henry>		===================================
//#################################################################
//=================================================================
/* set default */
#if !defined( DRXDAPFASI_LONG_ADDR_ALLOWED )
#define  DRXDAPFASI_LONG_ADDR_ALLOWED 1
#endif

/* set default */
#if !defined( DRXDAPFASI_SHORT_ADDR_ALLOWED )
#define  DRXDAPFASI_SHORT_ADDR_ALLOWED 1
#endif

/* check */
#if ( ( DRXDAPFASI_LONG_ADDR_ALLOWED==0 ) && \
      ( DRXDAPFASI_SHORT_ADDR_ALLOWED==0 ) )
#error  At least one of short- or long-addressing format must be allowed.
*;   /* illegal statement to force compiler error */
#endif


/* set default */
#ifndef DRXDAP_SINGLE_MASTER
#define DRXDAP_SINGLE_MASTER 1
#endif


/* set default */
#if !defined( DRXDAP_MAX_WCHUNKSIZE)
#define  DRXDAP_MAX_WCHUNKSIZE 60
#endif

/* check */
#if ( (DRXDAPFASI_LONG_ADDR_ALLOWED==0)&&(DRXDAPFASI_SHORT_ADDR_ALLOWED==1) )
#if DRXDAP_SINGLE_MASTER
#define  DRXDAP_MAX_WCHUNKSIZE_MIN 3
#else
#define  DRXDAP_MAX_WCHUNKSIZE_MIN 5
#endif
#else
#if DRXDAP_SINGLE_MASTER
#define  DRXDAP_MAX_WCHUNKSIZE_MIN 5
#else
#define  DRXDAP_MAX_WCHUNKSIZE_MIN 7
#endif
#endif

#if  DRXDAP_MAX_WCHUNKSIZE <  DRXDAP_MAX_WCHUNKSIZE_MIN
#if ( (DRXDAPFASI_LONG_ADDR_ALLOWED==0)&&(DRXDAPFASI_SHORT_ADDR_ALLOWED==1) )
#if DRXDAP_SINGLE_MASTER
#error  DRXDAP_MAX_WCHUNKSIZE must be at least 3 in single master mode
*;   /* illegal statement to force compiler error */
#else
#error  DRXDAP_MAX_WCHUNKSIZE must be at least 5 in multi master mode
*;   /* illegal statement to force compiler error */
#endif
#else
#if DRXDAP_SINGLE_MASTER
#error  DRXDAP_MAX_WCHUNKSIZE must be at least 5 in single master mode
*;   /* illegal statement to force compiler error */
#else
#error  DRXDAP_MAX_WCHUNKSIZE must be at least 7 in multi master mode
*;   /* illegal statement to force compiler error */
#endif
#endif
#endif

/* set default */
#if !defined( DRXDAP_MAX_RCHUNKSIZE)
#define  DRXDAP_MAX_RCHUNKSIZE 254
#endif

/* check */
#if  DRXDAP_MAX_RCHUNKSIZE < 2
#error  DRXDAP_MAX_RCHUNKSIZE must be at least 2
*;   /* illegal statement to force compiler error */
#endif

/* check */
#if  DRXDAP_MAX_RCHUNKSIZE & 1
#error  DRXDAP_MAX_RCHUNKSIZE must be even
*;   /* illegal statement to force compiler error */
#endif

/*-------- Public API functions ---------------------------------------------*/

extern struct DRXAccessFunc_t drxDapFASIFunct_g;

#define DRXDAP_FASI_RMW           0x10000000
#define DRXDAP_FASI_BROADCAST     0x20000000
#define DRXDAP_FASI_CLEARCRC      0x80000000
#define DRXDAP_FASI_SINGLE_MASTER 0xC0000000
#define DRXDAP_FASI_MULTI_MASTER  0x40000000
#define DRXDAP_FASI_SMM_SWITCH    0x40000000    /* single/multi master switch */
#define DRXDAP_FASI_MODEFLAGS     0xC0000000
#define DRXDAP_FASI_FLAGS         0xF0000000

#define DRXDAP_FASI_ADDR2BLOCK( addr )  (((addr)>>22)&0x3F)
#define DRXDAP_FASI_ADDR2BANK( addr )   (((addr)>>16)&0x3F)
#define DRXDAP_FASI_ADDR2OFFSET( addr ) ((addr)&0x7FFF)

#define DRXDAP_FASI_SHORT_FORMAT( addr )     (((addr)& 0xFC30FF80)==0)
#define DRXDAP_FASI_LONG_FORMAT( addr )      (((addr)& 0xFC30FF80)!=0)
#define DRXDAP_FASI_OFFSET_TOO_LARGE( addr ) (((addr)& 0x00008000)!=0)
//end of drx_dsp_fasi.h
//=================================================================
//#################################################################
//=================================================================


//context of BSB_tuner.h	<Henry>==========================================
//#################################################################
//=================================================================
   /* Sub-mode bits should be adjacent and incremental */
#define TUNER_MODE_SUB0 0x0001    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB1 0x0002    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB2 0x0004    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB3 0x0008    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB4 0x0010    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB5 0x0020    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB6 0x0040    /* for sub-mode (e.g. RF-AGC setting)    */
#define TUNER_MODE_SUB7 0x0080    /* for sub-mode (e.g. RF-AGC setting)    */

#define TUNER_MODE_DIGITAL 0x0100 /* for digital channel (e.g. DVB-T)      */
#define TUNER_MODE_ANALOG 0x0200  /* for analog channel  (e.g. PAL)        */
#define TUNER_MODE_SWITCH 0x0400  /* during channel switch & scanning      */
#define TUNER_MODE_LOCK 0x0800    /* after tuner has locked                */
#define TUNER_MODE_6MHZ 0x1000    /* for 6MHz bandwidth channels           */
#define TUNER_MODE_7MHZ 0x2000    /* for 7MHz bandwidth channels           */
#define TUNER_MODE_8MHZ 0x4000    /* for 8MHz bandwidth channels           */

#define TUNER_MODE_SUB_MAX  8
#define TUNER_MODE_SUBALL   ( TUNER_MODE_SUB0 | TUNER_MODE_SUB1 | \
                              TUNER_MODE_SUB2 | TUNER_MODE_SUB3 | \
                              TUNER_MODE_SUB4 | TUNER_MODE_SUB5 | \
                              TUNER_MODE_SUB6 | TUNER_MODE_SUB7 )

/*------------------------------------------------------------------------------
TYPEDEFS
------------------------------------------------------------------------------*/

typedef u32_t TUNERMode_t;
typedef pu32_t pTUNERMode_t;

typedef char*           TUNERSubMode_t;    /* description of submode */
typedef TUNERSubMode_t *pTUNERSubMode_t;


enum TUNERLockStatus_t{

   TUNER_LOCKED,
   TUNER_NOT_LOCKED

};
typedef enum TUNERLockStatus_t* pTUNERLockStatus_t;


struct TUNERCommonAttr_t{

   char           *name;         /* Tuner brand & type name */
   DRXFrequency_t minFreqRF;     /* Lowest  RF input frequency, in kHz */
   DRXFrequency_t maxFreqRF;     /* Highest RF input frequency, in kHz */

   u8_t            subMode;             /* Index to sub-mode in use */
   pTUNERSubMode_t subModeDescriptions; /* Pointer to description of sub-modes*/
   u8_t            subModes;            /* Number of available sub-modes      */

   /* The following fields will be either 0, NULL or FALSE and do not need
      initialisation */
   void           *selfCheck;     /* gives proof of initialization  */
   enum Bool_t         programmed;     /* only valid if selfCheck is OK  */
   DRXFrequency_t RFfrequency;    /* only valid if programmed       */
   DRXFrequency_t IFfrequency;    /* only valid if programmed       */

   void*          myUserData;     /* pointer to associated demod instance */
   u16_t          myCapabilities; /* value for storing application flags  */
};
typedef struct TUNERCommonAttr_t* pTUNERCommonAttr_t;

/*
* Generic functions for DRX devices.
*/
typedef struct TUNERInstance_t *pTUNERInstance_t;

typedef enum DRXStatus_t (*TUNEROpenFunc_t)  (pTUNERInstance_t tuner);
typedef enum DRXStatus_t (*TUNERCloseFunc_t) (pTUNERInstance_t tuner);
typedef enum DRXStatus_t (*TUNERSetFrequencyFunc_t)  (pTUNERInstance_t tuner,
                                                 TUNERMode_t      mode,
                                                 DRXFrequency_t   frequency);
typedef enum DRXStatus_t (*TUNERGetFrequencyFunc_t)  (pTUNERInstance_t tuner,
                                                 pDRXFrequency_t  RFfrequency,
                                                 pDRXFrequency_t  IFfrequency);
typedef enum DRXStatus_t (*TUNERLockStatusFunc_t)( pTUNERInstance_t   tuner,
                                              pTUNERLockStatus_t lockStat );

typedef enum DRXStatus_t (*TUNERi2cWriteReadFunc_t)( pTUNERInstance_t   tuner,
                                                pI2CDeviceAddr_t wDevAddr,
                                                u16_t            wCount,
                                                pu8_t            wData,
                                                pI2CDeviceAddr_t rDevAddr,
                                                u16_t            rCount,
                                                pu8_t            rData);

struct TUNERFunc_t
{
   TUNEROpenFunc_t         openFunc;
   TUNERCloseFunc_t        closeFunc;
   TUNERSetFrequencyFunc_t setFrequencyFunc;
   TUNERGetFrequencyFunc_t getFrequencyFunc;
   TUNERLockStatusFunc_t   lockStatusFunc;
   TUNERi2cWriteReadFunc_t i2cWriteReadFunc;

};
typedef struct TUNERFunc_t* pTUNERFunc_t;

struct TUNERInstance_t {

   struct I2CDeviceAddr_t      myI2CDevAddr;
   pTUNERCommonAttr_t   myCommonAttr;
   void*                myExtAttr;
   pTUNERFunc_t         myFunct;

};


/*------------------------------------------------------------------------------
ENUM
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
STRUCTS
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
Exported FUNCTIONS
------------------------------------------------------------------------------*/

enum DRXStatus_t DRXBSP_TUNER_Open ( pTUNERInstance_t tuner );

enum DRXStatus_t DRXBSP_TUNER_Close( pTUNERInstance_t tuner );

enum DRXStatus_t DRXBSP_TUNER_SetFrequency( pTUNERInstance_t tuner,
                                       TUNERMode_t      mode,
                                       DRXFrequency_t   frequency );

enum DRXStatus_t DRXBSP_TUNER_GetFrequency( pTUNERInstance_t tuner,
                                       pDRXFrequency_t  RFfrequency,
                                       pDRXFrequency_t  IFfrequency );

enum DRXStatus_t DRXBSP_TUNER_LockStatus( pTUNERInstance_t   tuner,
                                     pTUNERLockStatus_t lockStat );


enum DRXStatus_t DRXBSP_TUNER_DefaultI2CWriteRead( pTUNERInstance_t tuner,
                                              pI2CDeviceAddr_t wDevAddr,
                                              u16_t            wCount,
                                              pu8_t            wData,
                                              pI2CDeviceAddr_t rDevAddr,
                                              u16_t            rCount,
                                              pu8_t            rData);


//end of BSB_tuner.h
//=================================================================
//#################################################################
//=================================================================


/*-------------------------------------------------------------------------
TYPEDEFS
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
DEFINES
-------------------------------------------------------------------------*/

/**************
*
* This section configures the DRX Data Access Protocols (DAPs).
*
**************/

/**
* \def DRXDAP_SINGLE_MASTER
* \brief Enable I2C single or I2C multimaster mode on host.
*
* Set to 1 to enable single master mode
* Set to 0 to enable multi master mode
*
* The actual DAP implementation may be restricted to only one of the modes.
* A compiler warning or error will be generated if the DAP implementation
* overides or cannot handle the mode defined below.
*
*/
#ifndef DRXDAP_SINGLE_MASTER
#define DRXDAP_SINGLE_MASTER 1
#endif

/**
* \def DRXDAP_MAX_WCHUNKSIZE
* \brief Defines maximum chunksize of an i2c write action by host.
*
* This indicates the maximum size of data the I2C device driver is able to
* write at a time. This includes I2C device address and register addressing.
*
* This maximum size may be restricted by the actual DAP implementation.
* A compiler warning or error will be generated if the DAP implementation
* overides or cannot handle the chunksize defined below.
*
* Beware that the DAP uses  DRXDAP_MAX_WCHUNKSIZE to create a temporary data
* buffer. Do not undefine or choose too large, unless your system is able to
* handle a stack buffer of that size.
*
*/
#ifndef DRXDAP_MAX_WCHUNKSIZE
//#define  DRXDAP_MAX_WCHUNKSIZE 60   //have defined before
#endif

/**
* \def DRXDAP_MAX_RCHUNKSIZE
* \brief Defines maximum chunksize of an i2c read action by host.
*
* This indicates the maximum size of data the I2C device driver is able to read
* at a time. Minimum value is 2. Also, the read chunk size must be even.
*
* This maximum size may be restricted by the actual DAP implementation.
* A compiler warning or error will be generated if the DAP implementation
* overides or cannot handle the chunksize defined below.
*
*/
#ifndef DRXDAP_MAX_RCHUNKSIZE
#define  DRXDAP_MAX_RCHUNKSIZE 60
#endif

/**************
*
* This section describes drxdriver defines.
*
**************/

/**
* \def DRX_UNKNOWN
* \brief Generic UNKNOWN value for DRX enumerated types.
*
* Used to indicate that the parameter value is unknown or not yet initalized.
*/
#ifndef DRX_UNKNOWN
#define DRX_UNKNOWN (254)
#endif

/**
* \def DRX_AUTO
* \brief Generic AUTO value for DRX enumerated types.
*
* Used to instruct the driver to automatically determine the value of the
* parameter.
*/
#ifndef DRX_AUTO
#define DRX_AUTO    (255)
#endif

/*-------------------------------------------------------------------------
MACROS
-------------------------------------------------------------------------*/
/* Macros to stringify the version number */
#define DRX_VERSIONSTRING( MAJOR, MINOR, PATCH ) \
         DRX_VERSIONSTRING_HELP(MAJOR)"." \
         DRX_VERSIONSTRING_HELP(MINOR)"." \
         DRX_VERSIONSTRING_HELP(PATCH)
#define DRX_VERSIONSTRING_HELP( NUM ) #NUM

/*-------------------------------------------------------------------------
ENUM
-------------------------------------------------------------------------*/

/**
* \enum DRXStandard_t
* \brief Modulation standards.
*/
enum DRXStandard_t{
   DRX_STANDARD_DVBT    = 0,           /**< Terrestrial DVB-T.               */
   DRX_STANDARD_8VSB,                  /**< Terrestrial 8VSB.                */
   DRX_STANDARD_NTSC,                  /**< Terrestrial\Cable analog NTSC.   */
   DRX_STANDARD_PAL_SECAM_BG,          /**< Terrestrial analog PAL/SECAM B/G */
   DRX_STANDARD_PAL_SECAM_DK,          /**< Terrestrial analog PAL/SECAM D/K */
   DRX_STANDARD_PAL_SECAM_I,           /**< Terrestrial analog PAL/SECAM I   */
   DRX_STANDARD_PAL_SECAM_L,           /**< Terrestrial analog PAL/SECAM L
                                             with negative modulation        */
   DRX_STANDARD_PAL_SECAM_LP,          /**< Terrestrial analog PAL/SECAM L
                                             with positive modulation        */
   DRX_STANDARD_ITU_A,                 /**< Cable ITU ANNEX A.               */
   DRX_STANDARD_ITU_B,                 /**< Cable ITU ANNEX B.               */
   DRX_STANDARD_ITU_C,                 /**< Cable ITU ANNEX C.               */
   DRX_STANDARD_ITU_D,                 /**< Cable ITU ANNEX D.               */
   DRX_STANDARD_FM,                    /**< Terrestrial\Cable FM radio       */
   DRX_STANDARD_DTMB,                  /**< Terrestrial DTMB standard (China)*/
   DRX_STANDARD_UNKNOWN = DRX_UNKNOWN, /**< Standard unknown.                */
   DRX_STANDARD_AUTO    = DRX_AUTO     /**< Autodetect standard.             */
};
typedef enum DRXStandard_t* pDRXStandard_t;


/**
* \enum DRXStandard_t
* \brief Modulation sub-standards.
*/
enum DRXSubstandard_t{
   DRX_SUBSTANDARD_MAIN = 0,              /**< Main subvariant of standard   */
   DRX_SUBSTANDARD_ATV_BG_SCANDINAVIA,
   DRX_SUBSTANDARD_ATV_DK_POLAND,
   DRX_SUBSTANDARD_ATV_DK_CHINA,
   DRX_SUBSTANDARD_UNKNOWN = DRX_UNKNOWN, /**< Sub-standard unknown.         */
   DRX_SUBSTANDARD_AUTO    = DRX_AUTO     /**< Auto (default) sub-standard   */
};
typedef enum DRXSubstandard_t* pDRXSubstandard_t;

/**
* \enum DRXBandwidth_t
* \brief Channel bandwidth or channel spacing.
*/
enum DRXBandwidth_t{
   DRX_BANDWIDTH_8MHZ    = 0,             /**< Bandwidth 8 MHz.   */
   DRX_BANDWIDTH_7MHZ,                    /**< Bandwidth 7 MHz.   */
   DRX_BANDWIDTH_6MHZ,                    /**< Bandwidth 6 MHz.   */
   DRX_BANDWIDTH_UNKNOWN = DRX_UNKNOWN    /**< Bandwidth unknown. */
};

/**
* \enum DRXMirror_t
* \brief Indicate if channel spectrum is mirrored or not.
*/
enum DRXMirror_t{
   DRX_MIRROR_YES     = 0,           /**< Spectrum is mirrored.               */
   DRX_MIRROR_NO,                    /**< Spectrum is not mirrored.           */
   DRX_MIRROR_UNKNOWN = DRX_UNKNOWN, /**< Unknown if spectrum is mirrored.    */
   DRX_MIRROR_AUTO   = DRX_AUTO      /**< Autodetect if spectrum is mirrored. */
};

/**
* \enum DRXConstellation_t
* \brief Constellation type of the channel.
*/
enum DRXConstellation_t{
   DRX_CONSTELLATION_BPSK    = 0,            /**< Modulation is BPSK.       */
   DRX_CONSTELLATION_QPSK,                   /**< Constellation is QPSK.    */
   DRX_CONSTELLATION_PSK8,                   /**< Constellation is PSK8.    */
   DRX_CONSTELLATION_QAM16,                  /**< Constellation is QAM16.   */
   DRX_CONSTELLATION_QAM32,                  /**< Constellation is QAM32.   */
   DRX_CONSTELLATION_QAM64,                  /**< Constellation is QAM64.   */
   DRX_CONSTELLATION_QAM128,                 /**< Constellation is QAM128.  */
   DRX_CONSTELLATION_QAM256,                 /**< Constellation is QAM256.  */
   DRX_CONSTELLATION_QAM512,                 /**< Constellation is QAM512.  */
   DRX_CONSTELLATION_QAM1024,                /**< Constellation is QAM1024. */
   DRX_CONSTELLATION_QPSK_NR,                /**< Constellation is QPSK_NR  */
   DRX_CONSTELLATION_UNKNOWN = DRX_UNKNOWN,  /**< Constellation unknown.    */
   DRX_CONSTELLATION_AUTO    = DRX_AUTO      /**< Autodetect constellation. */
};

/**
* \enum DRXHierarchy_t
* \brief Hierarchy of the channel.
*/
enum DRXHierarchy_t{
   DRX_HIERARCHY_NONE    = 0,           /**< None hierarchical channel.     */
   DRX_HIERARCHY_ALPHA1,                /**< Hierarchical channel, alpha=1. */
   DRX_HIERARCHY_ALPHA2,                /**< Hierarchical channel, alpha=2. */
   DRX_HIERARCHY_ALPHA4,                /**< Hierarchical channel, alpha=4. */
   DRX_HIERARCHY_UNKNOWN = DRX_UNKNOWN, /**< Hierarchy unknown.             */
   DRX_HIERARCHY_AUTO    = DRX_AUTO     /**< Autodetect hierarchy.          */
};

/**
* \enum DRXPriority_t
* \brief Channel priority in case of hierarchical transmission.
*/
enum DRXPriority_t{
   DRX_PRIORITY_LOW     = 0,           /**< Low priority channel.  */
   DRX_PRIORITY_HIGH,                  /**< High priority channel. */
   DRX_PRIORITY_UNKNOWN = DRX_UNKNOWN  /**< Priority unknown.      */
};

/**
* \enum DRXCoderate_t
* \brief Channel priority in case of hierarchical transmission.
*/
enum DRXCoderate_t{
   DRX_CODERATE_1DIV2   = 0,            /**< Code rate 1/2nd.      */
   DRX_CODERATE_2DIV3,                  /**< Code rate 2/3nd.      */
   DRX_CODERATE_3DIV4,                  /**< Code rate 3/4nd.      */
   DRX_CODERATE_5DIV6,                  /**< Code rate 5/6nd.      */
   DRX_CODERATE_7DIV8,                  /**< Code rate 7/8nd.      */
   DRX_CODERATE_UNKNOWN = DRX_UNKNOWN,  /**< Code rate unknown.    */
   DRX_CODERATE_AUTO    = DRX_AUTO      /**< Autodetect code rate. */
};

/**
* \enum DRXGuard_t
* \brief Guard interval of a channel.
*/
enum DRXGuard_t{
   DRX_GUARD_1DIV32   = 0,            /**< Guard interval 1/32nd.     */
   DRX_GUARD_1DIV16,                  /**< Guard interval 1/16th.     */
   DRX_GUARD_1DIV8,                   /**< Guard interval 1/8th.      */
   DRX_GUARD_1DIV4,                   /**< Guard interval 1/4th.      */
   DRX_GUARD_UNKNOWN  = DRX_UNKNOWN,  /**< Guard interval unknown.    */
   DRX_GUARD_AUTO     = DRX_AUTO      /**< Autodetect guard interval. */
};

/**
* \enum DRXFftmode_t
* \brief FFT mode.
*/
enum DRXFftmode_t{
   DRX_FFTMODE_2K      = 0,            /**< 2K FFT mode.         */
   DRX_FFTMODE_4K,                     /**< 4K FFT mode.         */
   DRX_FFTMODE_8K,                     /**< 8K FFT mode.         */
   DRX_FFTMODE_UNKNOWN = DRX_UNKNOWN,  /**< FFT mode unknown.    */
   DRX_FFTMODE_AUTO    = DRX_AUTO      /**< Autodetect FFT mode. */
};

/**
* \enum DRXClassification_t
* \brief Channel classification.
*/
enum DRXClassification_t{
   DRX_CLASSIFICATION_GAUSS   = 0,            /**< Gaussion noise.            */
   DRX_CLASSIFICATION_HVY_GAUSS,              /**< Heavy Gaussion noise.      */
   DRX_CLASSIFICATION_COCHANNEL,              /**< Co-channel.                */
   DRX_CLASSIFICATION_STATIC,                 /**< Static echo.               */
   DRX_CLASSIFICATION_MOVING,                 /**< Moving echo.               */
   DRX_CLASSIFICATION_ZERODB,                 /**< Zero dB echo.              */
   DRX_CLASSIFICATION_UNKNOWN = DRX_UNKNOWN,  /**< Unknown classification     */
   DRX_CLASSIFICATION_AUTO    = DRX_AUTO      /**< Autodetect classification. */
};

/**
* /enum DRXInterleaveModes_t
* /brief Interleave modes
*/
enum DRXInterleaveModes_t{
   DRX_INTERLEAVEMODE_I128_J1    = 0,
   DRX_INTERLEAVEMODE_I128_J1_V2,
   DRX_INTERLEAVEMODE_I128_J2,
   DRX_INTERLEAVEMODE_I64_J2,
   DRX_INTERLEAVEMODE_I128_J3,
   DRX_INTERLEAVEMODE_I32_J4,
   DRX_INTERLEAVEMODE_I128_J4,
   DRX_INTERLEAVEMODE_I16_J8,
   DRX_INTERLEAVEMODE_I128_J5,
   DRX_INTERLEAVEMODE_I8_J16,
   DRX_INTERLEAVEMODE_I128_J6,
   DRX_INTERLEAVEMODE_RESERVED_11,
   DRX_INTERLEAVEMODE_I128_J7,
   DRX_INTERLEAVEMODE_RESERVED_13,
   DRX_INTERLEAVEMODE_I128_J8,
   DRX_INTERLEAVEMODE_RESERVED_15,
   DRX_INTERLEAVEMODE_I12_J17,
   DRX_INTERLEAVEMODE_I5_J4,
   DRX_INTERLEAVEMODE_B52_M240,
   DRX_INTERLEAVEMODE_B52_M720,
   DRX_INTERLEAVEMODE_UNKNOWN = DRX_UNKNOWN,  /**< Unknown interleave mode    */
   DRX_INTERLEAVEMODE_AUTO    = DRX_AUTO      /**< Autodetect interleave mode */
};

/**
* \enum DRXInterleaver_t
* \brief TPS interleaver modes.
*
* Interleave modes for DTMB.
*
*/
enum DRXInterleaver_t{
   DRX_INTERLEAVER_240     = 0,                 /**< Interleaver B=52, M=240  */
   DRX_INTERLEAVER_720,                         /**< Interleaver B=52, M=720  */
   DRX_INTERLEAVER_UNKNOWN = DRX_UNKNOWN,       /**< Interleaver unknown.     */
   DRX_INTERLEAVER_AUTO    = DRX_AUTO           /**< Autodetect Interleaver   */
};

/**
* \enum DRXCarrier_t
* \brief Channel Carrier Mode.
*/
enum DRXCarrier_t{
   DRX_CARRIER_MULTI     = 0,                   /**< Multi carrier mode       */
   DRX_CARRIER_SINGLE,                          /**< Single carrier mode      */
   DRX_CARRIER_UNKNOWN = DRX_UNKNOWN,           /**< Carrier mode unknown.    */
   DRX_CARRIER_AUTO = DRX_AUTO                  /**< Autodetect carrier mode  */
};

/**
* \enum DRXFramemode_t
* \brief Channel Frame Mode.
*/
enum DRXFramemode_t{
   DRX_FRAMEMODE_1     = 0,     /**< Frame 420 with variable PN initial phase */
   DRX_FRAMEMODE_2,             /**< Frame 595                                */
   DRX_FRAMEMODE_3,             /**< Frame 945 with variable PN initial phase */
   DRX_FRAMEMODE_4,             /**< Frame 420 with fixed PN initial phase    */
   DRX_FRAMEMODE_5,             /**< Frame 945 with fixed PN initial phase    */
   DRX_FRAMEMODE_UNKNOWN = DRX_UNKNOWN,   /**< Carrier mode unknown.          */
   DRX_FRAMEMODE_AUTO = DRX_AUTO          /**< Autodetect carrier mode        */
};

/**
* \enum DRXTPSFrame_t
* \brief Frame number in current super-frame.
*/
enum DRXTPSFrame_t{
   DRX_TPS_FRAME1     = 0,                /**< TPS frame 1.       */
   DRX_TPS_FRAME2,                        /**< TPS frame 2.       */
   DRX_TPS_FRAME3,                        /**< TPS frame 3.       */
   DRX_TPS_FRAME4,                        /**< TPS frame 4.       */
   DRX_TPS_FRAME_UNKNOWN = DRX_UNKNOWN    /**< TPS frame unknown. */
};

/**
* \enum DRXLDPC_t
* \brief TPS LDPC .
*/
enum DRXLDPC_t{
   DRX_LDPC_1     = 0,                    /**< LDPC 0.4           */
   DRX_LDPC_2,                            /**< LDPC 0.6           */
   DRX_LDPC_3,                            /**< LDPC 0.8           */
   DRX_LDPC_UNKNOWN = DRX_UNKNOWN,        /**< LDPC unknown.      */
   DRX_LDPC_AUTO = DRX_AUTO               /**< Autodetect LDPC    */
};

/**
* \enum DRXCtrlIndex_t
* \brief Indices of the control functions.
*/
#ifndef DRX_CTRL_BASE
#define DRX_CTRL_BASE          (0x0000)
#endif
enum DRXCtrlIndex_t{
   DRX_CTRL_NOP                 = DRX_CTRL_BASE , /**< No Operation           */
   DRX_CTRL_PROBE_DEVICE        ,                 /**< Probe device           */

   DRX_CTRL_LOAD_UCODE          ,                 /**< Load microcode         */
   DRX_CTRL_VERIFY_UCODE        ,                 /**< Verify microcode       */
   DRX_CTRL_SET_CHANNEL         ,                 /**< Set channel            */
   DRX_CTRL_GET_CHANNEL         ,                 /**< Get channel            */
   DRX_CTRL_LOCK_STATUS         ,                 /**< Get lock status        */
   DRX_CTRL_SIG_QUALITY         ,                 /**< Get signal quality     */
   DRX_CTRL_SIG_STRENGTH        ,                 /**< Get signal strength    */
   DRX_CTRL_RF_POWER            ,                 /**< Get RF power           */
   DRX_CTRL_CONSTEL             ,                 /**< Get constellation point*/
   DRX_CTRL_SCAN_INIT           ,                 /**< Initialize channel scan*/
   DRX_CTRL_SCAN_NEXT           ,                 /**< Scan for next channels */
   DRX_CTRL_SCAN_STOP           ,                 /**< Stop channel scanning  */
   DRX_CTRL_TPS_INFO            ,                 /**< Get TPS info (DVB-T)   */
   DRX_CTRL_SET_CFG             ,                 /**< Set configuration      */
   DRX_CTRL_GET_CFG             ,                 /**< Get configuration      */
   DRX_CTRL_VERSION             ,                 /**< Get version info       */
   DRX_CTRL_I2C_BRIDGE          ,                 /**< Open/close i2c bridge  */
   DRX_CTRL_SET_STANDARD        ,                 /**< Set demodulation std   */
   DRX_CTRL_GET_STANDARD        ,                 /**< Get demodulation std   */
   DRX_CTRL_SET_OOB             ,                 /**< Set out of band param  */
   DRX_CTRL_GET_OOB             ,                 /**< Get out of band param  */
   DRX_CTRL_SET_AUD             ,                 /**< Set audio param        */
   DRX_CTRL_GET_AUD             ,                 /**< Get audio param        */
   DRX_CTRL_READ_RDS            ,                 /**< Read RDS               */
   DRX_CTRL_I2C_READWRITE       ,                 /**< Read/write I2C         */

   /* Professional */
   DRX_CTRL_MB_CFG              ,                 /**<                        */
   DRX_CTRL_MB_READ             ,                 /**<                        */
   DRX_CTRL_MB_WRITE            ,                 /**<                        */
   DRX_CTRL_MB_CONSTEL          ,                 /**<                        */
   DRX_CTRL_MB_MER              ,                 /**<                        */

   /* Misc */
   DRX_CTRL_UIO_CFG             ,                 /**< Configure UIO          */
   DRX_CTRL_UIO_READ            ,                 /**< Read from UIO          */
   DRX_CTRL_UIO_WRITE           ,                 /**< Write to UIO           */
   DRX_CTRL_READ_EVENTS         ,                 /**< Read events            */
   DRX_CTRL_HDL_EVENTS          ,                 /**< Handle events          */
   DRX_CTRL_POWER_MODE          ,                 /**< Set power mode         */

   DRX_CTRL_MAX /* dummy, never to be used */

};

/**
* \enum DRXUCodeAction_t
* \brief Used to indicate if firmware has to be uploaded or verified.
*/
enum DRXUCodeAction_t{
   UCODE_UPLOAD,  /**< Upload the microcode image to device        */
   UCODE_VERIFY   /**< Compare microcode image with code on device */
};


/**
* \enum DRXLockStatus_t
* \brief Used to reflect current lock status of demodulator.
*
* The generic lock states have device dependent semantics.
*/
enum DRXLockStatus_t{
   DRX_NEVER_LOCK = 0,        /**< Device will never lock on this signal */
   DRX_NOT_LOCKED,            /**< Device has no lock at all             */
   DRX_LOCK_STATE_1,          /**< Generic lock state                    */
   DRX_LOCK_STATE_2,          /**< Generic lock state                    */
   DRX_LOCK_STATE_3,          /**< Generic lock state                    */
   DRX_LOCK_STATE_4,          /**< Generic lock state                    */
   DRX_LOCK_STATE_5,          /**< Generic lock state                    */
   DRX_LOCK_STATE_6,          /**< Generic lock state                    */
   DRX_LOCK_STATE_7,          /**< Generic lock state                    */
   DRX_LOCK_STATE_8,          /**< Generic lock state                    */
   DRX_LOCK_STATE_9,          /**< Generic lock state                    */
   DRX_LOCKED                 /**< Device is in lock                     */
};
typedef enum DRXLockStatus_t* pDRXLockStatus_t;

/**
* \enum DRXUIO_t
* \brief Used to address a User IO (UIO).
*/
enum DRXUIO_t{
   DRX_UIO1  ,
   DRX_UIO2  ,
   DRX_UIO3  ,
   DRX_UIO4  ,
   DRX_UIO5  ,
   DRX_UIO6  ,
   DRX_UIO7  ,
   DRX_UIO8  ,
   DRX_UIO9  ,
   DRX_UIO10 ,
   DRX_UIO11 ,
   DRX_UIO12 ,
   DRX_UIO13 ,
   DRX_UIO14 ,
   DRX_UIO15 ,
   DRX_UIO16 ,
   DRX_UIO17 ,
   DRX_UIO18 ,
   DRX_UIO19 ,
   DRX_UIO20 ,
   DRX_UIO21 ,
   DRX_UIO22 ,
   DRX_UIO23 ,
   DRX_UIO24 ,
   DRX_UIO25 ,
   DRX_UIO26 ,
   DRX_UIO27 ,
   DRX_UIO28 ,
   DRX_UIO29 ,
   DRX_UIO30 ,
   DRX_UIO31 ,
   DRX_UIO32 ,
   DRX_UIO_MAX = DRX_UIO32
};

/**
* \enum DRXUIOMode_t
* \brief Used to configure the modus oprandi of a UIO.
*
* DRX_UIO_MODE_FIRMWARE is an old uio mode.
* It is replaced by the modes DRX_UIO_MODE_FIRMWARE0 .. DRX_UIO_MODE_FIRMWARE9.
* To be backward compatible DRX_UIO_MODE_FIRMWARE is equivalent to
* DRX_UIO_MODE_FIRMWARE0.
*/
enum DRXUIOMode_t{
   DRX_UIO_MODE_DISABLE = 0, /**< UIO is not used, pin is configured as input */
   DRX_UIO_MODE_READWRITE,   /**< UIO is used for read/write by application   */
   DRX_UIO_MODE_FIRMWARE ,   /**< UIO is controlled by firmware, function 0   */
   DRX_UIO_MODE_FIRMWARE0 = DRX_UIO_MODE_FIRMWARE , /**< same as above        */
   DRX_UIO_MODE_FIRMWARE1,   /**< UIO is controlled by firmware, function 1   */
   DRX_UIO_MODE_FIRMWARE2,   /**< UIO is controlled by firmware, function 2   */
   DRX_UIO_MODE_FIRMWARE3,   /**< UIO is controlled by firmware, function 3   */
   DRX_UIO_MODE_FIRMWARE4,   /**< UIO is controlled by firmware, function 4   */
   DRX_UIO_MODE_FIRMWARE5,   /**< UIO is controlled by firmware, function 5   */
   DRX_UIO_MODE_FIRMWARE6,   /**< UIO is controlled by firmware, function 6   */
   DRX_UIO_MODE_FIRMWARE7,   /**< UIO is controlled by firmware, function 7   */
   DRX_UIO_MODE_FIRMWARE8,   /**< UIO is controlled by firmware, function 8   */
   DRX_UIO_MODE_FIRMWARE9    /**< UIO is controlled by firmware, function 9   */
};


/**
* \enum DRXOOBDownstreamStandard_t
* \brief Used to select OOB standard.
*
* Based on ANSI 55-1 and 55-2
*/
enum DRXOOBDownstreamStandard_t{
   DRX_OOB_MODE_A = 0,         /**< ANSI 55-1   */
   DRX_OOB_MODE_B_GRADE_A,     /**< ANSI 55-2 A */
   DRX_OOB_MODE_B_GRADE_B      /**< ANSI 55-2 B */
};


/*-------------------------------------------------------------------------
STRUCTS
-------------------------------------------------------------------------*/

/*============================================================================*/
/*============================================================================*/
/*== CTRL CFG related data structures ========================================*/
/*============================================================================*/
/*============================================================================*/

/**
* \enum DRXCfgType_t
* \brief Generic configuration function identifiers.
*/
enum DRXCfgType_t{
   DRX_CFG_MPEG_OUTPUT = 0,  /**< Configure MPEG TS output */
   DRX_CFG_PINS_SAFE_MODE,   /**< Configure pins to safe mode  */
   DRX_CFG_SUBSTANDARD,      /**< Configure modulation substandard  */
   DRX_CTRL_CFG_MAX          /* dummy, never to be used */
};

/*============================================================================*/
/*============================================================================*/
/*== CTRL related data structures ============================================*/
/*============================================================================*/
/*============================================================================*/

/**
* \struct DRXUCodeInfo_t
* \brief Parameters for microcode upload and verfiy.
*
* Used by DRX_CTRL_LOAD_UCODE and DRX_CTRL_VERIFY_UCODE
*/
struct DRXUCodeInfo_t{
   pu8_t    mcData;  /**< Pointer to microcode image. */
   u16_t    mcSize;  /**< Microcode image size.       */
};
typedef struct DRXUCodeInfo_t* pDRXUCodeInfo_t;

/*========================================*/

/**
* \struct DRXChannel_t
* \brief The set of parameters describing a single channel.
*
* Used by DRX_CTRL_SET_CHANNEL and DRX_CTRL_GET_CHANNEL.
* Only certain fields need to be used for a specfic standard.
*
*/
struct DRXChannel_t{
   DRXFrequency_t       frequency;      /**< frequency in kHz                 */
   enum DRXBandwidth_t       bandwidth;      /**< bandwidth                        */
   enum DRXMirror_t          mirror;         /**< mirrored or not on RF            */
   enum DRXConstellation_t   constellation;  /**< constellation                    */
   enum DRXHierarchy_t       hierarchy;      /**< hierarchy                        */
   enum DRXPriority_t        priority;       /**< priority                         */
   enum DRXCoderate_t        coderate;       /**< coderate                         */
   enum DRXGuard_t           guard;          /**< guard interval                   */
   enum DRXFftmode_t         fftmode;        /**< fftmode                          */
   enum DRXClassification_t  classification; /**< classification                   */
   u32_t                symbolrate;     /**< symbolrate in symbols/sec        */
   enum DRXInterleaveModes_t interleavemode; /**< interleaveMode QAM               */
   enum DRXLDPC_t            ldpc;           /**< ldpc                             */
   enum DRXInterleaver_t     interleaver;    /**< interleaver mode DTMB            */
   enum DRXCarrier_t         carrier;        /**< carrier                          */
   enum DRXFramemode_t       framemode;      /**< frame mode                       */
};
typedef struct DRXChannel_t* pDRXChannel_t;

/*========================================*/

/**
* \struct DRXSigQuality_t
* Signal quality metrics.
*
* Used by DRX_CTRL_SIG_QUALITY.
*/
struct DRXSigQuality_t{
   u16_t MER;                  /**< in steps of 0.1 dB                        */
   u32_t preViterbiBER ;       /**< in steps of 1/scaleFactorBER              */
   u32_t postViterbiBER ;      /**< in steps of 1/scaleFactorBER              */
   u32_t scaleFactorBER;       /**< scale factor for BER                      */
   u16_t packetError ;         /**< number of packet errors                   */
   u32_t postReedSolomonBER ;  /**< in steps of 1/scaleFactorBER              */
   u16_t indicator;            /**< indicative signal quality low=0..100=high */
};
typedef struct DRXSigQuality_t* pDRXSigQuality_t;

/*========================================*/

/**
* \struct DRXComplex_t
* A complex number.
*
* Used by DRX_CTRL_CONSTEL.
*/
struct DRXComplex_t{
   s16_t im;                  /**< Imaginary part. */
   s16_t re;                  /**< Real part.      */
};


/*========================================*/

/**
* \struct DRXFrequencyPlan_t
* Array element of a frequency plan.
*
* Used by DRX_CTRL_SCAN_INIT.
*/
struct DRXFrequencyPlan_t{
   DRXFrequency_t first;      /**< First centre frequency in this band        */
   DRXFrequency_t last;       /**< Last centre frequency in this band         */
   enum DRXBandwidth_t bandwidth;  /**< Bandwidth within this frequency band       */
   u16_t          chNumber;   /**< First channel number in this band,
                                    or first index in chNames                 */
   char         **chNames;    /**< Optional list of channel names in this band*/
};
typedef struct DRXFrequencyPlan_t* pDRXFrequencyPlan_t;

/*========================================*/

/**
* \struct DRXFrequencyPlanInfo_t
* Array element of a list of frequency plans.
*
* Used by frequency_plan.h
*/
struct DRXFrequencyPlanInfo_t{
   struct DRXFrequencyPlan_t  *freqPlan;
   int                  freqPlanSize;
   char                *freqPlanName;
};
typedef struct DRXFrequencyPlanInfo_t* pDRXFrequencyPlanInfo_t;
/*========================================*/

/**
* \struct DRXScanParam_t
* Parameters for channel scan.
*
* Used by DRX_CTRL_SCAN_INIT.
*/
struct DRXScanParam_t{
   struct DRXFrequencyPlan_t  *frequencyPlan;       /**< Frequency plan (array)*/
   u16_t                frequencyPlanSize;   /**< Number of bands       */
   u32_t                numTries;            /**< Max channels tried    */
};
typedef struct DRXScanParam_t* pDRXScanParam_t;

/*========================================*/

/**
* \brief Scan commands.
* Used by scanning algorithms.
*/
enum DRXScanCommand_t{
   DRX_SCAN_COMMAND_INIT = 0,             /**< Initialize scanning */
   DRX_SCAN_COMMAND_NEXT,                 /**< Next scan           */
   DRX_SCAN_COMMAND_STOP,                 /**< Stop scanning       */
};

/*========================================*/

typedef struct DRXDemodInstance_t *pDRXDemodInstance_t;

/**
* \brief Inner scan function prototype.
*/
typedef enum DRXStatus_t (*DRXScanFunc_t)  (pDRXDemodInstance_t demod,
                                       enum DRXScanCommand_t    scanCommand,
                                       struct DRXChannel_t       *scanChannel,
                                       pBool_t             getNextChannel  );

/*========================================*/

/**
* \struct DRXTPSInfo_t
* TPS information, DVB-T specific.
*
* Used by DRX_CTRL_TPS_INFO.
*/
struct DRXTPSInfo_t{
   enum DRXFftmode_t         fftmode;          /**< Fft mode       */
   enum DRXGuard_t           guard;            /**< Guard interval */
   enum DRXConstellation_t   constellation;    /**< Constellation  */
   enum DRXHierarchy_t       hierarchy;        /**< Hierarchy      */
   enum DRXCoderate_t        highCoderate;     /**< High code rate */
   enum DRXCoderate_t        lowCoderate;      /**< Low cod rate   */
   enum DRXTPSFrame_t        frame;            /**< Tps frame      */
   u8_t                 length;           /**< Length         */
   u16_t                cellId;           /**< Cell id        */
};
typedef struct DRXTPSInfo_t* pDRXTPSInfo_t;

/*========================================*/

/**
* \brief Power mode of device.
*
* Used by DRX_CTRL_SET_POWER_MODE.
*/
enum DRXPowerMode_t{
   DRX_POWER_UP = 0,     /**< Generic         , Power Up Mode   */
   DRX_POWER_MODE_1,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_2,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_3,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_4,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_5,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_6,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_7,     /**< Device specific , Power Up Mode   */
   DRX_POWER_MODE_8,     /**< Device specific , Power Up Mode   */

   DRX_POWER_MODE_9,     /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_10,    /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_11,    /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_12,    /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_13,    /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_14,    /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_15,    /**< Device specific , Power Down Mode */
   DRX_POWER_MODE_16,    /**< Device specific , Power Down Mode */
   DRX_POWER_DOWN = 255  /**< Generic         , Power Down Mode */
};
typedef enum DRXPowerMode_t* pDRXPowerMode_t;


/*========================================*/

/**
* \enum DRXModule_t
* \brief Software module identification.
*
* Used by DRX_CTRL_VERSION.
*/
enum DRXModule_t{
   DRX_MODULE_DEVICE,
   DRX_MODULE_MICROCODE,
   DRX_MODULE_DRIVERCORE,
   DRX_MODULE_DEVICEDRIVER,
   DRX_MODULE_DAP,
   DRX_MODULE_BSP_I2C,
   DRX_MODULE_BSP_TUNER,
   DRX_MODULE_BSP_HOST,
   DRX_MODULE_UNKNOWN
};


/**
* \enum DRXVersion_t
* \brief Version information of one software module.
*
* Used by DRX_CTRL_VERSION.
*/
struct DRXVersion_t{
   enum DRXModule_t  moduleType;    /**< Type identifier of the module */
   char        *moduleName;    /**< Name or description of module */
   u16_t        vMajor;        /**< Major version number          */
   u16_t        vMinor;        /**< Minor version number          */
   u16_t        vPatch;        /**< Patch version number          */
   char        *vString;       /**< Version as text string        */
};

typedef struct DRXVersion_t* pDRXVersion_t;

/**
* \enum DRXVersionList_t
* \brief List element of NULL terminated, linked list for version information.
*
* Used by DRX_CTRL_VERSION.
*/
struct DRXVersionList_t{
   pDRXVersion_t            version;  /**< Version information */
   struct DRXVersionList_t *next;     /**< Next list element   */
};
typedef struct DRXVersionList_t* pDRXVersionList_t;

/*========================================*/

/**
* \brief Parameters needed to confiugure a UIO.
*
* Used by DRX_CTRL_UIO_CFG.
*/
struct DRXUIOCfg_t{
   enum DRXUIO_t      uio;  /**< UIO identifier       */
   enum DRXUIOMode_t  mode; /**< UIO operational mode */
};
typedef struct DRXUIOCfg_t* pDRXUIOCfg_t;
/*========================================*/

/**
* \brief Parameters needed to read from or write to a UIO.
*
* Used by DRX_CTRL_UIO_READ and DRX_CTRL_UIO_WRITE.
*/
struct DRXUIOData_t{
   enum DRXUIO_t uio;   /**< UIO identifier              */
   enum Bool_t   value; /**< UIO value (TRUE=1, FALSE=0) */
};
typedef struct DRXUIOData_t* pDRXUIOData_t;
/*========================================*/

/**
* \brief Parameters needed to configure OOB.
*
* Used by DRX_CTRL_SET_OOB.
*/
struct DRXOOB_t{
   DRXFrequency_t               frequency;         /**< Frequency in kHz      */
   enum DRXOOBDownstreamStandard_t   standard;          /**< OOB standard          */
   enum Bool_t                       spectrumInverted;  /**< If TRUE, then spectrum
                                                         is inverted          */
};


/*========================================*/

/**
* \brief Metrics from OOB.
*
* Used by DRX_CTRL_GET_OOB.
*/
struct DRXOOBStatus_t{
   DRXFrequency_t       frequency;        /**< Frequency in Khz         */
   enum DRXLockStatus_t      lock;             /**< Lock status              */
   u32_t                mer;              /**< MER                      */
   s32_t                symbolRateOffset; /**< Symbolrate offset in ppm */
};


/*========================================*/

/**
* \brief Device dependent configuration data.
*
* Used by DRX_CTRL_SET_CFG and DRX_CTRL_GET_CFG.
* A sort of nested DRX_Ctrl() functionality for device specific controls.
*/
struct DRXCfg_t{
   enum DRXCfgType_t cfgType ; /**< Function identifier */
   void*        cfgData ; /**< Function data */
};
typedef struct DRXCfg_t* pDRXCfg_t;

/*========================================*/

/* CTRL CFG MPEG ouput */
/**
* \struct DRXCfgMPEGOutput_t
* \brief Configuartion parameters for MPEG output control.
*
* Used by DRX_CFG_MPEG_OUTPUT, in combination with DRX_CTRL_SET_CFG and
* DRX_CTRL_GET_CFG.
*/

struct DRXCfgMPEGOutput_t{
   enum Bool_t      enableMPEGOutput;  /**< If TRUE, enable MPEG output            */
   enum Bool_t      insertRSByte;      /**< If TRUE, insert RS byte                */
   enum Bool_t      enableParallel;    /**< If TRUE, parallel out otherwise serial */
   enum Bool_t      invertDATA;        /**< If TRUE, invert DATA signals           */
   enum Bool_t      invertERR;         /**< If TRUE, invert ERR signal             */
   enum Bool_t      invertSTR;         /**< If TRUE, invert STR signals            */
   enum Bool_t      invertVAL;         /**< If TRUE, invert VAL signals            */
   enum Bool_t      invertCLK;         /**< If TRUE, invert CLK signals            */
   enum Bool_t      staticCLK;         /**< If TRUE, static MPEG clockrate will
                                       be used, otherwise clockrate will
                                       adapt to the bitrate of the TS         */
   u32_t       bitrate;           /**< Maximum bitrate in b/s in case
                                       static clockrate is selected           */
};
typedef struct DRXCfgMPEGOutput_t* pDRXCfgMPEGOutput_t;
/*========================================*/

/**
* \struct DRXI2CData_t
* \brief Data for I2C via 2nd or 3rd or etc I2C port.
*
* Used by DRX_CTRL_I2C_READWRITE.
* If portNr is equal to primairy portNr BSPI2C will be used.
*
*/
struct DRXI2CData_t{
   u16_t            portNr;     /**< I2C port number               */
   pI2CDeviceAddr_t wDevAddr;   /**< Write device address          */
   u16_t            wCount;     /**< Size of write data in bytes   */
   pu8_t            wData;      /**< Pointer to write data         */
   pI2CDeviceAddr_t rDevAddr;   /**< Read device address           */
   u16_t            rCount;     /**< Size of data to read in bytes */
   pu8_t            rData;      /**< Pointer to read buffer        */
};

/*========================================*/

/**
* \enum DRXAudStandard_t
* \brief Audio standard identifier.
*
* Used by DRX_CTRL_SET_AUD.
*/
enum DRXAudStandard_t{
   DRX_AUD_STANDARD_BTSC,                  /**< set BTSC standard (USA)       */
   DRX_AUD_STANDARD_A2,                    /**< set A2-Korea FM Stereo        */
   DRX_AUD_STANDARD_EIAJ,                  /**< set to Japanese FM Stereo     */
   DRX_AUD_STANDARD_FM_STEREO,             /**< set to FM-Stereo Radio        */
   DRX_AUD_STANDARD_M_MONO,                /**< for 4.5 MHz mono detected     */
   DRX_AUD_STANDARD_D_K_MONO,              /**< for 6.5 MHz mono detected     */
   DRX_AUD_STANDARD_BG_FM,                 /**< set BG_FM standard            */
   DRX_AUD_STANDARD_D_K1,                  /**< set D_K1 standard             */
   DRX_AUD_STANDARD_D_K2,                  /**< set D_K2 standard             */
   DRX_AUD_STANDARD_D_K3,                  /**< set D_K3 standard             */
   DRX_AUD_STANDARD_BG_NICAM_FM,           /**< set BG_NICAM_FM standard      */
   DRX_AUD_STANDARD_L_NICAM_AM,            /**< set L_NICAM_AM standard       */
   DRX_AUD_STANDARD_I_NICAM_FM,            /**< set I_NICAM_FM standard       */
   DRX_AUD_STANDARD_D_K_NICAM_FM,          /**< set D_K_NICAM_FM standard     */
   DRX_AUD_STANDARD_AUTO = DRX_AUTO,       /**< Automatic Standard Detection  */
   DRX_AUD_STANDARD_UNKNOWN = DRX_UNKNOWN  /**< used as auto and for readback */
};

/**
* \enum DRXAudChannelSelect_t
* \brief Used to select input to I2S.
* If it is a dual language broadcast it is possible to take
* either sound A or B as an input
*/
enum DRXAudChannelSelect_t{
   DRX_AUD_CHANNELSELECT_MONO,             /**< mono signal as input to I2S   */
   DRX_AUD_CHANNELSELECT_STEREO_AB,        /**< stereo signal as input to I2S */
   DRX_AUD_CHANNELSELECT_STEREO_A,         /**< sound A is an input           */
   DRX_AUD_CHANNELSELECT_STEREO_B          /**< sound B is an input           */
};

/**
* \enum DRXAudAVCDecayTime_t
* \brief Used to set time for volume change, automatic volume control.
*/
enum DRXAudAVCDecayTime_t{
   DRX_AUD_AVC_OFF,                /**< Audio Volume Control is off      */
   DRX_AUD_AVC_DECAYTIME_8S,       /**< level volume in 8 seconds        */
   DRX_AUD_AVC_DECAYTIME_4S,       /**< level volume in 4 seconds        */
   DRX_AUD_AVC_DECAYTIME_2S,       /**< level volume in 2 seconds        */
   DRX_AUD_AVC_DECAYTIME_20MS      /**< level volume in 20 milli seconds */
};

/**
* \enum DRXAudVidSync_t
* \brief Used to audio video synchronization.
* both auto 1 and 2 are hints for automatic video standard detection,
* because close frequencies between NTSC & Monochrome (59.94Hz & 60Hz)
*/
enum DRXAudVidSync_t{
   DRX_AUD_VIDSYNC_OFF,            /**< Audio Video Synchronization is off    */
   DRX_AUD_VIDSYNC_NTSC,           /**< it is a NTSC system                   */
   DRX_AUD_VIDSYNC_MONOCHROME,     /**< it is a MONOCHROME system             */
   DRX_AUD_VIDSYNC_PAL,            /**< it is a PAL system                    */
   DRX_AUD_VIDSYNC_SECAM,          /**< it is a SECAM system                  */
   DRX_AUD_VIDSYNC_AUTO_1,         /**< auto-detecting video (hint NTSC)      */
   DRX_AUD_VIDSYNC_AUTO_2          /**< auto-detecting video (hint Monochrome)*/
};
/**
* \enum DRXAudI2SMatrix_t
* \brief Used for selecting I2S output.
*/
enum DRXAudI2SMatrix_t{
   DRX_AUD_I2SMATRIX_A_MONO,              /**< A sound only, stereo or mono   */
   DRX_AUD_I2SMATRIX_B_MONO,              /**< B sound only, stereo or mono   */
   DRX_AUD_I2SMATRIX_STEREO,              /**< A+B sound, transparant         */
   DRX_AUD_I2SMATRIX_MONO                 /**< A+B mixed to mono sum, (L+R)/2 */
};

/**
* /enum DRXAudFMDeemphasis_t
* setting for FM-Deemphasis in audio demodulator.
*
*/
enum DRXAudFMDeemphasis_t{
   DRX_AUD_FM_DEEMPH_50US,
   DRX_AUD_FM_DEEMPH_75US,
   DRX_AUD_FM_DEEMPH_OFF
};

/**
* /enum DRXAudFMMatrix_t
* setting for FM-Matrix in audio demodulator.
*
*/
enum DRXAudFMMatrix_t{
   DRX_AUD_FM_MATRIX_NO_MATRIX,
   DRX_AUD_FM_MATRIX_GERMAN,
   DRX_AUD_FM_MATRIX_KOREAN,
   DRX_AUD_FM_MATRIX_SOUND_A,
   DRX_AUD_FM_MATRIX_SOUND_B
};

/**
* /enum DRXAudDeviation_t
* setting for deviation mode in audio demodulator.
*
*/
enum DRXAudDeviation_t{
   DRX_AUD_DEVIATION_AUTO,
   DRX_AUD_DEVIATION_NORMAL,
   DRX_AUD_DEVIATION_HIGH
};

/**
* /enum DRXAudMaxAVCGain_t
* setting for AVC max gain in audio baseband.
*
*/
enum DRXAudAVCMaxGain_t{
   DRX_AUD_AVC_MAX_GAIN_0DB,
   DRX_AUD_AVC_MAX_GAIN_6DB,
   DRX_AUD_AVC_MAX_GAIN_12DB
};

/**
* /enum DRXAudMaxAVCAtten_t
* setting for AVC max Attenuation in audio baseband.
*
*/
enum DRXAudAVCMaxAtten_t{
   DRX_AUD_AVC_MAX_ATTEN_12DB,
   DRX_AUD_AVC_MAX_ATTEN_18DB,
   DRX_AUD_AVC_MAX_ATTEN_24DB
};

/**
* \enum DRXI2SFormat_t
* \brief Used to set data wordstrobe alignment for I2S.
*/
enum DRXI2SFormat_t{
   DRX_I2S_FORMAT_WS_WITH_DATA,    /**< I2S data and wordstrobe are aligned   */
   DRX_I2S_FORMAT_WS_ADVANCED      /**< I2S data is one cycle after wordstrobe*/
};

/**
* \enum DRXI2SWordLength_t
* \brief Used to determine width of data in I2S
* if it is 0, data is 32 bit wide
*/
enum DRXI2SWordLength_t{
   DRX_I2S_WORDLENGTH_32,          /**< I2S data is 32 bit wide */
   DRX_I2S_WORDLENGTH_16           /**< I2S data is 16 bit wide */
};

/**
* \enum DRXI2SPolarity_t
* \brief Used to set polarity of I2S data.
* polarity left - left high (right low) and vice versa
*/
enum DRXI2SPolarity_t{
   DRX_I2S_POLARITY_RIGHT,         /**< wordstrobe - right high, left low */
   DRX_I2S_POLARITY_LEFT           /**< wordstrobe - right low, left high */
};

/**
* \enum DRXI2SMode_t
* \brief Used to set output mode for I2S.
*/
enum DRXI2SMode_t{
   DRX_I2S_MODE_MASTER,            /**< I2S is in master mode */
   DRX_I2S_MODE_SLAVE              /**< I2S is in slave mode  */
};

/**
* \enum DRXAudNICAMStatus_t
* \brief Indicate status of NICAM carrier.
*/
enum DRXAudNICAMStatus_t{
   DRX_AUD_NICAM_DETECTED = 0,  /**< NICAM carrier detected     */
   DRX_AUD_NICAM_NOT_DETECTED,  /**< NICAM carrier not detected */
   DRX_AUD_NICAM_BAD            /**< NICAM carrier bad quality  */
};

/**
* \struct DRXI2SConfig_t
* \brief Container with settings for I2S configuration.
*/
struct DRXI2SConfig_t{
   enum Bool_t                outputEnable;     /**< I2S output enable             */
   u32_t                 frequency;        /**< range from 8000-48000 Hz      */
   enum DRXI2SMode_t          mode;             /**< I2S mode, master or slave     */
   enum DRXI2SWordLength_t    wordLength;       /**< I2S wordlength, 16 or 32 bits */
   enum DRXI2SPolarity_t      polarity;         /**< I2S wordstrobe polarity       */
   enum DRXI2SFormat_t        format;           /**< I2S wordstrobe delay to data  */
};

/**
* \struct DRXAudConfig_t
* \brief Container with settings for driver for AUD block.
*/
struct DRXAudConfig_t{

   /* basic configuration */
   enum DRXAudStandard_t      standard;         /**< audio standard selection      */
   enum DRXAudChannelSelect_t channelSelect;    /**< only if ASD is enabled        */
   u16_t                 thresholdNicam;   /**< NICAM-Threshold               */
   u16_t                 thresholdBTSC;    /**< BTSC/FM-Radio-Threshold       */
   u16_t                 thresholdCarrierA;/**< A Carrier Detection Threshold */
   u16_t                 thresholdCarrierB;/**< B Carrier Detection Threshold */

   /* ASS related configuration */
   enum Bool_t                autoSoundSelect;  /**< automatic sound selection     */
   enum Bool_t                muteNoCarrierA;   /**< no A carrier, mute/noise      */
   enum Bool_t                muteNoCarrierB;   /**< no B carrier, mute/noise      */
   enum Bool_t                selectSAP;        /**< select SAP+mono if available  */
   enum DRXAudDeviation_t     deviationMode;    /**< deviation mode selection      */
   DRXFrequency_t        maxFMDeviation;   /**< prescale, new deviation used  */
   s16_t                 nicamPrescaler;   /**< NICAM prescale [dB]           */
   enum DRXAudFMDeemphasis_t  deemphasis;       /**< FM deemphasis setting         */
   enum DRXAudFMMatrix_t      matrix;           /**< FM matrix setting             */

   /* volume control */
   enum Bool_t                mute;             /**< mute 'overrules' volume       */
   s16_t                 volume;           /**< range: -114 to 12 dB          */
   enum DRXAudAVCDecayTime_t  avcDecayTime;     /**< automatic volume control      */
   u16_t                 avcRefLevel;      /**< AVC Reference Level           */
   enum DRXAudAVCMaxGain_t    avcMaxGain;       /**< AVC max gain selection        */
   enum DRXAudAVCMaxAtten_t   avcMaxAtten;      /**< AVC max attenuation selection */

   /* miscellaneous settings */
   enum DRXAudVidSync_t       vidSync;          /**< audio-video synchronization   */

   /* I2S output */
   enum DRXAudI2SMatrix_t     i2sMatrix;        /**< matrix of carriers, I2S output*/
   struct DRXI2SConfig_t        i2sConfig;        /**< settings for I2S              */
};
typedef struct DRXAudConfig_t* pDRXAudConfig_t;
/*========================================*/
/*   DRX_CTRL_GET_AUD  */

/**
* \struct DRXAudStatus_t
* \brief Used for retrieving AUD configuration.
* quasi-peak, before I2S mixer, in 0.1 dB, range approx -100..+12 dB
*/
struct DRXAudStatus_t{

   enum Bool_t            stereo;               /**< stereo detection              */
   enum Bool_t            sap;                  /**< sap / bilingual detection     */
   enum Bool_t            carrierA;             /**< carrier A detected            */
   enum Bool_t            carrierB;             /**< carrier B detected            */
   enum Bool_t            rds;                  /**< new RDS data array present    */
   enum DRXAudNICAMStatus_t  nicamStatus;       /**< status of NICAM carrier       */

   s16_t             strengthLeft;         /**< quasi-peak, left speaker      */
   s16_t             strengthRight;        /**< quasi-peak, right speaker     */

   enum DRXAudStandard_t  standard;             /**< detected or last standard set */
};
typedef struct DRXAudStatus_t* pDRXAudStatus_t;

/*========================================*/
/*   DRX_CTRL_READ_RDS */

/**
* \struct DRXRDSdata_t
* \brief Used for retriveing RDS array.
* it returns array number and data within that array
* control function must return error if no RDS data available
*/
struct DRXRDSdata_t{
   enum Bool_t            valid;                /**< RDS data validation          */
   u16_t             data[18];             /**< data from one RDS data array */
};
typedef struct DRXRDSdata_t* pDRXRDSdata_t;


/*============================================================================*/
/*============================================================================*/
/*== Data access structures ==================================================*/
/*============================================================================*/
/*============================================================================*/

/* Address on device */
typedef u32_t DRXaddr_t, *pDRXaddr_t;

/* Protocol specific flags */
typedef u32_t DRXflags_t, *pDRXflags_t;

/* Write block of data to device */
typedef enum DRXStatus_t (*DRXWriteBlockFunc_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         u16_t            datasize,      /* size of data in bytes        */
         pu8_t            data,          /* data to send                 */
         DRXflags_t       flags);

/* Read block of data from device */
typedef enum DRXStatus_t (*DRXReadBlockFunc_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         u16_t            datasize,      /* size of data in bytes        */
         pu8_t            data,          /* receive buffer               */
         DRXflags_t       flags);

/* Write 8-bits value to device */
typedef enum DRXStatus_t (*DRXWriteReg8Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         u8_t             data,          /* data to send                 */
         DRXflags_t       flags);

/* Read 8-bits value to device */
typedef enum DRXStatus_t (*DRXReadReg8Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         pu8_t            data,          /* receive buffer               */
         DRXflags_t       flags);

/* Read modify write 8-bits value to device */
typedef enum DRXStatus_t (*DRXReadModifyWriteReg8Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device       */
         DRXaddr_t        waddr,         /* write address of register   */
         DRXaddr_t        raddr,         /* read  address of register   */
         u8_t             wdata,         /* data to write               */
         pu8_t            rdata);        /* data to read                */

/* Write 16-bits value to device */
typedef enum DRXStatus_t (*DRXWriteReg16Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         u16_t            data,          /* data to send                 */
         DRXflags_t       flags);

/* Read 16-bits value to device */
typedef enum DRXStatus_t (*DRXReadReg16Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         pu16_t           data,          /* receive buffer               */
         DRXflags_t       flags);

/* Read modify write 16-bits value to device */
typedef enum DRXStatus_t (*DRXReadModifyWriteReg16Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device       */
         DRXaddr_t        waddr,         /* write address of register   */
         DRXaddr_t        raddr,         /* read  address of register   */
         u16_t            wdata,         /* data to write               */
         pu16_t           rdata);        /* data to read                */

/* Write 32-bits value to device */
typedef enum DRXStatus_t (*DRXWriteReg32Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         u32_t            data,          /* data to send                 */
         DRXflags_t       flags);

/* Read 32-bits value to device */
typedef enum DRXStatus_t (*DRXReadReg32Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device        */
         DRXaddr_t        addr,          /* address of register/memory   */
         pu32_t           data,          /* receive buffer               */
         DRXflags_t       flags);

/* Read modify write 32-bits value to device */
typedef enum DRXStatus_t (*DRXReadModifyWriteReg32Func_t) (
         pI2CDeviceAddr_t devAddr,       /* address of I2C device       */
         DRXaddr_t        waddr,         /* write address of register   */
         DRXaddr_t        raddr,         /* read  address of register   */
         u32_t            wdata,         /* data to write               */
         pu32_t           rdata);        /* data to read                */

/**
* \struct DRXAccessFunc_t
* \brief Interface to an access protocol.
*/
struct DRXAccessFunc_t{
   struct DRXVersion_t                   *protocolVersion;
   DRXWriteBlockFunc_t             writeBlockFunc;
   DRXReadBlockFunc_t              readBlockFunc;
   DRXWriteReg8Func_t              writeReg8Func;
   DRXReadReg8Func_t               readReg8Func;
   DRXReadModifyWriteReg8Func_t    readModifyWriteReg8Func;
   DRXWriteReg16Func_t             writeReg16Func;
   DRXReadReg16Func_t              readReg16Func;
   DRXReadModifyWriteReg16Func_t   readModifyWriteReg16Func;
   DRXWriteReg32Func_t             writeReg32Func;
   DRXReadReg32Func_t              readReg32Func;
   DRXReadModifyWriteReg32Func_t   readModifyWriteReg32Func;
};
typedef struct DRXAccessFunc_t* pDRXAccessFunc_t;

/*============================================================================*/
/*============================================================================*/
/*== Demod instance data structures ==========================================*/
/*============================================================================*/
/*============================================================================*/

/**
* \struct DRXCommonAttr_t
* \brief Set of common attributes, shared by all DRX devices.
*/
struct DRXCommonAttr_t{
   /* Microcode (firmware) attributes */
   pu8_t          microcode;       /**< Pointer to microcode image.           */
   u16_t          microcodeSize;   /**< Size of microcode image in bytes.     */
   enum Bool_t         verifyMicrocode; /**< Use microcode verify or not.          */

   /* Clocks and tuner attributes */
   DRXFrequency_t intermediateFreq;  /**< IF,if tuner instance not used. (kHz)*/
   DRXFrequency_t sysClockFreq;      /**< Systemclock frequency.  (kHz)       */
   DRXFrequency_t oscClockFreq;      /**< Oscillator clock frequency.  (kHz)  */
   s16_t          oscClockDeviation; /**< Oscillator clock deviation.  (ppm)  */
   enum Bool_t         mirrorFreqSpect;   /**< Mirror IF frequency spectrum or not.*/

   /* Initial MPEG output attributes */
   enum Bool_t      enableMPEGOutput;  /**< If TRUE, enable MPEG output            */
   enum Bool_t      insertRSByte;      /**< If TRUE, insert RS byte                */
   enum Bool_t      enableParallel;    /**< If TRUE, parallel out otherwise serial */
   enum Bool_t      invertDATA;        /**< If TRUE, invert DATA signals           */
   enum Bool_t      invertERR;         /**< If TRUE, invert ERR signal             */
   enum Bool_t      invertSTR;         /**< If TRUE, invert STR signals            */
   enum Bool_t      invertVAL;         /**< If TRUE, invert VAL signals            */
   enum Bool_t      invertCLK;         /**< If TRUE, invert CLK signals            */
   enum Bool_t      staticCLK;         /**< If TRUE, static MPEG clockrate will
                                       be used, otherwise clockrate will
                                       adapt to the bitrate of the TS         */
   u32_t       bitrate;           /**< Maximum bitrate in b/s in case
                                       static clockrate is selected           */

   enum Bool_t      isOpened;          /**< if TRUE instance is already opened.    */

   /* Channel scan - outer loop: frequency plan related */
   struct DRXScanParam_t	*scanParam;         /**< scan parameters                    */
   u16_t           scanFreqPlanIndex; /**< next index in freq plan            */
   DRXFrequency_t  scanNextFrequency; /**< next freq to scan                  */
   enum Bool_t          scanReady;         /**< scan ready flag                    */
   u32_t           scanMaxChannels;   /**< number of channels in freqplan     */
   u32_t           scanChannelsScanned; /**< number of channels scanned       */

   /* Channel scan - inner loop: demod related */
   DRXScanFunc_t   scanFunction;      /**< function to check channel          */

   /* Channel scan - parameters for default DTV scan function in core driver  */
   u16_t           scanDemodLockTimeout; /**< millisecs to wait for lock      */
   enum DRXLockStatus_t scanDesiredLock;   /**< lock requirement for channel found */

   /* scanActive can be used by SetChannel to decide how to program the tuner,
      fast or slow (but stable). Usually fast during scan. */
   enum Bool_t          scanActive;        /**< TRUE when scan routines are active */

   /* Power management */
   enum DRXPowerMode_t  currentPowerMode;  /**< current power management mode      */

   /* Tuner */
   u8_t            tunerPortNr;       /**< nr of I2C port to wich tuner is    */
   DRXFrequency_t  tunerMinFreqRF;    /**< minimum RF input frequency, in kHz */
   DRXFrequency_t  tunerMaxFreqRF;    /**< maximum RF input frequency, in kHz */
   enum Bool_t          tunerRfAgcPol;     /**< if TRUE invert RF AGC polarity     */
   enum Bool_t          tunerIfAgcPol;     /**< if TRUE invert IF AGC polarity     */

};

typedef struct DRXCommonAttr_t  *pDRXCommonAttr_t;

/*
* Generic functions for DRX devices.
*/
typedef enum DRXStatus_t (*DRXOpenFunc_t)  (pDRXDemodInstance_t demod);
typedef enum DRXStatus_t (*DRXCloseFunc_t) (pDRXDemodInstance_t demod);
typedef enum DRXStatus_t (*DRXCtrlFunc_t)  (pDRXDemodInstance_t demod,
                                       enum DRXCtrlIndex_t ctrl,
                                       void *ctrlData);

/**
* \struct DRXDemodFunc_t
* \brief A stucture containing all functions of a demodulator.
*/
struct DRXDemodFunc_t{
   u32_t          typeId;                /**< Device type identifier.      */
   DRXOpenFunc_t  openFunc;              /**< Pointer to Open() function.  */
   DRXCloseFunc_t closeFunc;             /**< Pointer to Close() function. */
   DRXCtrlFunc_t  ctrlFunc;              /**< Pointer to Ctrl() function.  */
};
typedef struct DRXDemodFunc_t* pDRXDemodFunc_t;

/**
* \struct DRXDemodInstance_t
* \brief Top structure of demodulator instance.
*/
struct DRXDemodInstance_t {
   /* type specific demodulator data */
   pDRXDemodFunc_t   myDemodFunct;  /**< demodulator functions                */
   pDRXAccessFunc_t  myAccessFunct; /**< data access protocol functions       */
   struct TUNERInstance_t  *myTuner;       /**< tuner instance,if NULL then baseband */
   pI2CDeviceAddr_t  myI2CDevAddr;  /**< i2c address and device identifier    */
   pDRXCommonAttr_t  myCommonAttr;  /**< common DRX attributes                */
   void*             myExtAttr;     /**< device specific attributes           */
   /* generic demodulator data */
};



/*-------------------------------------------------------------------------
MACROS
Conversion from enum values to human readable form.
-------------------------------------------------------------------------*/

/* standard */

#define DRX_STR_STANDARD(x) ( \
   ( x == DRX_STANDARD_DVBT              ) ? "DVB-T"                         : \
   ( x == DRX_STANDARD_8VSB              ) ? "8VSB"                          : \
   ( x == DRX_STANDARD_NTSC              ) ? "NTSC"                          : \
   ( x == DRX_STANDARD_PAL_SECAM_BG      ) ? "PAL/SECAM B/G"                 : \
   ( x == DRX_STANDARD_PAL_SECAM_DK      ) ? "PAL/SECAM D/K"                 : \
   ( x == DRX_STANDARD_PAL_SECAM_I       ) ? "PAL/SECAM I"                   : \
   ( x == DRX_STANDARD_PAL_SECAM_L       ) ? "PAL/SECAM L"                   : \
   ( x == DRX_STANDARD_PAL_SECAM_LP      ) ? "PAL/SECAM LP"                  : \
   ( x == DRX_STANDARD_ITU_A             ) ? "ITU-A"                         : \
   ( x == DRX_STANDARD_ITU_B             ) ? "ITU-B"                         : \
   ( x == DRX_STANDARD_ITU_C             ) ? "ITU-C"                         : \
   ( x == DRX_STANDARD_ITU_D             ) ? "ITU-D"                         : \
   ( x == DRX_STANDARD_FM                ) ? "FM"                            : \
   ( x == DRX_STANDARD_DTMB              ) ? "DTMB"                          : \
   ( x == DRX_STANDARD_AUTO              ) ? "Auto"                          : \
   ( x == DRX_STANDARD_UNKNOWN           ) ? "Unknown"                       : \
                                             "(Invalid)"  )

/* channel */

#define DRX_STR_BANDWIDTH(x) ( \
   ( x == DRX_BANDWIDTH_8MHZ             ) ? "8 MHz"                         : \
   ( x == DRX_BANDWIDTH_7MHZ             ) ? "7 MHz"                         : \
   ( x == DRX_BANDWIDTH_6MHZ             ) ? "6 MHz"                         : \
   ( x == DRX_BANDWIDTH_UNKNOWN          ) ? "Unknown"                       : \
                                             "(Invalid)"  )
#define DRX_STR_FFTMODE(x) ( \
   ( x == DRX_FFTMODE_2K                 ) ? "2k"                            : \
   ( x == DRX_FFTMODE_4K                 ) ? "4k"                            : \
   ( x == DRX_FFTMODE_8K                 ) ? "8k"                            : \
   ( x == DRX_FFTMODE_AUTO               ) ? "Auto"                          : \
   ( x == DRX_FFTMODE_UNKNOWN            ) ? "Unknown"                       : \
                                             "(Invalid)"  )
#define DRX_STR_GUARD(x) ( \
   ( x == DRX_GUARD_1DIV32               ) ? "1/32nd"                        : \
   ( x == DRX_GUARD_1DIV16               ) ? "1/16th"                        : \
   ( x == DRX_GUARD_1DIV8                ) ? "1/8th"                         : \
   ( x == DRX_GUARD_1DIV4                ) ? "1/4th"                         : \
   ( x == DRX_GUARD_AUTO                 ) ? "Auto"                          : \
   ( x == DRX_GUARD_UNKNOWN              ) ? "Unknown"                       : \
                                             "(Invalid)"  )
#define DRX_STR_CONSTELLATION(x) ( \
   ( x == DRX_CONSTELLATION_BPSK         ) ? "BPSK"                          : \
   ( x == DRX_CONSTELLATION_QPSK         ) ? "QPSK"                          : \
   ( x == DRX_CONSTELLATION_PSK8         ) ? "PSK8"                          : \
   ( x == DRX_CONSTELLATION_QAM16        ) ? "QAM16"                         : \
   ( x == DRX_CONSTELLATION_QAM32        ) ? "QAM32"                         : \
   ( x == DRX_CONSTELLATION_QAM64        ) ? "QAM64"                         : \
   ( x == DRX_CONSTELLATION_QAM128       ) ? "QAM128"                        : \
   ( x == DRX_CONSTELLATION_QAM256       ) ? "QAM256"                        : \
   ( x == DRX_CONSTELLATION_QAM512       ) ? "QAM512"                        : \
   ( x == DRX_CONSTELLATION_QAM1024      ) ? "QAM1024"                       : \
   ( x == DRX_CONSTELLATION_AUTO         ) ? "Auto"                          : \
   ( x == DRX_CONSTELLATION_UNKNOWN      ) ? "Unknown"                       : \
                                             "(Invalid)" )
#define DRX_STR_CODERATE(x) ( \
   ( x == DRX_CODERATE_1DIV2             ) ? "1/2nd"                         : \
   ( x == DRX_CODERATE_2DIV3             ) ? "2/3rd"                         : \
   ( x == DRX_CODERATE_3DIV4             ) ? "3/4th"                         : \
   ( x == DRX_CODERATE_5DIV6             ) ? "5/6th"                         : \
   ( x == DRX_CODERATE_7DIV8             ) ? "7/8th"                         : \
   ( x == DRX_CODERATE_AUTO              ) ? "Auto"                          : \
   ( x == DRX_CODERATE_UNKNOWN           ) ? "Unknown"                       : \
                                             "(Invalid)" )
#define DRX_STR_HIERARCHY(x) ( \
   ( x == DRX_HIERARCHY_NONE             ) ? "None"                          : \
   ( x == DRX_HIERARCHY_ALPHA1           ) ? "Alpha=1"                       : \
   ( x == DRX_HIERARCHY_ALPHA2           ) ? "Alpha=2"                       : \
   ( x == DRX_HIERARCHY_ALPHA4           ) ? "Alpha=4"                       : \
   ( x == DRX_HIERARCHY_AUTO             ) ? "Auto"                          : \
   ( x == DRX_HIERARCHY_UNKNOWN          ) ? "Unknown"                       : \
                                             "(Invalid)" )
#define DRX_STR_PRIORITY(x) ( \
   ( x == DRX_PRIORITY_LOW               ) ? "Low"                           : \
   ( x == DRX_PRIORITY_HIGH              ) ? "High"                          : \
   ( x == DRX_PRIORITY_UNKNOWN           ) ? "Unknown"                       : \
                                             "(Invalid)" )
#define DRX_STR_MIRROR(x) ( \
   ( x == DRX_MIRROR_NO                  ) ? "Normal"                        : \
   ( x == DRX_MIRROR_YES                 ) ? "Mirrored"                      : \
   ( x == DRX_MIRROR_AUTO                ) ? "Auto"                          : \
   ( x == DRX_MIRROR_UNKNOWN             ) ? "Unknown"                       : \
                                             "(Invalid)" )
#define DRX_STR_CLASSIFICATION(x) ( \
   ( x == DRX_CLASSIFICATION_GAUSS       ) ? "Gaussion"                      : \
   ( x == DRX_CLASSIFICATION_HVY_GAUSS   ) ? "Heavy Gaussion"                : \
   ( x == DRX_CLASSIFICATION_COCHANNEL   ) ? "Co-channel"                    : \
   ( x == DRX_CLASSIFICATION_STATIC      ) ? "Static echo"                   : \
   ( x == DRX_CLASSIFICATION_MOVING      ) ? "Moving echo"                   : \
   ( x == DRX_CLASSIFICATION_ZERODB      ) ? "Zero dB echo"                  : \
   ( x == DRX_CLASSIFICATION_UNKNOWN     ) ? "Unknown"                       : \
   ( x == DRX_CLASSIFICATION_AUTO        ) ? "Auto"                          : \
                                             "(Invalid)" )

#define DRX_STR_INTERLEAVEMODE(x) ( \
   ( x == DRX_INTERLEAVEMODE_I128_J1     ) ? "I128_J1"                       : \
   ( x == DRX_INTERLEAVEMODE_I128_J1_V2  ) ? "I128_J1_V2"                    : \
   ( x == DRX_INTERLEAVEMODE_I128_J2     ) ? "I128_J2"                       : \
   ( x == DRX_INTERLEAVEMODE_I64_J2      ) ? "I64_J2"                        : \
   ( x == DRX_INTERLEAVEMODE_I128_J3     ) ? "I128_J3"                       : \
   ( x == DRX_INTERLEAVEMODE_I32_J4      ) ? "I32_J4"                        : \
   ( x == DRX_INTERLEAVEMODE_I128_J4     ) ? "I128_J4"                       : \
   ( x == DRX_INTERLEAVEMODE_I16_J8      ) ? "I16_J8"                        : \
   ( x == DRX_INTERLEAVEMODE_I128_J5     ) ? "I128_J5"                       : \
   ( x == DRX_INTERLEAVEMODE_I8_J16      ) ? "I8_J16"                        : \
   ( x == DRX_INTERLEAVEMODE_I128_J6     ) ? "I128_J6"                       : \
   ( x == DRX_INTERLEAVEMODE_RESERVED_11 ) ? "Reserved 11"                   : \
   ( x == DRX_INTERLEAVEMODE_I128_J7     ) ? "I128_J7"                       : \
   ( x == DRX_INTERLEAVEMODE_RESERVED_13 ) ? "Reserved 13"                   : \
   ( x == DRX_INTERLEAVEMODE_I128_J8     ) ? "I128_J8"                       : \
   ( x == DRX_INTERLEAVEMODE_RESERVED_15 ) ? "Reserved 15"                   : \
   ( x == DRX_INTERLEAVEMODE_I12_J17     ) ? "I12_J17"                       : \
   ( x == DRX_INTERLEAVEMODE_I5_J4       ) ? "I5_J4"                         : \
   ( x == DRX_INTERLEAVEMODE_UNKNOWN     ) ? "Unknown"                       : \
   ( x == DRX_INTERLEAVEMODE_AUTO        ) ? "Auto"                          : \
                                             "(Invalid)" )

#define DRX_STR_LDPC(x) ( \
   ( x == DRX_LDPC_1                     ) ? "LDPC=0.4"                      : \
   ( x == DRX_LDPC_2                     ) ? "LDPC=0.6"                      : \
   ( x == DRX_LDPC_3                     ) ? "LDPC=0.8"                      : \
   ( x == DRX_LDPC_AUTO                  ) ? "Auto"                          : \
   ( x == DRX_LDPC_UNKNOWN               ) ? "Unknown"                       : \
                                             "(Invalid)" )

#define DRX_STR_INTERLEAVER(x) ( \
   ( x == DRX_INTERLEAVER_240            ) ? "Interleaver:B=52,M=240"        : \
   ( x == DRX_INTERLEAVER_720            ) ? "Interleaver:B=52,M=720"        : \
   ( x == DRX_INTERLEAVER_AUTO           ) ? "Auto"                          : \
   ( x == DRX_INTERLEAVER_UNKNOWN        ) ? "Unknown"                       : \
                                             "(Invalid)" )

#define DRX_STR_CARRIER(x) ( \
   ( x == DRX_CARRIER_MULTI              ) ? "Multi carrier mode"            : \
   ( x == DRX_CARRIER_SINGLE             ) ? "Single carrier mode"           : \
   ( x == DRX_CARRIER_AUTO               ) ? "Auto"                          : \
   ( x == DRX_CARRIER_UNKNOWN            ) ? "Unknown"                       : \
                                             "(Invalid)" )

#define DRX_STR_FRAMEMODE(x) ( \
   ( x == DRX_FRAMEMODE_1          ) ? "Frame 420 w/ variable PN init phase" : \
   ( x == DRX_FRAMEMODE_2          ) ? "Frame 595"                           : \
   ( x == DRX_FRAMEMODE_3          ) ? "Frame 945 w/ variable PN init phase" : \
   ( x == DRX_FRAMEMODE_4          ) ? "Frame 420 w/ fixed PN init phase"    : \
   ( x == DRX_FRAMEMODE_5          ) ? "Frame 945 w/ fixed PN init phase"    : \
   ( x == DRX_FRAMEMODE_AUTO       ) ? "Auto"                                : \
   ( x == DRX_FRAMEMODE_UNKNOWN    ) ? "Unknown"                             : \
                                       "(Invalid)" )

#define DRX_STR_TPS_FRAME(x)  ( \
   ( x == DRX_TPS_FRAME1                 ) ? "Frame1"                        : \
   ( x == DRX_TPS_FRAME2                 ) ? "Frame2"                        : \
   ( x == DRX_TPS_FRAME3                 ) ? "Frame3"                        : \
   ( x == DRX_TPS_FRAME4                 ) ? "Frame4"                        : \
   ( x == DRX_TPS_FRAME_UNKNOWN          ) ? "Unknown"                       : \
                                             "(Invalid)" )

/* lock status */

#define DRX_STR_LOCKSTATUS(x) ( \
   ( x == DRX_NEVER_LOCK                 ) ? "Never"                         : \
   ( x == DRX_NOT_LOCKED                 ) ? "No"                            : \
   ( x == DRX_LOCKED                     ) ? "Locked"                        : \
   ( x == DRX_LOCK_STATE_1               ) ? "Lock state 1"                  : \
   ( x == DRX_LOCK_STATE_2               ) ? "Lock state 2"                  : \
   ( x == DRX_LOCK_STATE_3               ) ? "Lock state 3"                  : \
   ( x == DRX_LOCK_STATE_4               ) ? "Lock state 4"                  : \
   ( x == DRX_LOCK_STATE_5               ) ? "Lock state 5"                  : \
   ( x == DRX_LOCK_STATE_6               ) ? "Lock state 6"                  : \
   ( x == DRX_LOCK_STATE_7               ) ? "Lock state 7"                  : \
   ( x == DRX_LOCK_STATE_8               ) ? "Lock state 8"                  : \
   ( x == DRX_LOCK_STATE_9               ) ? "Lock state 9"                  : \
                                             "(Invalid)" )

/* version information , modules */
#define DRX_STR_MODULE(x) ( \
   ( x == DRX_MODULE_DEVICE              ) ? "Device"                        : \
   ( x == DRX_MODULE_MICROCODE           ) ? "Microcode"                     : \
   ( x == DRX_MODULE_DRIVERCORE          ) ? "Core Driver"                   : \
   ( x == DRX_MODULE_DEVICEDRIVER        ) ? "Device Driver"                 : \
   ( x == DRX_MODULE_BSP_I2C             ) ? "BSP I2C"                       : \
   ( x == DRX_MODULE_BSP_TUNER           ) ? "BSP Tuner"                     : \
   ( x == DRX_MODULE_BSP_HOST            ) ? "BSP Host"                      : \
   ( x == DRX_MODULE_DAP                 ) ? "Data Access Protocol"          : \
   ( x == DRX_MODULE_UNKNOWN             ) ? "Unknown"                       : \
                                             "(Invalid)" )

#define DRX_STR_POWER_MODE(x) ( \
   ( x == DRX_POWER_UP                   ) ? "DRX_POWER_UP    "              : \
   ( x == DRX_POWER_MODE_1               ) ? "DRX_POWER_MODE_1"              : \
   ( x == DRX_POWER_MODE_2               ) ? "DRX_POWER_MODE_2"              : \
   ( x == DRX_POWER_MODE_3               ) ? "DRX_POWER_MODE_3"              : \
   ( x == DRX_POWER_MODE_4               ) ? "DRX_POWER_MODE_4"              : \
   ( x == DRX_POWER_MODE_5               ) ? "DRX_POWER_MODE_5"              : \
   ( x == DRX_POWER_MODE_6               ) ? "DRX_POWER_MODE_6"              : \
   ( x == DRX_POWER_MODE_7               ) ? "DRX_POWER_MODE_7"              : \
   ( x == DRX_POWER_MODE_8               ) ? "DRX_POWER_MODE_8"              : \
   ( x == DRX_POWER_MODE_9               ) ? "DRX_POWER_MODE_9"              : \
   ( x == DRX_POWER_MODE_10              ) ? "DRX_POWER_MODE_10"             : \
   ( x == DRX_POWER_MODE_11              ) ? "DRX_POWER_MODE_11"             : \
   ( x == DRX_POWER_MODE_12              ) ? "DRX_POWER_MODE_12"             : \
   ( x == DRX_POWER_MODE_13              ) ? "DRX_POWER_MODE_13"             : \
   ( x == DRX_POWER_MODE_14              ) ? "DRX_POWER_MODE_14"             : \
   ( x == DRX_POWER_MODE_15              ) ? "DRX_POWER_MODE_15"             : \
   ( x == DRX_POWER_MODE_16              ) ? "DRX_POWER_MODE_16"             : \
   ( x == DRX_POWER_DOWN                 ) ? "DRX_POWER_DOWN  "              : \
                                             "(Invalid)" )

#define DRX_STR_OOB_STANDARD(x) ( \
   ( x == DRX_OOB_MODE_A                 ) ? "ANSI 55-1  " : \
   ( x == DRX_OOB_MODE_B_GRADE_A         ) ? "ANSI 55-2 A" : \
   ( x == DRX_OOB_MODE_B_GRADE_B         ) ? "ANSI 55-2 B" : \
                                             "(Invalid)" )

#define DRX_STR_AUD_STANDARD(x) ( \
   ( x == DRX_AUD_STANDARD_BTSC          ) ? "BTSC"                          : \
   ( x == DRX_AUD_STANDARD_A2            ) ? "A2"                            : \
   ( x == DRX_AUD_STANDARD_EIAJ          ) ? "EIAJ"                          : \
   ( x == DRX_AUD_STANDARD_FM_STEREO     ) ? "FM Stereo"                     : \
   ( x == DRX_AUD_STANDARD_AUTO          ) ? "Auto"                          : \
   ( x == DRX_AUD_STANDARD_M_MONO        ) ? "M-Standard Mono"               : \
   ( x == DRX_AUD_STANDARD_D_K_MONO      ) ? "D/K Mono FM"                   : \
   ( x == DRX_AUD_STANDARD_BG_FM         ) ? "B/G-Dual Carrier FM (A2)"      : \
   ( x == DRX_AUD_STANDARD_D_K1          ) ? "D/K1-Dual Carrier FM"          : \
   ( x == DRX_AUD_STANDARD_D_K2          ) ? "D/K2-Dual Carrier FM"          : \
   ( x == DRX_AUD_STANDARD_D_K3          ) ? "D/K3-Dual Carrier FM"          : \
   ( x == DRX_AUD_STANDARD_BG_NICAM_FM   ) ? "B/G-NICAM-FM"                  : \
   ( x == DRX_AUD_STANDARD_L_NICAM_AM    ) ? "L-NICAM-AM"                    : \
   ( x == DRX_AUD_STANDARD_I_NICAM_FM    ) ? "I-NICAM-FM"                    : \
   ( x == DRX_AUD_STANDARD_D_K_NICAM_FM  ) ? "D/K-NICAM-FM"                  : \
   ( x == DRX_AUD_STANDARD_UNKNOWN       ) ? "Unknown"                       : \
                                             "(Invalid)"  )

#define DRX_STR_AUD_STEREO(x) ( \
   ( x == TRUE                           ) ? "Stereo"                        : \
   ( x == FALSE                          ) ? "Mono"                          : \
                                             "(Invalid)"  )

#define DRX_STR_AUD_SAP(x) ( \
   ( x == TRUE                           ) ? "Present"                       : \
   ( x == FALSE                          ) ? "Not present"                   : \
                                             "(Invalid)"  )

#define DRX_STR_AUD_CARRIER(x) ( \
   ( x == TRUE                           ) ? "Present"                       : \
   ( x == FALSE                          ) ? "Not present"                   : \
                                             "(Invalid)"  )

#define DRX_STR_AUD_RDS(x) ( \
   ( x == TRUE                           ) ? "Available"                     : \
   ( x == FALSE                          ) ? "Not Available"                 : \
                                             "(Invalid)"  )

#define DRX_STR_AUD_NICAM_STATUS(x) ( \
   ( x == DRX_AUD_NICAM_DETECTED         ) ? "Detected"                      : \
   ( x == DRX_AUD_NICAM_NOT_DETECTED     ) ? "Not detected"                  : \
   ( x == DRX_AUD_NICAM_BAD              ) ? "Bad"                           : \
                                             "(Invalid)"  )

#define DRX_STR_RDS_VALID(x) ( \
   ( x == TRUE                           ) ? "Valid"                         : \
   ( x == FALSE                          ) ? "Not Valid"                     : \
                                             "(Invalid)"  )

struct drxk3913_params {
	u32_t	freq;					/* Frequency	*/
	u32_t	srate;					/* Symbol rate	*/
};

struct drxk3913_config {	
	/* the demodulator's i2c address */
	u8_t		demod_address;
	/* minimum delay before retuning */
	u32_t	min_delay_ms;
	enum MTTune_atv_standard		standard;
	//point to tuner
	unsigned int(*set_tuner)(struct dvb_frontend *fe, UData_t f_in, UData_t bw_in, enum MTTune_atv_standard tv_type) ;
	unsigned int(*tuner_getlocked)(struct dvb_frontend *fe);
	unsigned int(*tuner_MT2063_Open)(struct dvb_frontend *fe);
	unsigned int(*tuner_MT2063_SoftwareShutdown)(struct dvb_frontend * fe);
	unsigned int(*tuner_MT2063_ClearPowerMaskBits)(struct dvb_frontend *fe);
};

struct drxk3913_state {
	struct i2c_adapter		*i2c;
	struct drxk3913_config		*config;
	struct dvb_frontend		frontend;

	unsigned int				*verbose;	/* Cached module verbosity level	*/

	u32_t		rx_freq;	/* DiSEqC 2.0 receiver freq		*/
	struct DRXDemodInstance_t	demod;
	struct mutex			search_lock;
};

/*-------------------------------------------------------------------------
Exported FUNCTIONS
-------------------------------------------------------------------------*/

enum DRXStatus_t DRX_Init (  pDRXDemodInstance_t  demods[] );

enum DRXStatus_t DRX_Term (  struct dvb_frontend *fe	);

enum DRXStatus_t DRX_Open (  pDRXDemodInstance_t  demod    );

enum DRXStatus_t DRX_Close(  pDRXDemodInstance_t  demod    );

enum DRXStatus_t DRX_Ctrl (  pDRXDemodInstance_t  demod,
                        enum DRXCtrlIndex_t       ctrl,
                        void                *ctrlData );

enum DRXStatus_t MicronasInit(struct dvb_frontend *fe);
enum DRXStatus_t MicronasSetTune(struct dvb_frontend *fe,
								enum DRXStandard_t     standard,
								struct DRXChannel_t      channel);
bool MicronasCheckLock(pDRXDemodInstance_t demod, enum Bool_t bDelay);
enum DRXStatus_t MicronasClose(struct dvb_frontend *fe);


#if defined(CONFIG_DVB_DRXK3913) || (defined(CONFIG_DVB_DRXK3913_MODULE) && defined(MODULE))

extern struct dvb_frontend *drxk3913_attach(struct drxk3913_config *config,
					   struct i2c_adapter *i2c);

#else

static inline struct dvb_frontend *drxk3913_attach(struct drxk3913_config *config,
						  struct i2c_adapter *i2c)
{
	printk(KERN_WARNING "%s: Driver disabled by Kconfig\n", __func__);
	return NULL;
}
#endif

/*-------------------------------------------------------------------------
THE END
-------------------------------------------------------------------------*/
#endif /* __DRXDRIVER_H__ */
