#ifndef __DRXK_TYPES_H__
#define __DRXK_TYPES_H__

#define VERSION_FIXED 0
#if     VERSION_FIXED
#define VERSION_MAJOR 0
#define VERSION_MINOR 0
#define VERSION_PATCH 0
#else
#define VERSION__A      0x0
#define   VERSION_MAJOR 0       
#define   VERSION_MINOR 9       
#define   VERSION_PATCH 4300  
#endif


/**
* \typedef unsigned char u8_t
* \brief type definition of an unsigned 8 bits integer
*/
typedef unsigned char  u8_t;
/**
* \typedef char s8_t
* \brief type definition of a signed 8 bits integer
*/
typedef char           s8_t;
/**
* \typedef unsigned short u16_t *pu16_t
* \brief type definition of an unsigned 16 bits integer
*/
typedef unsigned short u16_t;
/**
* \typedef short s16_t
* \brief type definition of a signed 16 bits integer
*/
typedef short          s16_t;
/**
* \typedef unsigned long u32_t
* \brief type definition of an unsigned 32 bits integer
*/
typedef unsigned long  u32_t;
/**
* \typedef long s32_t
* \brief type definition of a signed 32 bits integer
*/
typedef long           s32_t;
/*
* \typedef struct ... u64_t
* \brief type definition of an usigned 64 bits integer
*/
struct u64_t{
    u32_t MSLW;
    u32_t LSLW;
};
/*
* \typedef struct ... i64_t
* \brief type definition of a signed 64 bits integer
*/
struct s64_t{
    s32_t MSLW;
    u32_t LSLW;
};

/**
* \typedef u8_t *pu8_t
* \brief type definition of pointer to an unsigned 8 bits integer
*/
typedef u8_t         *pu8_t;
/**
* \typedef s8_t *ps8_t
* \brief type definition of pointer to a signed 8 bits integer
*/
typedef s8_t         *ps8_t;
/**
* \typedef u16_t *pu16_t
* \brief type definition of pointer to an unsigned 16 bits integer
*/
typedef u16_t        *pu16_t;
/**
* \typedef s16_t *ps16_t
* \brief type definition of pointer to a signed 16 bits integer
*/
typedef s16_t        *ps16_t;
/**
* \typedef u32_t *pu32_t
* \brief type definition of pointer to an unsigned 32 bits integer
*/
typedef u32_t        *pu32_t;
/**
* \typedef s32_t *ps32_t
* \brief type definition of pointer to a signed 32 bits integer
*/
typedef s32_t        *ps32_t;
/**
* \typedef u64_t *pu64_t
* \brief type definition of pointer to an usigned 64 bits integer
*/
typedef struct u64_t        *pu64_t;
/**
* \typedef s64_t *ps64_t
* \brief type definition of pointer to a signed 64 bits integer
*/
typedef struct s64_t        *ps64_t;


/**
* \typedef s32_t DRXFrequency_t
* \brief type definition of frequency
*/
typedef s32_t DRXFrequency_t;

/**
* \typedef u32_t pDRXFrequency_t
* \brief type definition of a pointer to a frequency
*/
typedef DRXFrequency_t *pDRXFrequency_t;

/*-------------------------------------------------------------------------
DEFINES
-------------------------------------------------------------------------*/
/**
* \def NULL
* \brief Define NULL for target.
*/
#ifndef NULL
#define NULL            (0)
#endif

/*-------------------------------------------------------------------------
ENUM
-------------------------------------------------------------------------*/

/*
 * Boolean datatype. Only define if not already defined TRUE or FALSE.
 */
#if defined (TRUE) || defined (FALSE)
typedef int Bool_t;
#else
/**
* \enum Bool_t
* \brief Boolean type
*/
enum Bool_t{
  FALSE = 0,
  TRUE
};
#endif

typedef enum Bool_t  *pBool_t;

/**
* \enum DRXStatus_t
* \brief Various return statusses
*/
enum DRXStatus_t{
  DRX_STS_READY = 3,         /**< indicating that device/service is ready */
  DRX_STS_BUSY = 2,          /**< indicating that device/service is busy */
  DRX_STS_OK = 1,            /**< indicating that everything is OK */
  DRX_STS_INVALID_ARG = -1,  /**< indicating invalid arguments */
  DRX_STS_ERROR = -2,         /**< indicating general error */
  DRX_STS_FUNC_NOT_AVAILABLE = -3 /**< indicating unavaiable functionality */
};

// change this value to determine 
// whether the MT2063 controlled by micronas driver...
#define TUNER_CONTROL_BY_DRXK_DRIVER 0

#undef	drxk_info
#define drxk_info(format, arg...) printk(KERN_INFO    "drx3913: " format "\n" , ## arg)

#endif	/* __DRXK_TYPES_H__ */
